@@ -52,12 +52,12 @@ my $default;
 # XXX this synchronised...
 my %bitfields = (
     'calendar_component_set' => 'VEVENT VTODO VJOURNAL VFREEBUSY VAVAILABILITY VPOLL',
-    'event_extra_params' => 'bodyStructure clientAddress diskUsed flagNames messageContent messageSize messages modseq service timestamp uidnext vnd.cmu.midset vnd.cmu.unseenMessages vnd.cmu.envelope vnd.cmu.sessionId vnd.cmu.mailboxACL vnd.cmu.mbtype vnd.cmu.davFilename vnd.cmu.davUid vnd.fastmail.clientId vnd.fastmail.sessionId vnd.fastmail.convExists vnd.fastmail.convUnseen vnd.fastmail.cid vnd.fastmail.counters vnd.fastmail.jmapEmail vnd.fastmail.jmapStates vnd.cmu.emailid vnd.cmu.threadid',
+    'event_extra_params' => 'bodyStructure clientAddress diskUsed flagNames messageContent messageSize messages modseq service timestamp uidnext vnd.cmu.midset vnd.cmu.unseenMessages vnd.cmu.envelope vnd.cmu.sessionId vnd.cmu.mailboxACL vnd.cmu.mbtype vnd.cmu.davFilename vnd.cmu.davUid vnd.fastmail.clientId vnd.fastmail.sessionId vnd.fastmail.convExists vnd.fastmail.convUnseen vnd.fastmail.cid vnd.fastmail.counters vnd.fastmail.jmapEmail vnd.fastmail.jmapStates vnd.cmu.emailid vnd.cmu.threadid vnd.cmu.visibleUsers',
     'event_groups' => 'message quota flags access mailbox subscription calendar applepushservice jmap',
     'httpmodules' => 'admin caldav carddav cgi domainkey freebusy ischedule jmap prometheus rss tzdist webdav',
     'metapartition_files' => 'header index cache expunge squat annotations lock dav archivecache',
     'newsaddheaders' => 'to replyto',
-    'sieve_extensions' => 'fileinto reject vacation vacation-seconds notify include envelope environment body relational regex subaddress copy date index imap4flags imapflags mailbox mboxmetadata servermetadata variables editheader extlists duplicate ihave fcc special-use redirect-dsn redirect-deliverby mailboxid vnd.cyrus.log x-cyrus-log vnd.cyrus.jmapquery x-cyrus-jmapquery snooze vnd.cyrus.snooze x-cyrus-snooze vnd.cyrus.imip',
+    'sieve_extensions' => 'fileinto reject vacation vacation-seconds notify include envelope environment body relational regex subaddress copy date index imap4flags imapflags mailbox mboxmetadata servermetadata variables editheader extlists duplicate ihave fcc special-use redirect-dsn redirect-deliverby mailboxid vnd.cyrus.log x-cyrus-log vnd.cyrus.jmapquery x-cyrus-jmapquery vnd.cyrus.imip snooze vnd.cyrus.snooze x-cyrus-snooze vnd.cyrus.implicit_keep_target',
 );
 my $bitfields_fixed = 0;
 
@@ -104,6 +104,7 @@ sub default
             syslog_prefix => '@name@',
             sievedir => '@basedir@/conf/sieve',
             master_pid_file => '@basedir@/run/master.pid',
+            master_ready_file => '@basedir@/master.ready',
             defaultpartition => 'default',
             defaultdomain => 'defdomain',
             'partition-default' => '@basedir@/data',
@@ -111,8 +112,6 @@ sub default
             allowplaintext => 'yes',
             # for debugging - see cassandane.ini.example
             debug_command => '@prefix@/utils/gdbtramp %s %d',
-            # everyone should be running this
-            improved_mboxlist_sort => 'yes',
             # default changed, we want to be explicit about it
             unixhierarchysep => 'no',
             # let's hear all about it
@@ -120,6 +119,17 @@ sub default
             chatty => 'yes',
             debug => 'yes',
             httpprettytelemetry => 'yes',
+
+            # smtpclient_open should fail by default!
+            #
+            # If your test fails and writes something like
+            #     smptclient_open: can't connect to host: bogus:0/noauth
+            # in syslog, then Cyrus is calling smtpclient_open(), and you
+            # will need to arrange for fakesmtpd to be listening.  To do
+            # this add :want_smtpdaemon to the test attributes, or enable
+            # smtpdaemon in the suite constructor.
+            smtp_backend => 'host',
+            smtp_host => 'bogus:0',
         );
         my $defs = Cassandane::Cassini->instance()->get_section('config');
         $default->set(%$defs);
@@ -140,7 +150,7 @@ sub clone
 sub _explode_bit_string
 {
     my ($s) = @_;
-    return split /[_ ]/, $s;
+    return split / /, $s;
 }
 
 sub set
@@ -174,8 +184,8 @@ sub set_bits
 
     die "$name is not a bitfield option" if not exists $bitfields{$name};
 
-    # explode space- or underscore-delimited list as only bit
-    if (scalar @bits == 1 && $bits[0] =~ m/[_ ]/) {
+    # explode space-delimited list as only bit
+    if (scalar @bits == 1 && $bits[0] =~ m/ /) {
         @bits = _explode_bit_string($bits[0]);
     }
 
@@ -193,8 +203,8 @@ sub clear_bits
 
     die "$name is not a bitfield option" if not exists $bitfields{$name};
 
-    # explode space- or underscore-delimited list as only bit
-    if (scalar @bits == 1 && $bits[0] =~ m/[_ ]/) {
+    # explode space-delimited list as only bit
+    if (scalar @bits == 1 && $bits[0] =~ m/ /) {
         @bits = _explode_bit_string($bits[0]);
     }
 
