@@ -83,7 +83,10 @@ sub new
                      conversations_counted_flags => "\\Draft \\Flagged \$IsMailingList \$IsNotification \$HasAttachment",
                      ctl_conversationsdb_conversations_max_thread => 5);
 
-        return $class->SUPER::new({ config => $config }, @args);
+        return $class->SUPER::new({
+            config => $config,
+            adminstore => 1,
+        }, @args);
     }
 }
 
@@ -422,6 +425,46 @@ sub test_reconstruct_splitconv
     $talk->select("INBOX");
 }
 
+#
+# test clearing the modseq
+#
+sub test_clearmodseq
+    :min_version_3_1
+{
+    my ($self) = @_;
+
+    my $talk = $self->{store}->get_client();
+
+    # check IMAP server has the XCONVERSATIONS capability
+    $self->assert($talk->capability()->{xconversations});
+
+    my $admintalk = $self->{adminstore}->get_client();
+    $admintalk->setquota('user.cassandane', ['STORAGE', 500000]);
+
+    for (1..20) {
+      $self->make_message("Message A$_");
+    }
+
+    my $precounters = $self->{store}->get_counters();
+    # we've made a bunch of changes, this should be more than 10!
+    $self->assert($precounters->{highestmodseq} > 10, "$precounters->{highestmodseq} > 10");
+
+    # zero out the modseqs
+    $self->{instance}->run_command({ cyrus => 1 }, 'ctl_conversationsdb', '-M', 'cassandane');
+
+    my $midcounters = $self->{store}->get_counters();
+    # we haven't made any changes, should actually be zero!
+    $self->assert($midcounters->{highestmodseq} < 10, "$midcounters->{highestmodseq} < 10");
+
+    $self->make_message("Message B1");
+    $self->make_message("Message B2");
+
+    my $postcounters = $self->{store}->get_counters();
+    # we've only created two more emails, it shouldn't have bounced higher
+    $self->assert($postcounters->{highestmodseq} < 10, "$postcounters->{highestmodseq} < 10");
+
+}
+
 #
 # Test APPEND of messages to IMAP
 #
@@ -1108,4 +1151,68 @@ sub test_rename_between_users
     $self->assert_str_equals('-', $mfolders->[1]);
 }
 
+#
+# Test user rename without splitting conversations
+#
+sub test_rename_user_nosplitconv
+    :AllowMoves :Replication :needs_component_replication
+{
+    my ($self) = @_;
+
+    xlog $self, "Test user rename without splitting conversations";
+
+    my %exp;
+
+    # check IMAP server has the XCONVERSATIONS capability
+    my $master_store = $self->{master_store};
+    $self->assert($master_store->get_client()->capability()->{xconversations});
+
+    $master_store->set_fetch_attributes('uid', 'cid', 'basecid');
+
+    xlog $self, "generating message A";
+    $exp{A} = $self->make_message("Message A", store => $master_store);
+    $exp{A}->set_attributes(uid => 1, cid => $exp{A}->make_cid());
+    $self->check_messages(\%exp);
+
+    xlog $self, "generating replies";
+    for (1..20) {
+        $exp{"A$_"} = $self->make_message("Re: Message A",
+                                          references => [ $exp{A} ],
+                                          store => $master_store);
+        $exp{"A$_"}->set_attributes(uid => 1+$_, cid => $exp{A}->make_cid());
+    }
+
+    my $talk = $master_store->get_client();
+    $talk->create('foo');
+    $talk->copy('1:*', 'foo');
+
+    $self->check_messages(\%exp, keyed_on => 'uid', store => $master_store);
+    $self->check_conversations();
+
+    $self->run_replication();
+    $self->check_replication('cassandane');
+
+    # Reduce the conversation thread size
+    my $config = $self->{instance}->{config};
+    $config->set(conversations_max_thread => 5);
+    $config->generate($self->{instance}->_imapd_conf());
+
+    $config = $self->{replica}->{config};
+    $config->set(conversations_max_thread => 5);
+    $config->generate($self->{replica}->_imapd_conf());
+
+    # Rename the user
+    my $admintalk = $self->{adminstore}->get_client();
+    my $res = $admintalk->rename('user.cassandane', 'user.newuser');
+    $self->assert(not $admintalk->get_last_error());
+
+    $self->{adminstore}->set_folder("user.newuser");
+    $self->{adminstore}->set_fetch_attributes('uid', 'cid', 'basecid');
+    $self->check_messages(\%exp, keyed_on => 'uid', store => $self->{adminstore});
+    $self->check_conversations();
+
+    $self->run_replication(user => 'newuser');
+    $self->check_replication('newuser');
+}
+
 1;
