@@ -98,27 +98,32 @@ sub new
     $config->set(caldav_historical_age => -1);
     $config->set(virtdomains => 'no');
 
-    return $class->SUPER::new({
+    my $self = $class->SUPER::new({
         config => $config,
         jmap => 1,
         adminstore => 1,
         deliver => 1,
         services => [ 'imap', 'sieve', 'http' ],
     }, @args);
+
+    $self->needs('component', 'jmap');
+    return $self;
 }
 
 sub set_up
 {
     my ($self) = @_;
     $self->SUPER::set_up();
-    $self->{jmap}->DefaultUsing([
-        'urn:ietf:params:jmap:core',
-        'urn:ietf:params:jmap:calendars',
-        'urn:ietf:params:jmap:principals',
-        'urn:ietf:params:jmap:calendars:preferences',
-        'https://cyrusimap.org/ns/jmap/calendars',
-        'https://cyrusimap.org/ns/jmap/debug',
-    ]);
+    if ($self->{_want}->{start_instances}) {
+        $self->{jmap}->DefaultUsing([
+            'urn:ietf:params:jmap:core',
+            'urn:ietf:params:jmap:calendars',
+            'urn:ietf:params:jmap:principals',
+            'urn:ietf:params:jmap:calendars:preferences',
+            'https://cyrusimap.org/ns/jmap/calendars',
+            'https://cyrusimap.org/ns/jmap/debug',
+        ]);
+    }
 }
 
 sub encode_eventid
