@@ -54,6 +54,7 @@ use Cassandane::Util::Metronome;
 use Cassandane::Instance;
 use Cassandane::Service;
 use Cassandane::Config;
+use Time::HiRes qw(sleep);
 
 my $lemming_bin = getcwd() . '/utils/lemming';
 
@@ -158,6 +159,8 @@ sub lemming_cull
     return unless defined $self->{instance};
     my $coresdir = $self->{instance}->{basedir} . '/conf/cores';
 
+    my $culled;
+
     return unless -d $coresdir;
     opendir LEMM,$coresdir
         or die "cannot open $coresdir for reading: $!";
@@ -166,10 +169,14 @@ sub lemming_cull
         my ($tag, $pid) = m/^lemming\.(\w+).(\d+)$/;
         next
             unless defined $pid;
-        xlog $self, "culled lemming tag=$tag pid=$pid"
-            if kill(9, $pid);
+        if (kill(9, $pid)) {
+            xlog $self, "culled lemming tag=$tag pid=$pid";
+            $culled++;
+        }
     }
     closedir LEMM;
+
+    return $culled;
 }
 
 sub _lemming_args
@@ -192,6 +199,12 @@ sub lemming_service
     return $self->{instance}->add_service(_lemming_args(%params));
 }
 
+sub lemming_daemon
+{
+    my ($self, %params) = @_;
+    return $self->{instance}->add_daemon(_lemming_args(%params));
+}
+
 sub lemming_start
 {
     my ($self, %params) = @_;
@@ -531,6 +544,13 @@ sub test_service_exit_during_start
     $self->assert_null($lemm);
     $self->assert_deep_equals({ A => { live => 0, dead => 5 } },
                               $self->lemming_census());
+
+    if ($self->{instance}->{have_syslog_replacement}) {
+        xlog $self, "check that the error was syslogged";
+        my @lines = $self->{instance}->getsyslog(qr/too many failures for service/);
+        $self->assert_num_equals(1, scalar @lines);
+        $self->assert_matches(qr/disabling until next SIGHUP/, $lines[0]);
+    }
 }
 
 sub test_startup
@@ -1392,13 +1412,13 @@ sub test_pidfile_lock_symlink
     rmdir $tmp;
 }
 
-sub test_ready_file_new
+sub test_ready_file
 {
     my ($self) = @_;
 
     # config isn't fully initialised until start() is called, so we can't
     # just read these the sensible way. instead, predict the defaults.
-    my $ready_file = $self->{instance}->get_basedir() . '/conf/master.ready';
+    my $ready_file = $self->{instance}->get_basedir() . '/master.ready';
     my $pid_file = $self->{instance}->get_basedir() . '/run/master.pid';
 
     # pid file should not already exist
@@ -1417,60 +1437,95 @@ sub test_ready_file_new
     $self->assert_not_null($pid_sb);
 
     # ready file should exist soon...
-    timed_wait(sub { $ready_sb = stat($ready_file) },
-               description => "$ready_file to exist");
+    $ready_sb = stat($ready_file);
     $self->assert_not_null($ready_sb);
 
     # ready file should be newer than pid file
     $self->assert_num_gte($pid_sb->mtime, $ready_sb->mtime);
 }
 
-sub test_ready_file_exists
+sub test_daemon_exits
 {
     my ($self) = @_;
 
-    # config isn't fully initialised until start() is called, so we can't
-    # just read these the sensible way. instead, predict the defaults.
-    my $pid_file = $self->{instance}->get_basedir() . '/run/master.pid';
+    my $lemming_delay = 500; #ms
+    my $test_delay = 10; #s
 
-    # cannot be under basedir because it'll be blown away at startup
-    my $ready_file = "/tmp/cassandane-$$-master.ready";
-    $self->{instance}->{config}->set('master_ready_file', $ready_file);
+    xlog $self, "Test a program in the DAEMON section which fails";
+    $self->lemming_daemon(tag => 'A', delay => $lemming_delay, mode => 'exit');
+    # This service won't be used
+    my $srv = $self->lemming_service(tag => 'C');
 
-    system("touch", $ready_file) == 0 or die "touch $ready_file: $?";
-    sleep 3;
+    $self->{instance}->start();
 
-    # pid file should not already exist
-    my $pid_sb = stat($pid_file);
-    $self->assert_null($pid_sb);
+    # make sure it runs long enough for the child janitor to wake up
+    sleep $test_delay;
 
-    # ready file should already exist
-    my $ready_sb = stat($ready_file);
-    $self->assert_not_null($ready_sb);
-    my $orig_mtime = $ready_sb->mtime;
+    # master had better still be running!
+    $self->assert($self->{instance}->is_running(),
+                  "master is no longer running");
 
-    # start cyrus
+    # make sure those lemmings did actually die
+    my $census = $self->lemming_census();
+    # XXX upper limit on this should be MAX_READY_FAILS if we run for less
+    # XXX than MAX_READY_FAIL_INTERVAL seconds, but the delayed restart of
+    # XXX failed daemons doesn't seem to be actually delayed???
+    $self->assert_num_gte(1, $census->{A}->{dead});
+
+    # there'll be syslog errors as long as this is running, so stop it early
+    $self->{instance}->stop(no_check_syslog => 1);
+    # then consume and check them
+    $self->assert_syslog_matches($self->{instance},
+                                 qr{too many failures for});
+}
+
+sub test_babysit
+    :NoCheckSyslog
+{
+    my ($self) = @_;
+
+    my $srv = $self->lemming_service(prefork => 1, babysit => 1);
     $self->start();
+    $self->lemming_wait(A => { live => 1 });
 
-    # pid file should exist now
-    $pid_sb = stat($pid_file);
-    $self->assert_not_null($pid_sb);
+    xlog $self, "preforked, so one lemming running already";
+    $self->assert_deep_equals({ A => { live => 1, dead => 0 } },
+                              $self->lemming_census());
 
-    # ready file should be touched soon
-    timed_wait(sub {
-                    $ready_sb = stat($ready_file);
-                    return 1 if $ready_sb->mtime >= $pid_sb->mtime;
-                    return undef;
-               },
-               description => "$ready_file to be newer than $pid_file");
-    $self->assert_not_null($ready_sb);
+    # Quickly kill 5 children to trigger the backoff
+    my $killed = 0;
 
-    # ready file should be newer than pid file
-    $self->assert_num_gte($pid_sb->mtime, $ready_sb->mtime);
-    $self->assert_num_gt($orig_mtime, $ready_sb->mtime);
+    for (1..20) {
+      $killed++ if $self->lemming_cull();
 
-    # don't pollute /tmp
-    unlink $ready_file;
+      last if $killed >= 5;
+
+      sleep(.5);
+    };
+
+    # make sure it said it's going to wait
+    my @lines = $self->{instance}->getsyslog(qr/ERROR: too many failures .*disabling/);
+    $self->assert_num_equals(1, scalar @lines);
+
+    # master should not have restarted after 5 deaths
+    $self->assert_deep_equals({ A => { live => 0, dead => 5 } },
+                              $self->lemming_census());
+
+    # wait 5 seconds, we still shouldn't have one
+    sleep 5;
+
+    $self->assert_deep_equals({ A => { live => 0, dead => 5 } },
+                              $self->lemming_census());
+
+    # wait up to 30 seconds, there should be one
+    for (1..30) {
+      last if $self->lemming_census->{A}{live};
+
+      sleep 1;
+    }
+
+    $self->assert_deep_equals({ A => { live => 1, dead => 5 } },
+                              $self->lemming_census());
 }
 
 1;
