@@ -72,11 +72,16 @@ sub new
         vcard_max_size => 100000,
     );
 
-    return $class->SUPER::new({
+    my $self = $class->SUPER::new({
         config => $config,
         jmap => 1,
         services => ['imap', 'http', 'sieve'],
+        smtpdaemon => 1,
     }, @_);
+
+    $self->needs('component', 'jmap');
+    $self->needs('component', 'sieve');
+    return $self;
 }
 
 sub set_up
@@ -93,16 +98,6 @@ sub tear_down
     $self->SUPER::tear_down();
 }
 
-sub skip_check
-{
-    my ($self) = @_;
-
-    # XXX skip all tests from this suite in verbose mode for now -- see
-    # XXX detailed comment on put_submission below
-    my $reason = "test would hang in verbose mode";
-    return get_verbose() ? $reason : undef;
-}
-
 sub _random_vevent
 {
     my ($self) = @_;
@@ -246,17 +241,6 @@ sub _submissions_mailbox
     return $res->[0][1]{created}{"m$counter"}{id};
 }
 
-# XXX When creating JMAP EmailSubmissions objects, the http service
-# XXX makes a few SMTP connections to verify that from/to addresses
-# XXX are acceptable, that sort of thing.
-# XXX When Cassandane is run in verbose mode, some weird bug in the
-# XXX smtp listener results in it only handling a few connections
-# XXX before stopping, and then the test stalls while http waits for
-# XXX an SMTP connection that will never succeed.
-# XXX Until we fix the issue in the SMTP thing, these tests won't run
-# XXX correctly in verbose mode, sorry.  This affects tests in the
-# XXX JMAPEmailSubmission suite too, though it affects this one worse
-# XXX because it wants to create a lot more EmailSubmissions objects.
 sub put_submission
 {
     my ($self) = @_;
@@ -345,13 +329,11 @@ sub put_email
 
     $counter ++;
 
-    $self->make_message(subject => "message $counter");
+    $self->make_message("message $counter");
 }
 
 sub test_maxmsg_addressbook_limited
-    :needs_component_sieve :needs_component_jmap
-    :JMAPExtensions
-    :NoStartInstances
+    :JMAPExtensions :NoStartInstances
 {
     my ($self) = @_;
 
@@ -408,9 +390,7 @@ sub test_maxmsg_addressbook_limited
 }
 
 sub test_maxmsg_calendar_limited
-    :needs_component_sieve :needs_component_jmap
-    :JMAPExtensions
-    :NoStartInstances
+    :JMAPExtensions :NoStartInstances
 {
     my ($self) = @_;
 
@@ -467,9 +447,7 @@ sub test_maxmsg_calendar_limited
 }
 
 sub test_maxmsg_email_limited
-    :needs_component_sieve :needs_component_jmap
-    :JMAPExtensions
-    :NoStartInstances
+    :JMAPExtensions :NoStartInstances
 {
     my ($self) = @_;
 
@@ -491,7 +469,7 @@ sub test_maxmsg_email_limited
     };
     my $e = $@;
     $self->assert_not_null($e);
-    $self->assert_matches(qr{over.*quota}, $e);
+    $self->assert_matches(qr{Over quota}, $e);
 
     # should have syslogged about it too
     $self->assert_syslog_matches($self->{instance},
@@ -525,7 +503,6 @@ sub test_maxmsg_email_limited
 }
 
 sub test_maxmsg_unlimited
-    :needs_component_sieve :needs_component_jmap
     :JMAPExtensions
 {
     my ($self) = @_;
