@@ -40,6 +40,7 @@
 package Cassandane::Cyrus::Simple;
 use strict;
 use warnings;
+use Cwd qw(getcwd realpath);
 use Data::Dumper;
 use DateTime;
 
@@ -214,6 +215,73 @@ sub test_cmdtimer_sessionid
     }
 }
 
+sub test_toggleable_debug_logging
+    :min_version_3_9
+{
+    my ($self) = @_;
+
+    my $config_debug = $self->{instance}->{config}->get_bool('debug', 'no');
+    my $imaptalk = $self->{store}->get_client();
+
+    # can't do anything without captured syslog
+    if (!$self->{instance}->{have_syslog_replacement}) {
+        xlog $self, "can't examine syslog, test is useless";
+        return;
+    }
+
+    # find our imapd pid from syslog
+    my $loginpat = qr{
+        \bimap\[(\d+)\]:\slogin:
+        \s\S+\s\[[\d\.]+\]\scassandane\splaintext
+        \sUser\slogged\sin
+    }x;
+    my @logins = $self->{instance}->getsyslog($loginpat);
+    $self->assert_num_equals(1, scalar @logins);
+    $logins[0] =~ m/$loginpat/;
+    my $imapd_pid = $1;
+
+    for (1..5) {
+        $imaptalk->unselect();
+        my $res = $imaptalk->select('INBOX');
+        $self->assert_str_equals('ok',
+                                 $imaptalk->get_last_completion_response());
+
+        # this is really looking at cassandane's own injected syslog
+        # output, so it depends on the injected syslog doing the right
+        # thing with masking
+        my $selectpat = qr/open: user cassandane opened INBOX/;
+        my @lines = $self->{instance}->getsyslog($selectpat);
+        if ($config_debug) {
+            $self->assert_num_equals(1, scalar @lines);
+            $self->assert_matches(qr/imap\[$imapd_pid\]:/, $lines[0]);
+        }
+        else {
+            $self->assert_num_equals(0, scalar @lines);
+        }
+
+        $config_debug = !$config_debug;
+
+        # toggle debug logging by sending SIGUSR1
+        my $count = kill 'SIGUSR1', $imapd_pid;
+        $self->assert_num_equals(1, $count);
+
+        # we can also look for the message logged by cyrus at the
+        # time it toggles the value
+        my $statuspat = qr/debug logging turned (on|off)/;
+        @lines = $self->{instance}->getsyslog($statuspat);
+        $self->assert_num_equals(1, scalar @lines);
+        $self->assert_matches(qr/imap\[$imapd_pid\]:/, $lines[0]);
+        $lines[0] =~ $statuspat;
+        my $status = $1;
+        if ($config_debug) {
+            $self->assert_str_equals('on', $status);
+        }
+        else {
+            $self->assert_str_equals('off', $status);
+        }
+    }
+}
+
 sub test_append_binary
 {
     my ($self) = @_;
@@ -236,6 +304,94 @@ EOF
     $self->assert_str_equals("test\r\n", $res->{1}{binary});
 }
 
+sub test_fatals_abort_enabled
+    :NoStartInstances
+{
+    my ($self) = @_;
+
+    $self->{instance}->{config}->set(
+        'fatals_abort' => 'yes',
+        'prometheus_enabled' => 'no',
+    );
+    $self->_start_instances();
+
+    my $basedir = $self->{instance}->get_basedir();
+
+    # run `promstatsd -1` without having set up for prometheus, which should
+    # produce a "Prometheus metrics are not being tracked..." fatal error
+    eval {
+        $self->{instance}->run_command({ cyrus => 1 }, 'promstatsd', '-1');
+    };
+    my $e = $@;
+    $self->assert_not_null($e);
+    $self->assert_matches(qr{promstatsd pid \d+\) terminated by signal 6},
+                          $e->{'-text'});
+
+    my @cores = $self->{instance}->find_cores();
+    if (@cores) {
+        # if we dumped core, there'd better only be one core file
+        $self->assert_num_equals(1, scalar @cores);
+
+        # don't barf on it existing during shutdown
+        unlink $cores[0];
+    }
+}
+
+sub test_fatals_abort_disabled
+    :NoStartInstances
+{
+    my ($self) = @_;
+
+    $self->{instance}->{config}->set(
+        'fatals_abort' => 'no',
+        'prometheus_enabled' => 'no',
+    );
+    $self->_start_instances();
+
+    my $basedir = $self->{instance}->get_basedir();
+
+    # run `promstatsd -1` without having set up for prometheus, which should
+    # produce a "Prometheus metrics are not being tracked..." fatal error
+    eval {
+        $self->{instance}->run_command({ cyrus => 1 }, 'promstatsd', '-1');
+    };
+    my $e = $@;
+    $self->assert_not_null($e);
+    $self->assert_matches(qr{promstatsd pid \d+\) exited with code 78},
+                          $e->{'-text'});
+
+    # post-test sanity checks will complain for us if a core was left behind
+}
+
+sub test_fork_noexec
+{
+    my ($self) = @_;
+
+    # need some not-executable file to test with, crash.c will do
+    my $noexec_file = realpath('utils/crash.c');
+    # it had better exist, and not be executable
+    $self->assert_file_test($noexec_file, '-e');
+    $self->assert_not_file_test($noexec_file, '-x');
+
+    my $expect_pid = $$;
+    my $expect_cwd = getcwd();
+
+    # try to run it... the exec in the forked child process will fail
+    eval {
+        $self->{instance}->run_command({ cyrus => 0, }, $noexec_file);
+    };
+    my $e = $@;
+    $self->assert_not_null($e);
+    # the child process had better exit!
+    $self->assert_matches(qr{exited with code 71}, $e->get_message());
+
+    # this test had better still be running in the same process!
+    $self->assert_num_equals($expect_pid, $$);
+
+    # cwd better not have changed!
+    $self->assert_str_equals($expect_cwd, getcwd());
+}
+
 sub test_sasl_ir
 {
     my ($self) = @_;
