@@ -78,7 +78,6 @@ use Cassandane::MasterEvent;
 use Cassandane::MasterDaemon;
 use Cassandane::Cassini;
 use Cassandane::PortManager;
-use Cassandane::Net::SMTPServer;
 use Cassandane::BuildInfo;
 
 use lib '../perl/imap';
@@ -120,6 +119,7 @@ sub new
         _pwcheck => $cassini->val('cassandane', 'pwcheck', 'alwaystrue'),
         install_certificates => 0,
         _pid => $$,
+        smtpdaemon => 0,
     };
 
     $self->{name} = $params{name}
@@ -152,6 +152,8 @@ sub new
         if defined $params{pwcheck};
     $self->{install_certificates} = $params{install_certificates}
         if defined $params{install_certificates};
+    $self->{smtpdaemon} = $params{smtpdaemon}
+        if defined $params{smtpdaemon};
 
     # XXX - get testcase name from caller, to apply even finer
     # configuration from cassini ?
@@ -533,10 +535,10 @@ sub _find_binary
 
     my $base = $self->{cyrus_destdir} . $self->{cyrus_prefix};
 
-    if ($name eq 'delve') {
+    if ($name =~ m/xapian-.*$/) {
         my $lib = `ldd $base/libexec/imapd` || die "can't ldd imapd";
         $lib =~ m{(/\S+)/lib/libxapian-([0-9.]+)\.so};
-        return "$1/bin/xapian-delve-$2";
+        return "$1/bin/$name-$2";
     }
 
     foreach (qw( bin sbin libexec libexec/cyrus-imapd lib cyrus/bin ))
@@ -561,6 +563,31 @@ sub _find_binary
     die "Couldn't locate $name under $base";
 }
 
+sub _valgrind_setup
+{
+    my ($self, $name) = @_;
+
+    my @cmd;
+
+    my $cassini = Cassandane::Cassini->instance();
+
+    my $arguments = '-q --tool=memcheck --leak-check=full --run-libc-freeres=no';
+    my $valgrind_logdir = $self->{basedir} . '/vglogs';
+    my $valgrind_suppressions =
+        abs_path($cassini->val('valgrind', 'suppression', 'vg.supp'));
+    mkpath $valgrind_logdir
+        unless ( -d $valgrind_logdir );
+    push(@cmd,
+        $cassini->val('valgrind', 'binary', '/usr/bin/valgrind'),
+        "--log-file=$valgrind_logdir/$name.%p",
+        "--suppressions=$valgrind_suppressions",
+        "--gen-suppressions=all",
+        split(/\s+/, $cassini->val('valgrind', 'arguments', $arguments))
+    );
+
+    return @cmd;
+}
+
 sub _binary
 {
     my ($self, $name) = @_;
@@ -571,23 +598,11 @@ sub _binary
     my $cassini = Cassandane::Cassini->instance();
 
     if ($cassini->bool_val('valgrind', 'enabled') &&
-        !($name =~ m/delve$/) &&
+        !($name =~ m/xapian.*$/) &&
         !($name =~ m/\.pl$/) &&
         !($name =~ m/^\//))
     {
-        my $arguments = '-q --tool=memcheck --leak-check=full --run-libc-freeres=no';
-        my $valgrind_logdir = $self->{basedir} . '/vglogs';
-        my $valgrind_suppressions =
-            abs_path($cassini->val('valgrind', 'suppression', 'vg.supp'));
-        mkpath $valgrind_logdir
-            unless ( -d $valgrind_logdir );
-        push(@cmd,
-            $cassini->val('valgrind', 'binary', '/usr/bin/valgrind'),
-            "--log-file=$valgrind_logdir/$name.%p",
-            "--suppressions=$valgrind_suppressions",
-            "--gen-suppressions=all",
-            split(/\s+/, $cassini->val('valgrind', 'arguments', $arguments))
-        );
+        push @cmd, $self->_valgrind_setup($name);
         $valground = 1;
     }
 
@@ -732,13 +747,6 @@ sub _generate_imapd_conf
     );
     if ($cyrus_major_version >= 3) {
         $config->set_bits('event_groups', 'mailbox message flags calendar');
-
-        if ($cyrus_major_version > 3 || $cyrus_minor_version >= 1) {
-            $config->set(
-                smtp_backend => 'host',
-                smtp_host => $self->{smtphost},
-            );
-        }
     }
     else {
         $config->set_bits('event_groups', 'mailbox message flags');
@@ -1004,6 +1012,9 @@ sub _start_master
                 },
                 description => $srv->address() . " to be in LISTEN state");
     }
+    timed_wait(sub { -e "$self->{basedir}/master.ready" },
+               description => "master.ready file exists"
+    );
     xlog "_start_master: all services listening";
 }
 
@@ -1063,8 +1074,15 @@ sub create_user
     my $adminclient = $adminstore->get_client();
 
     my @mboxes = ( $mb->to_external() );
-    map { push(@mboxes, $mb->make_child($_)->to_external()); } @{$params{subdirs}}
-        if ($params{subdirs});
+    foreach my $subdir ($params{subdirs} ? @{$params{subdirs}} : ())
+    {
+        if (ref $subdir eq 'Cassandane::Mboxname') {
+            push(@mboxes, $subdir->to_external());
+        }
+        else {
+            push(@mboxes, $mb->make_child($subdir)->to_external());
+        }
+    }
 
     foreach my $mb (@mboxes)
     {
@@ -1096,41 +1114,22 @@ sub _start_smtpd
 {
     my ($self) = @_;
 
-    my $basedir = $self->{basedir};
+    return if not $self->{smtpdaemon};
 
-    my $host = '127.0.0.1';
+    my $smtp_host = $self->{config}->get('smtp_host');
+    die "smtp_host requested but not configured"
+        if not $smtp_host or $smtp_host eq 'bogus:0';
 
-    my $port = Cassandane::PortManager::alloc($host);
+    my ($host, $port) = split /:/, $smtp_host;
 
-    my $smtppid = fork();
-    unless ($smtppid) {
-        # Child process.
-        # XXX This child still has the whole test's process space
-        # XXX still mapped, and when it exits, all our destructors
-        # XXX will be called, leaving the test in who knows what
-        # XXX state...
-        $SIG{TERM} = sub { die "killed" };
-
-        POSIX::close( $_ ) for 3 .. 1024; ## Arbitrary upper bound
-
-        $0 = "cassandane smtpd: $basedir";
-
-        my $smtpd = Cassandane::Net::SMTPServer->new({
-            cass_verbose => 1,
-            xmtp_personality => 'smtp',
-            host => $host,
-            port => $port,
-            max_servers => 3, # default is 50, yikes
-            control_file => "$basedir/conf/smtpd.json",
-            xmtp_tmp_dir => "$basedir/tmp/",
-            store_msg => 1,
-            messages_dir => "$basedir/smtpd/",
-        });
-        $smtpd->run() or die;
-        exit 0; # Never reached
-    }
-
-    # Parent process.
+    my $smtppid = $self->run_command({
+            cyrus => 0,
+            background => 1,
+        },
+        abs_path('utils/fakesmtpd'),
+        '-h', $host,
+        '-p', $port,
+    );
 
     # give the child a moment to actually start up
     sleep 1;
@@ -1138,21 +1137,19 @@ sub _start_smtpd
     # and then make sure it did!
     my $waitstatus = waitpid($smtppid, WNOHANG);
     if ($waitstatus == 0) {
-        $self->{smtphost} = $host . ':' . $port;
-
-        xlog "started smtpd as $smtppid";
+        xlog "started fakesmtpd as $smtppid";
         push @{$self->{_shutdowncallbacks}}, sub {
             local *__ANON__ = "kill_smtpd";
             my $self = shift;
-            xlog "killing smtpd $smtppid";
+            xlog "killing fakesmtpd $smtppid";
             kill(15, $smtppid);
-            waitpid($smtppid, 0);
+            $self->reap_command($smtppid);
         };
     }
     else {
         # child process already exited, something has gone wrong
         Cassandane::PortManager::free($port);
-        die "smtpd with pid=$smtppid failed to start";
+        die "fakesmtpd with pid=$smtppid failed to start";
     }
 }
 
@@ -1161,7 +1158,7 @@ sub start_httpd {
 
     my $basedir = $self->{basedir};
 
-    my $host = '127.0.0.1';
+    my $host = 'localhost';
     $port ||= Cassandane::PortManager::alloc($host);
 
     my $httpdpid = fork();
@@ -1217,10 +1214,23 @@ sub start
     $self->_init_basedir_and_name();
     xlog "start $self->{description}: basedir $self->{basedir}";
 
-    if ($self->{description} =~ m/^main instance for test /) {
-        # Start SMTP server before generating imapd config, we need to
-        # to set smtp_host to the auto-assigned TCP port it listens on.
-        $self->_start_smtpd();
+    # arrange for fakesmtpd to be started by Cassandane if we need it
+    # XXX should make it a Cyrus waitdaemon instead like fakesaslauthd
+    if ($self->{smtpdaemon}) {
+        my ($maj, $min) =
+            Cassandane::Instance->get_version($self->{installation});
+
+        if ($maj > 3 || ($maj == 3 && $min >= 1)) {
+            my $host = '127.0.0.1';
+            my $port = Cassandane::PortManager::alloc($host);
+
+            $self->{config}->set(
+                smtp_host => "$host:$port",
+            );
+        }
+        else {
+            die "smtpdaemon requested but Cyrus $maj.$min is too old";
+        }
     }
 
     # arrange for fakesaslauthd to be started by master
@@ -1289,6 +1299,7 @@ sub start
         # XXX cassandane won't know about it.
     }
     $self->setup_syslog_replacement();
+    $self->_start_smtpd() if $self->{smtpdaemon};
     $self->_start_notifyd();
     $self->_uncompress_berkeley_crud();
     $self->_start_master();
@@ -1417,12 +1428,18 @@ sub _detect_core_program
     my $lines = 0;
     my $prog;
 
+    my $bindir_pattern = qr{
+        \/
+        (?:bin|sbin|libexec)
+        \/
+    }x;
+
     open STRINGS, '-|', ('strings', '-a', $core)
         or die "Cannot run strings on $core: $!";
     while (<STRINGS>)
     {
         chomp;
-        if (m/\/bin\//)
+        if (m/$bindir_pattern/)
         {
             $prog = $_;
             last;
@@ -1435,12 +1452,15 @@ sub _detect_core_program
     return $prog;
 }
 
-sub _check_cores
+sub find_cores
 {
     my ($self) = @_;
-
     my $coredir = $self->{basedir} . '/conf/cores';
-    my $ncores = 0;
+
+    my $cassini = Cassandane::Cassini->instance();
+    my $core_pattern = $cassini->get_core_pattern();
+
+    my @cores;
 
     return unless -d $coredir;
     opendir CORES, $coredir
@@ -1448,22 +1468,32 @@ sub _check_cores
     while ($_ = readdir CORES)
     {
         next if m/^\./;
-        next unless m/^core(\.\d+)?$/;
+        next unless m/$core_pattern/;
         my $core = "$coredir/$_";
         next if -z $core;
         chmod(0644, $core);
-        $ncores++;
+        push @cores, $core;
 
         my $prog = _detect_core_program($core);
 
         xlog "Found core file $core";
-        xlog "   from program $prog" if defined $prog;
+        if (defined $prog) {
+           xlog "   from program $prog";
+           my ($bin) = $prog =~ m/^(\S+)/; # binary only
+           xlog "   debug: sudo gdb $bin $core";
+        }
     }
     closedir CORES;
 
-    return "Core files found in $coredir" if $ncores;
+    return @cores;
+}
 
-    return;
+sub _check_cores
+{
+    my ($self) = @_;
+    my $coredir = $self->{basedir} . '/conf/cores';
+
+    return "Core files found in $coredir" if scalar $self->find_cores();
 }
 
 sub _check_mupdate
@@ -1672,6 +1702,7 @@ sub stop
     push @errors, $self->_check_valgrind_logs();
     push @errors, $self->_check_cores();
     push @errors, $self->_check_syslog() unless $params{no_check_syslog};
+    push @errors, "master ready file still exists" if -e "$self->{basedir}/master.ready";
 
     # filter out empty errors (shouldn't be any, but just in case)
     @errors = grep { $_ } @errors;
@@ -1940,7 +1971,7 @@ sub _fork_command
     {
         push(@cmd, $self->_binary($binary), '-C', $self->_imapd_conf());
     }
-    elsif ($binary eq 'delve') {
+    elsif ($binary =~ m/xapian.*$/) {
         push(@cmd, $self->_binary($binary));
     }
     else {
@@ -1997,6 +2028,9 @@ sub _fork_command
         $ENV{CASSANDANE_SYSLOG_FNAME} = abs_path($self->{syslog_fname});
         $ENV{LD_PRELOAD} = abs_path('utils/syslog.so')
     }
+    $ENV{PKG_CONFIG_PATH} = $self->{cyrus_destdir}
+                          . $self->{cyrus_prefix}
+                          . "/lib/pkgconfig";
 
 #     xlog "\$PERL5LIB is"; map { xlog "    $_"; } split(/:/, $ENV{PERL5LIB});
 
@@ -2006,13 +2040,10 @@ sub _fork_command
     # entirely clear how to detect that - we could use readelf -d
     # on an executable to discover what it thinks it's RPATH ought
     # to be, then prepend destdir to that.
-    if ($self->{cyrus_destdir} ne "")
-    {
-        $ENV{LD_LIBRARY_PATH} = join(':', (
-                $self->{cyrus_destdir} . $self->{cyrus_prefix} . "/lib",
-                split(/:/, $ENV{LD_LIBRARY_PATH} || "")
-        ));
-    }
+    $ENV{LD_LIBRARY_PATH} = join(':', (
+            $self->{cyrus_destdir} . $self->{cyrus_prefix} . "/lib",
+            split(/:/, $ENV{LD_LIBRARY_PATH} || "")
+    ));
 #     xlog "\$LD_LIBRARY_PATH is"; map { xlog "    $_"; } split(/:/, $ENV{LD_LIBRARY_PATH});
 
     my $cd = $options->{workingdir};
@@ -2063,8 +2094,18 @@ sub _fork_command
             or die "Cannot redirect STDERR to $redirects{stderr}: $!";
     }
 
-    exec @cmd;
-    die "Cannot run $binary: $!";
+    # exec in a block by itself shushes the "Statement unlikely to be reached"
+    # warning, which is generated when exec is followed by something other
+    # than die
+    { exec @cmd; }
+
+    # If exec failed, then this process is still a clone of a Worker.  If we
+    # die here it would report a test failure, then loop around and try to
+    # run the next test!  And if we exit here, it would deconstruct the real
+    # Worker's memory space out from under it.  Need to use POSIX::_exit
+    # to bypass all that and have the child process actually exit.
+    xlog "Cannot run $binary: $!";
+    POSIX::_exit(71); # EX_OSERR
 }
 
 sub _handle_wait_status
