@@ -953,10 +953,20 @@ sub _finish_workitem
     my ($self, $witem, $result, $runner) = @_;
     my ($suite, $test) = $self->_get_suite_and_test($witem);
 
+    # The test was actually started earlier by _run_workitem, but its
+    # start_test event wasn't sent.  It might have got swallowed due to
+    # the output format listeners being removed in the workitem handling.
+    # Send the event again now, to make sure the formatters actually get
+    # it...
     $result->start_test($test);
-    if ($runner->can('fake_start_time'))
+    # But! If they're computing their own start time based on this event
+    # they'll get it wrong.  We know the real start time, so tell the
+    # formatter to use that instead.
+    if ($runner->can('tell_formatters'))
     {
-        $runner->fake_start_time($test, $witem->{start_time});
+        $runner->tell_formatters('fake_start_time',
+                                 $test,
+                                 $witem->{start_time});
     }
 
     $test->annotate_from_file($witem->{logfile});
