@@ -444,7 +444,7 @@ static int callout_receive_reply(const char *callout,
     p = prot_new(fd, /*write*/0);
 
     /* read and parse the reply as a dlist */
-    c = dlist_parse(results, /*parsekeys*/0, /*isarchive*/0, /*isbackup*/0, p);
+    c = dlist_parse(results, /*parsekeys*/0, /*isarchive*/0, p);
     r = (c == EOF ? IMAP_SYS_ERROR : 0);
 
 out:
@@ -1438,7 +1438,8 @@ out:
  * contains the name of the user whose \Seen flag gets set.
  */
 EXPORTED int append_copy(struct mailbox *mailbox, struct appendstate *as,
-                         ptrarray_t *msgrecs, int nolink, int is_same_user)
+                         ptrarray_t *msgrecs, int nolink, int is_same_user,
+                         struct progress_rock *prock)
 {
     int msg;
     char *srcfname = NULL;
@@ -1470,6 +1471,8 @@ EXPORTED int append_copy(struct mailbox *mailbox, struct appendstate *as,
         uint32_t src_system_flags;
         uint32_t src_internal_flags;
 
+        if (prock) prock->cb(msg, msgrecs->count, prock);
+
         r = msgrecord_get_uid(src_msgrec, &src_uid);
         if (r) goto out;
         r = msgrecord_get_systemflags(src_msgrec, &src_system_flags);
