@@ -129,9 +129,15 @@ static int logout(struct backend *s __attribute__((unused)))
     return 0;
 }
 
+static const struct tls_alpn_t http_alpn_map[] = {
+    { "http/1.1", NULL, NULL },
+    { "",         NULL, NULL },
+};
 
 static struct protocol_t http =
-{ "http", "HTTP", TYPE_SPEC, { .spec = { &login, &ping, &logout } } };
+{ "http", "HTTP", http_alpn_map, TYPE_SPEC,
+  { .spec = { &login, &ping, &logout } }
+};
 
 static int extractor_connect(struct extractor_ctx *ext)
 {
