@@ -536,6 +536,7 @@ EXPORTED int backend_starttls(  struct backend *s,
     /* SASL and openssl have different ideas about whether ssf is signed */
     layerp = (int *) &s->ext_ssf;
     r = tls_start_clienttls(s->in->fd, s->out->fd, layerp, &auth_id,
+                            s->prot ? s->prot->alpn_map : NULL,
                             &s->tlsconn, &s->tlssess);
     if (r == -1) return -1;
 
@@ -1056,6 +1057,9 @@ EXPORTED struct backend *backend_connect(struct backend *ret_backend, const char
             }
         }
 
+        // don't do lookups if the port number is zero
+        if (!strcmp(service, "0")) goto error;
+
         memset(&hints, 0, sizeof(hints));
         hints.ai_family = PF_UNSPEC;
         hints.ai_socktype = SOCK_STREAM;
