@@ -56,6 +56,7 @@
 #include "caldav_util.h"
 #include "cyrusdb.h"
 #include "defaultalarms.h"
+#include "hashset.h"
 #include "httpd.h"
 #include "http_dav.h"
 #include "ical_support.h"
@@ -225,8 +226,12 @@ static sqldb_t *caldav_alarm_open()
         return NULL;
     }
 
-    // XXX - config option?
-    char *dbfilename = strconcat(config_dir, "/caldav_alarm.sqlite3", NULL);
+    char *tempname = NULL;
+    const char *dbfilename = config_getstring(IMAPOPT_CALDAV_ALARM_DB_PATH);
+    if (!dbfilename) {
+        tempname = strconcat(config_dir, "/caldav_alarm.sqlite3", NULL);
+        dbfilename = tempname;
+    }
     my_alarmdb = sqldb_open(dbfilename, CMD_CREATE, DBVERSION, upgrade,
                             config_getduration(IMAPOPT_DAV_LOCK_TIMEOUT, 's') * 1000);
 
@@ -235,7 +240,7 @@ static sqldb_t *caldav_alarm_open()
         mboxname_release(&my_alarmdb_lock);
     }
 
-    free(dbfilename);
+    free(tempname);
     refcount = 1;
     return my_alarmdb;
 }
@@ -511,29 +516,38 @@ static int process_alarm_cb(icalcomponent *comp,
     icalcomponent *alarm;
     icalproperty *prop;
     icalvalue *val;
-
     int alarmno = 0;
+    int keep_processing_recurrences = 1;
+    int suppress_duplicates =
+        config_getswitch(IMAPOPT_CALENDAR_SUPPRESS_DUPLICATE_ALARMS);
+    size_t suppressed_n_duplicates = 0;
+
+    struct alarm_id {
+        icaltimetype alarmtime;
+        icalproperty_action action;
+    };
+
+    struct hashset *seen_alarms = hashset_new(sizeof(struct alarm_id));
 
     /* Skip cancelled events */
-    if (icalcomponent_get_status(comp) == ICAL_STATUS_CANCELLED) return 1;
+    if (icalcomponent_get_status(comp) == ICAL_STATUS_CANCELLED) goto done;
 
     /* Skip declined events */
     get_schedule_addresses(data->mboxname, data->userid, &sched_addrs);
 
     for (prop = icalcomponent_get_first_invitee(comp);
          prop; prop = icalcomponent_get_next_invitee(comp)) {
-        const char *attendee = icalproperty_get_invitee(prop);
+        const char *attendee = icalproperty_get_decoded_calendaraddress(prop);
 
         if (!attendee) continue;
-        if (!strncasecmp(attendee, "mailto:", 7)) attendee += 7;
 
-        if (strarray_find_case(&sched_addrs, attendee, 0) >= 0) {
+        if (strarray_contains_case(&sched_addrs, attendee)) {
             const char *partstat =
                 icalproperty_get_parameter_as_string(prop, "PARTSTAT");
 
             if (!strcasecmpsafe(partstat, "DECLINED")) {
                 strarray_fini(&sched_addrs);
-                return 1;
+                goto done;
             }
         }
     }
@@ -569,7 +583,7 @@ static int process_alarm_cb(icalcomponent *comp,
         struct icaltriggertype trigger = icalvalue_get_trigger(val);
         /* XXX validate trigger */
 
-        icaltimetype alarmtime = icaltime_null_time();
+        icaltimetype alarmtime;
         unsigned is_duration = (icalvalue_isa(val) == ICAL_DURATION_VALUE);
         if (is_duration) {
             icalparameter *param =
@@ -604,6 +618,12 @@ static int process_alarm_cb(icalcomponent *comp,
             prop = icalcomponent_get_first_property(comp, ICAL_SUMMARY_PROPERTY);
             const char *summary =
                 prop ? icalproperty_get_value_as_string(prop) : "[no summary]";
+
+            struct alarm_id alarm_id;
+            memset(&alarm_id, 0, sizeof(struct alarm_id));
+            alarm_id.alarmtime = alarmtime;
+            alarm_id.action = action;
+
             int age = data->now - check;
             if (age > 7200) { // more than 2 hours stale?  Just log it
                 syslog(LOG_ERR, "suppressing alarm aged %d seconds "
@@ -614,6 +634,9 @@ static int process_alarm_cb(icalcomponent *comp,
                        icaltime_as_ical_string(start), alarmno,
                        summary);
             }
+            else if (!hashset_add(seen_alarms, &alarm_id) && suppress_duplicates) {
+                suppressed_n_duplicates++;
+            }
             else {
                 syslog(LOG_NOTICE, "sending alarm at %s for %s %u - %s(%d) %s",
                        icaltime_as_ical_string(alarmtime),
@@ -635,11 +658,20 @@ static int process_alarm_cb(icalcomponent *comp,
             time_t next = data->now + 86400*30;
             if (!data->nextcheck || next < data->nextcheck)
                 data->nextcheck = next;
-            return 0;
+            keep_processing_recurrences = 0;
+            goto done;
         }
     }
 
-    return 1; /* keep going */
+    if (suppressed_n_duplicates) {
+        xsyslog(LOG_INFO, "suppressed duplicate alarms",
+                "mboxname=<%s> imap_uid=<%d> n_duplicates=<%zu>",
+                data->mboxname, data->imap_uid, suppressed_n_duplicates);
+    }
+
+done:
+    hashset_free(&seen_alarms);
+    return keep_processing_recurrences;
 }
 
 static int update_alarmdb(const char *mboxname,
@@ -717,7 +749,7 @@ static int has_peruser_alarms_cb(const char *mailbox,
         return 0;
     }
 
-    dlist_parsemap(&dl, 1, 0, buf_base(value), buf_len(value));
+    dlist_parsemap(&dl, 1, buf_base(value), buf_len(value));
     const char *strval = NULL;
     if (dlist_getatom(dl, "USEDEFAULTALERTS", &strval)) {
         if (!strcasecmp(strval, "YES")) {
@@ -767,7 +799,7 @@ static int check_by_array(const short *byX, short size,
     memcpy(my_byX, byX, size * sizeof(short));
 
     /* convert each value to seconds and determine actual length of data */
-    for (len = 0; len < size && my_byX[len] != ICAL_RECURRENCE_ARRAY_MAX; len++) {
+    for (len = 0; len < size; len++) {
         my_byX[len] *= to_seconds;
     }
 
@@ -792,6 +824,34 @@ static int check_by_array(const short *byX, short size,
     return disable;
 }
 
+static int is_supported_component(icalcomponent *ical)
+{
+    icalcomponent *comp = icalcomponent_get_first_real_component(ical);
+    icalcomponent_kind kind = icalcomponent_isa(comp);
+    uint64_t is_supported = 0;
+
+    switch (kind) {
+    case ICAL_VEVENT_COMPONENT:
+        is_supported =
+            config_getbitfield(IMAPOPT_CALDAV_ALARM_SUPPORT_COMPONENTS) &
+            IMAP_ENUM_CALDAV_ALARM_SUPPORT_COMPONENTS_VEVENT;
+        break;
+    case ICAL_VTODO_COMPONENT:
+        is_supported = config_getbitfield(IMAPOPT_CALDAV_ALARM_SUPPORT_COMPONENTS) &
+               IMAP_ENUM_CALDAV_ALARM_SUPPORT_COMPONENTS_VTODO;
+        break;
+    default:
+        ; // do nothing
+    }
+
+    if (!is_supported) {
+        xsyslog(LOG_DEBUG, "alarms are not supported for iCalendar component",
+                "kind=<%s>", icalenum_component_kind_to_string(kind));
+    }
+
+    return is_supported;
+}
+
 static int has_alarms(void *data, struct mailbox *mailbox,
                       uint32_t uid, unsigned *num_rcpts)
 {
@@ -814,6 +874,12 @@ static int has_alarms(void *data, struct mailbox *mailbox,
 
     icalcomponent *ical = (icalcomponent *) data;
     if (ical) {
+        if (!is_supported_component(ical)) {
+            xsyslog(LOG_DEBUG, "ignoring alarms for index record",
+                    "mboxname=<%s> imap_uid=<%d>", mailbox_name(mailbox), uid);
+            return 0;
+        }
+
         /* Check iCalendar resource for VALARMs */
         icalcomponent *comp = icalcomponent_get_first_real_component(ical);
         icalcomponent_kind kind = icalcomponent_isa(comp);
@@ -829,12 +895,16 @@ static int has_alarms(void *data, struct mailbox *mailbox,
                  prop =
                      icalcomponent_get_next_property(comp, ICAL_RRULE_PROPERTY)) {
 
-                struct icalrecurrencetype rrule = icalproperty_get_rrule(prop);
-                int recur_interval = rrule.interval;
+                struct icalrecurrencetype *rrule =
+                    icalproperty_get_recurrence(prop);
+                if (!rrule) continue;
+
+                int recur_interval = rrule->interval;
                 const char *bypart = "";
                 int disable = 0;
+                short size;
 
-                switch (rrule.freq) {
+                switch (rrule->freq) {
                 case ICAL_YEARLY_RECURRENCE:
                 case ICAL_MONTHLY_RECURRENCE:
                 case ICAL_WEEKLY_RECURRENCE:
@@ -863,20 +933,20 @@ static int has_alarms(void *data, struct mailbox *mailbox,
                     break;
                 }
 
-                if (rrule.by_second[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                if ((size = icalrecur_byrule_size(rrule, ICAL_BY_SECOND))) {
                     bypart = "SECOND";
-                    disable = check_by_array(rrule.by_second, ICAL_BY_SECOND_SIZE,
-                                             recur_interval, 1);
+                    disable = check_by_array(icalrecur_byrule_data(rrule, ICAL_BY_SECOND),
+                                             size, recur_interval, 1);
                 }
-                else if (rrule.by_minute[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                else if ((size = icalrecur_byrule_size(rrule, ICAL_BY_MINUTE))) {
                     bypart = "MINUTE";
-                    disable = check_by_array(rrule.by_minute, ICAL_BY_MINUTE_SIZE,
-                                             recur_interval, 60);
+                    disable = check_by_array(icalrecur_byrule_data(rrule, ICAL_BY_MINUTE),
+                                             size, recur_interval, 60);
                 }
-                else if (rrule.by_hour[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                else if ((size = icalrecur_byrule_size(rrule, ICAL_BY_HOUR))) {
                     bypart = "HOUR";
-                    disable = check_by_array(rrule.by_hour, ICAL_BY_HOUR_SIZE,
-                                             recur_interval, 3600);
+                    disable = check_by_array(icalrecur_byrule_data(rrule, ICAL_BY_HOUR),
+                                             size, recur_interval, 3600);
                 }
                 else if (recur_interval < min_interval) {
                     disable = 1;
@@ -887,11 +957,14 @@ static int has_alarms(void *data, struct mailbox *mailbox,
                             "Disabling alarms for high frequence calendar entry",
                             "freq=<%s> interval=<%u> bypart=<%s>"
                             " mboxname=<%s> imap_uid=<%d>",
-                            icalrecur_freq_to_string(rrule.freq),
-                            rrule.interval, bypart,
+                            icalrecur_freq_to_string(rrule->freq),
+                            rrule->interval, bypart,
                             mailbox_name(mailbox), uid);
+                    icalrecurrencetype_unref(rrule);
                     return 0;
                 }
+
+                icalrecurrencetype_unref(rrule);
             }
 
             if (icalcomponent_get_first_component(comp, ICAL_VALARM_COMPONENT))
@@ -917,6 +990,17 @@ static time_t process_alarms(const char *mboxname, uint32_t imap_uid,
                              icalcomponent *ical, time_t lastrun,
                              time_t runtime, int dryrun)
 {
+    if (!is_supported_component(ical)) {
+        // Ignore unsupported components that already made it into
+        // the database. They might have been added before we added
+        // support to ignore component kinds, or because an installation
+        // changed their config of supported components and did not
+        // rebuild the alarm database.
+        xsyslog(LOG_DEBUG, "ignoring alarms for index record",
+                "mboxname=<%s> imap_uid=<%d>", mboxname, imap_uid);
+        return 0;
+    }
+
     icalcomponent *myical = NULL;
 
     /* Add default alarms */
@@ -925,7 +1009,7 @@ static time_t process_alarms(const char *mboxname, uint32_t imap_uid,
 
         if (!defaultalarms_load(mboxname, userid, &defalarms)) {
             myical = icalcomponent_clone(ical);
-            defaultalarms_insert(&defalarms, myical);
+            defaultalarms_insert(&defalarms, myical, 0);
             ical = myical;
         }
 
@@ -1187,7 +1271,7 @@ static int process_peruser_alarms_cb(const char *mailbox, uint32_t uid,
     }
 
     /* Extract VPATCH from per-user-cal-data annotation */
-    dlist_parsemap(&dl, 1, 0, buf_base(value), buf_len(value));
+    dlist_parsemap(&dl, 1, buf_base(value), buf_len(value));
     vpatch = vpatch_from_peruserdata(dl);
 
     /* Apply VPATCH to a clone of the iCalendar resource */
@@ -1714,7 +1798,7 @@ static int process_futurerelease(struct caldav_alarm_data *data,
     if (r) {
         /* Determine if we should retry (again) or cancel the submission.
            We try at 5m, 15m, 30m, 60m after original scheduled time. */
-        unsigned duration;
+        unsigned duration = 0;
         switch (data->num_retries) {
         case 0: duration =  300; break;
         case 1: duration =  600; break;
@@ -2012,6 +2096,20 @@ EXPORTED int caldav_alarm_process(time_t runtime, time_t *intervalp, int dryrun)
 {
     int i;
 
+    // we don't process alarms on replicas
+    if (config_getswitch(IMAPOPT_REPLICAONLY))
+        return 0;
+
+    // temporarily disable alarms
+    const char *suppress_file = config_getstring(IMAPOPT_CALDAV_ALARM_SUPPRESS_FILE);
+    if (suppress_file) {
+        struct stat sbuf;
+        if (!stat(suppress_file, &sbuf)) {
+            syslog(LOG_NOTICE, "NOTICE: suppressing alarms due to existence of %s", suppress_file);
+            return 0;
+        }
+    }
+
     syslog(LOG_DEBUG, "processing alarms");
 
     if (!runtime) {
@@ -2042,6 +2140,7 @@ EXPORTED int caldav_alarm_process(time_t runtime, time_t *intervalp, int dryrun)
         int skipped_some = 0;
         int did_some = 0;
         int num_user_records = 0;
+        int skip_user = 0;
         char *userid = NULL;
         struct mboxlock *nslock = NULL;
         for (i = 0; i < rock.list.count; i++) {
@@ -2058,13 +2157,20 @@ EXPORTED int caldav_alarm_process(time_t runtime, time_t *intervalp, int dryrun)
             // userid will be next to each other
             if (strcmpsafe(userid, mbname_userid(mbname))) {
                 num_user_records = 0;
+                skip_user = 0;
                 free(userid);
                 mboxname_release(&nslock);
                 userid = xstrdup(mbname_userid(mbname));
+                if (user_isreplicaonly(userid)) {
+                    skip_user = 1;
+                    continue;
+                }
                 nslock = user_namespacelock_full(userid, LOCK_NONBLOCKING);
             }
             mbname_free(&mbname);
 
+            if (skip_user) continue;
+
             // if we failed to lock the user, or have done too many for this user, skip
             if (!nslock || ++num_user_records > MAX_CONSECUTIVE_ALARMS_PER_USER) {
                 skipped_some++;
@@ -2083,7 +2189,8 @@ EXPORTED int caldav_alarm_process(time_t runtime, time_t *intervalp, int dryrun)
         mboxname_release(&nslock);
 
         // if we both made some progress AND skipped some, then retry again immediately
-        if (did_some && skipped_some) rock.next = runtime;
+        if (did_some && skipped_some) *intervalp = 0;
+        else *intervalp = rock.next - runtime;
     }
     else {
         // we're testing or reconstructing, run everything!
@@ -2099,8 +2206,6 @@ EXPORTED int caldav_alarm_process(time_t runtime, time_t *intervalp, int dryrun)
 
     syslog(LOG_DEBUG, "done");
 
-    if (intervalp) *intervalp = rock.next - runtime;
-
     return rc;
 }
 
