@@ -1269,8 +1269,10 @@ static int _carddav_store(struct mailbox *mailbox, struct buf *vcard,
     /* Since we use the vCard UID in the resource name,
        this param may be long and needs to get properly split per RFC 2231 */
     struct buf value = BUF_INITIALIZER;
-    charset_write_mime_param(&value, /*extended*/1, MIME_MAX_HEADER_LENGTH,
-                             "filename", resource);
+    charset_append_mime_param(&value,
+                              CHARSET_PARAM_XENCODE | CHARSET_PARAM_NEWLINE,
+                              "filename",
+                              resource);
     fprintf(f, "Content-Disposition: inline%s\r\n", buf_cstring(&value));
     buf_free(&value);
 
@@ -1309,7 +1311,7 @@ static int _carddav_store(struct mailbox *mailbox, struct buf *vcard,
         goto done;
     }
 
-    /* Commit the append to the calendar mailbox */
+    /* Commit the append to the addressbook mailbox */
     r = append_commit(&as);
     if (r) {
         syslog(LOG_ERR, "append_commit() failed");
@@ -1422,6 +1424,7 @@ EXPORTED int carddav_writecard_x(struct carddav_db *carddavdb,
     strarray_t emails = STRARRAY_INITIALIZER;
     strarray_t member_uids = STRARRAY_INITIALIZER;
     vcardproperty *prop;
+    char *propval = NULL;
 
     for (prop = vcardcomponent_get_first_property(vcard, VCARD_ANY_PROPERTY);
          prop;
@@ -1433,7 +1436,8 @@ EXPORTED int carddav_writecard_x(struct carddav_db *carddavdb,
          * So, we use vcardproperty_get_value_as_string_r() here instead
          * and manage the memory ourselves.
          */
-        char *propval = vcardproperty_get_value_as_string_r(prop);
+        free(propval);
+        propval = vcardproperty_get_value_as_string_r(prop);
         const char *userid = "";
 
         if (!propval) continue;
@@ -1475,15 +1479,8 @@ EXPORTED int carddav_writecard_x(struct carddav_db *carddavdb,
                       vcardproperty_get_first_parameter(prop,
                                                         VCARD_TYPE_PARAMETER))) {
                 vcardenumarray *types = vcardparameter_get_type(param);
-                size_t i;
-                for (i = 0; i < vcardenumarray_size(types); i++) {
-                    const vcardenumarray_element *type =
-                        vcardenumarray_element_at(types, i);
-                    if (!strcasecmpsafe(type->xvalue, "pref")) {
-                        ispref = 1;
-                        break;
-                    }
-                }
+                vcardenumarray_element pref = { .val = VCARD_TYPE_PREF };
+                if (vcardenumarray_find(types, &pref) >= 0) ispref = 1;
             }
             strarray_appendm(&emails, propval);
             strarray_append(&emails, ispref ? "1" : "");
@@ -1526,8 +1523,6 @@ EXPORTED int carddav_writecard_x(struct carddav_db *carddavdb,
         default:
             break;
         }
-
-        free(propval);
     }
 
     int r;
@@ -1538,17 +1533,19 @@ EXPORTED int carddav_writecard_x(struct carddav_db *carddavdb,
             { NULL,            SQLITE_NULL,    { .s = NULL             } } };
 
         r = sqldb_exec(carddavdb->db, CMD_DELETE_JSCARDCACHE, bval, NULL, NULL);
-        if (r) return r;
+        if (r) goto done;
     }
     r = carddav_write(carddavdb, cdata);
     if (!r) r = carddav_write_emails(carddavdb,
                                      cdata->dav.rowid, &emails, ispinned);
     if (!r) r = carddav_write_groups(carddavdb, cdata->dav.rowid, &member_uids);
 
+done:
     buf_free(&nicknames);
     strarray_fini(&values);
     strarray_fini(&emails);
     strarray_fini(&member_uids);
+    free(propval);
 
     return r;
 }
