@@ -291,7 +291,7 @@
 
 static int sievedb_upgrade(sqldb_t *db);
 
-struct sqldb_upgrade davdb_upgrade[] = {
+static const struct sqldb_upgrade davdb_upgrade[] = {
   { 2, CMD_DBUPGRADEv2, NULL },
   { 3, CMD_DBUPGRADEv3, NULL },
   { 4, CMD_DBUPGRADEv4, NULL },
@@ -434,7 +434,7 @@ static int _dav_reconstruct_mb(const mbentry_t *mbentry,
             strarray_t *specialuse =
                 strarray_split(buf_cstring(&attrib), NULL, 0);
 
-            if (strarray_find(specialuse, "\\Snoozed", 0) >= 0) {
+            if (strarray_contains(specialuse, "\\Snoozed")) {
                 addproc = &mailbox_add_email_alarms;
             }
             strarray_free(specialuse);
