@@ -66,6 +66,7 @@
 #include "charset.h"
 #include "cyr_lock.h"
 #include "gmtoff.h"
+#include "haproxy.h"
 #include "iptostring.h"
 #include "global.h"
 #include "ical_support.h"
@@ -98,7 +99,7 @@ EXPORTED volatile sig_atomic_t in_shutdown = 0;
 
 EXPORTED int config_fulldirhash;                                /* 0 */
 EXPORTED int config_implicitrights;                     /* "lkxa" */
-EXPORTED unsigned long config_metapartition_files;      /* 0 */
+EXPORTED uint64_t config_metapartition_files;      /* 0 */
 EXPORTED const char *config_mboxlist_db;
 EXPORTED const char *config_quota_db;
 EXPORTED const char *config_subscription_db;
@@ -112,11 +113,14 @@ EXPORTED const char *config_statuscache_db;
 HIDDEN const char *config_userdeny_db;
 EXPORTED const char *config_zoneinfo_db;
 EXPORTED const char *config_conversations_db;
-EXPORTED const char *config_backup_db;
 EXPORTED int charset_flags;
 EXPORTED int charset_snippet_flags;
 EXPORTED size_t config_search_maxsize;
 EXPORTED int config_httpprettytelemetry;
+EXPORTED int config_take_globallock;
+EXPORTED char *config_skip_userlock;
+EXPORTED int haproxy_protocol = 0;
+EXPORTED int imaply_strict = 1;
 
 static char session_id_buf[MAX_SESSIONID_SIZE];
 static int session_id_time = 0;
@@ -183,6 +187,21 @@ static void cyrus_modules_done()
     }
 }
 
+static void debug_toggled(void)
+{
+    int logmask = setlogmask(0); /* gets the current log mask */
+
+    if (config_debug)
+        logmask |= LOG_MASK(LOG_DEBUG);
+    else
+        logmask &= ~LOG_MASK(LOG_DEBUG);
+
+    syslog(LOG_INFO, "debug logging turned %s",
+                     config_debug ? "on" : "off");
+
+    setlogmask(logmask);
+}
+
 /* Called before a cyrus application starts (but after command line parameters
  * are read) */
 EXPORTED int cyrus_init(const char *alt_config, const char *ident, unsigned flags, int config_need_data)
@@ -201,6 +220,8 @@ EXPORTED int cyrus_init(const char *alt_config, const char *ident, unsigned flag
         cyrus_init_run = RUNNING;
     }
 
+    srand(time(NULL) * getpid());
+
     cyrus_init_nodb = (flags & CYRUSINIT_NODB);
 #ifdef LOG_PERROR
     if ((flags & CYRUSINIT_PERROR))
@@ -238,6 +259,9 @@ EXPORTED int cyrus_init(const char *alt_config, const char *ident, unsigned flag
         openlog(config_ident, syslog_opts, SYSLOG_FACILITY);
     }
 
+    /* allow toggleable debug logging */
+    config_toggle_debug_cb = &debug_toggled;
+
     /* Load configuration file.  This will set config_dir when it finds it */
     config_read(alt_config, config_need_data);
 
@@ -266,10 +290,6 @@ EXPORTED int cyrus_init(const char *alt_config, const char *ident, unsigned flag
     }
     /* Do not free ident_buf, syslog needs it for the life of this process! */
 
-    /* allow debug logging */
-    if (!config_debug)
-        setlogmask(~LOG_MASK(LOG_DEBUG));
-
     /* Look up default partition */
     config_defpartition = config_getstring(IMAPOPT_DEFAULTPARTITION);
     for (p = (char *)config_defpartition; p && *p; p++) {
@@ -346,7 +366,6 @@ EXPORTED int cyrus_init(const char *alt_config, const char *ident, unsigned flag
         config_userdeny_db = config_getstring(IMAPOPT_USERDENY_DB);
         config_zoneinfo_db = config_getstring(IMAPOPT_ZONEINFO_DB);
         config_conversations_db = config_getstring(IMAPOPT_CONVERSATIONS_DB);
-        config_backup_db = config_getstring(IMAPOPT_BACKUP_DB);
 
         /* configure libcyrus as needed */
         libcyrus_config_setstring(CYRUSOPT_CONFIG_DIR, config_dir);
@@ -401,11 +420,24 @@ EXPORTED int cyrus_init(const char *alt_config, const char *ident, unsigned flag
         debug_locks_longer_than = atof(locktime);
     }
 
+    const char *locked_user = getenv("CYRUS_HAVELOCK_USER");
+    if (locked_user) {
+        config_skip_userlock = xstrdup(locked_user);
+    }
+
+    const char *locked_global = getenv("CYRUS_HAVELOCK_GLOBAL");
+    config_take_globallock = config_getswitch(IMAPOPT_GLOBAL_LOCK);
+    if (locked_global && config_parse_switch(locked_global)) {
+        config_take_globallock = 0;
+    }
+
 #ifdef HAVE_ICAL
     /* Initialize libical */
     ical_support_init();
 #endif
 
+    register_mboxgroups_cb(mboxlist_lookup_usergroups);
+
     return 0;
 }
 
@@ -446,25 +478,20 @@ EXPORTED int mysasl_config(void *context __attribute__((unused)),
                   const char **result,
                   unsigned *len)
 {
-    if (!strcmp(option, "srvtab")) {
-        /* we don't transform srvtab! */
-        *result = config_getstring(IMAPOPT_SRVTAB);
-    } else {
-        *result = NULL;
+    *result = NULL;
 
-        if (plugin_name) {
-            /* first try it with the plugin name */
-            char *opt = strconcat("sasl_", plugin_name, "_", option, (char*)NULL);
-            *result = config_getoverflowstring(opt, NULL);
-            free(opt);
-        }
+    if (plugin_name) {
+        /* first try it with the plugin name */
+        char *opt = strconcat("sasl_", plugin_name, "_", option, (char*)NULL);
+        *result = config_getoverflowstring(opt, NULL);
+        free(opt);
+    }
 
-        if (*result == NULL) {
-            /* try without the plugin name */
-            char *opt = strconcat("sasl_", option, (char *)NULL);
-            *result = config_getoverflowstring(opt, NULL);
-            free(opt);
-        }
+    if (*result == NULL) {
+        /* try without the plugin name */
+        char *opt = strconcat("sasl_", option, (char *)NULL);
+        *result = config_getoverflowstring(opt, NULL);
+        free(opt);
     }
 
     if (*result != NULL) {
@@ -792,6 +819,8 @@ EXPORTED void cyrus_done(void)
 
     if (!cyrus_init_nodb)
         libcyrus_done();
+
+    xzfree(config_skip_userlock);
 }
 
 /*
@@ -914,107 +943,7 @@ EXPORTED void parse_sessionid(const char *str, char *sessionid)
 
 EXPORTED int capa_is_disabled(const char *str)
 {
-    if (!suppressed_capabilities) return 0;
-
-    return (strarray_find_case(suppressed_capabilities, str, 0) >= 0);
-}
-
-
-/* Find a message-id looking thingy in a string.  Returns a pointer to the
- * alloc'd id and the remaining string is returned in the **loc parameter.
- *
- * This is a poor-man's way of finding the message-id.  We simply look for
- * any string having the format "< ... @ ... >" and assume that the mail
- * client created a properly formatted message-id.
- */
-#define MSGID_SPECIALS "<> @\\"
-
-EXPORTED char *find_msgid(char *str, char **rem)
-{
-    char *msgid, *src, *dst, *cp;
-
-    if (!str) return NULL;
-
-    msgid = NULL;
-    src = str;
-
-    /* find the start of a msgid (don't go past the end of the header) */
-    while ((cp = src = strpbrk(src, "<\r")) != NULL) {
-
-        /* check for fold or end of header
-         *
-         * Per RFC 2822 section 2.2.3, a long header may be folded by
-         * inserting CRLF before any WSP (SP and HTAB, per section 2.2.2).
-         * Any other CRLF is the end of the header.
-         */
-        if (*cp++ == '\r') {
-            if (*cp++ == '\n' && !(*cp == ' ' || *cp == '\t')) {
-                /* end of header, we're done */
-                break;
-            }
-
-            /* skip fold (or junk) */
-            src++;
-            continue;
-        }
-
-        /* see if we have (and skip) a quoted localpart */
-        if (*cp == '\"') {
-            /* find the endquote, making sure it isn't escaped */
-            do {
-                ++cp; cp = strchr(cp, '\"');
-            } while (cp && *(cp-1) == '\\');
-
-            /* no endquote, so bail */
-            if (!cp) {
-                src++;
-                continue;
-            }
-        }
-
-        /* find the end of the msgid */
-        if ((cp = strchr(cp, '>')) == NULL)
-            return NULL;
-
-        /* alloc space for the msgid */
-        dst = msgid = (char*) xrealloc(msgid, cp - src + 2);
-
-        *dst++ = *src++;
-
-        /* quoted string */
-        if (*src == '\"') {
-            src++;
-            while (*src != '\"') {
-                if (*src == '\\') {
-                    src++;
-                }
-                *dst++ = *src++;
-            }
-            src++;
-        }
-        /* atom */
-        else {
-            while (!strchr(MSGID_SPECIALS, *src))
-                *dst++ = *src++;
-        }
-
-        if (*src != '@' || *(dst-1) == '<') continue;
-        *dst++ = *src++;
-
-        /* domain atom */
-        while (!strchr(MSGID_SPECIALS, *src))
-            *dst++ = *src++;
-
-        if (*src != '>' || *(dst-1) == '@') continue;
-        *dst++ = *src++;
-        *dst = '\0';
-
-        if (rem) *rem = src;
-        return msgid;
-    }
-
-    if (msgid) free(msgid);
-    return NULL;
+    return strarray_contains_case(suppressed_capabilities, str);
 }
 
 /*
@@ -1025,13 +954,14 @@ EXPORTED const char *get_clienthost(int s, const char **localip, const char **re
 {
 #define IPBUF_SIZE (NI_MAXHOST+NI_MAXSERV+2)
     socklen_t salen;
-    struct sockaddr_storage localaddr, remoteaddr;
+    struct sockaddr_storage localaddr = { 0 }, remoteaddr = { 0 };
     struct sockaddr *localsock = (struct sockaddr *)&localaddr;
     struct sockaddr *remotesock = (struct sockaddr *)&remoteaddr;
     static struct buf clientbuf = BUF_INITIALIZER;
     static char lipbuf[IPBUF_SIZE], ripbuf[IPBUF_SIZE];
     char hbuf[NI_MAXHOST];
     int niflags;
+    int r = 0;
 
     buf_reset(&clientbuf);
     *localip = *remoteip = NULL;
@@ -1039,7 +969,16 @@ EXPORTED const char *get_clienthost(int s, const char **localip, const char **re
     /* determine who we're talking to */
 
     salen = sizeof(struct sockaddr_storage);
-    if (getpeername(s, remotesock, &salen) == 0 &&
+
+    if (haproxy_protocol && haproxy_read_hdr(s, localsock, remotesock) != 0) {
+        fatal("unable to read HAProxy protocol header", EX_IOERR);
+    }
+
+    if (remotesock->sa_family == PF_UNSPEC) {
+        r = getpeername(s, remotesock, &salen);
+    }
+
+    if (r == 0 &&
         (remotesock->sa_family == AF_INET ||
          remotesock->sa_family == AF_INET6)) {
         /* connected to an internet socket */
@@ -1060,7 +999,12 @@ EXPORTED const char *get_clienthost(int s, const char **localip, const char **re
         buf_printf(&clientbuf, "[%s]", hbuf);
 
         salen = sizeof(struct sockaddr_storage);
-        if (getsockname(s, localsock, &salen) == 0) {
+
+        if (localsock->sa_family == PF_UNSPEC) {
+            r = getsockname(s, localsock, &salen);
+        }
+
+        if (r == 0) {
             /* set the ip addresses here */
             if (iptostring(localsock, salen,
                           lipbuf, sizeof(lipbuf)) == 0) {
