@@ -159,13 +159,13 @@ static const struct prop_entry notify_props[] = {
       propfind_fromdb, proppatch_todb, NULL },
     { "getcontentlanguage", NS_DAV,
       PROP_ALLPROP | PROP_RESOURCE,
-      propfind_fromhdr, NULL, "Content-Language" },
+      propfind_fromhdr, NULL, (void *) "Content-Language" },
     { "getcontentlength", NS_DAV,
       PROP_ALLPROP | PROP_COLLECTION | PROP_RESOURCE,
       propfind_getlength, NULL, NULL },
     { "getcontenttype", NS_DAV,
       PROP_ALLPROP | PROP_RESOURCE,
-      propfind_fromhdr, NULL, "Content-Type" },
+      propfind_fromhdr, NULL, (void *) "Content-Type" },
     { "getetag", NS_DAV,
       PROP_ALLPROP | PROP_COLLECTION | PROP_RESOURCE,
       propfind_getetag, NULL, NULL },
@@ -218,7 +218,7 @@ static const struct prop_entry notify_props[] = {
     /* WebDAV Sync (RFC 6578) properties */
     { "sync-token", NS_DAV,
       PROP_COLLECTION,
-      propfind_sync_token, NULL, SYNC_TOKEN_URL_SCHEME },
+      propfind_sync_token, NULL, (void *) SYNC_TOKEN_URL_SCHEME },
 
     /* WebDAV Notifications (draft-pot-webdav-notifications) properties */
     { "notificationtype", NS_DAV,
@@ -228,12 +228,12 @@ static const struct prop_entry notify_props[] = {
     /* Backwards compatibility with Apple notifications clients */
     { "notificationtype", NS_CS,
       PROP_RESOURCE,
-      propfind_notifytype, NULL, "calendarserver-sharing" },
+      propfind_notifytype, NULL, (void *) "calendarserver-sharing" },
 
     /* Apple Calendar Server properties */
     { "getctag", NS_CS,
       PROP_ALLPROP | PROP_COLLECTION,
-      propfind_sync_token, NULL, "" },
+      propfind_sync_token, NULL, (void *) "" },
 
     { NULL, 0, 0, NULL, NULL, NULL }
 };
@@ -318,7 +318,7 @@ static void my_dav_init(struct buf *serverinfo __attribute__((unused)))
 }
 
 
-int dav_lookup_notify_collection(const char *userid, mbentry_t **mbentry)
+int dav_lookup_notify_collection(const char *userid, mbentry_t **mbentryp)
 {
     mbname_t *mbname;
     const char *notifyname;
@@ -341,22 +341,22 @@ int dav_lookup_notify_collection(const char *userid, mbentry_t **mbentry)
 
     /* Locate the mailbox */
     notifyname = mbname_intname(mbname);
-    r = proxy_mlookup(notifyname, mbentry, NULL, NULL);
+    r = proxy_mlookup(notifyname, mbentryp, NULL, NULL);
     if (r == IMAP_MAILBOX_NONEXISTENT) {
         /* Find location of INBOX */
         char *inboxname = mboxname_user_mbox(userid, NULL);
 
-        int r1 = proxy_mlookup(inboxname, mbentry, NULL, NULL);
+        int r1 = proxy_mlookup(inboxname, mbentryp, NULL, NULL);
         free(inboxname);
         if (r1 == IMAP_MAILBOX_NONEXISTENT) {
             r = IMAP_INVALID_USER;
             goto done;
         }
 
-        mboxlist_entry_free(mbentry);
-        *mbentry = mboxlist_entry_create();
-        (*mbentry)->name = xstrdup(notifyname);
-        (*mbentry)->mbtype = MBTYPE_COLLECTION;
+        mboxlist_entry_free(mbentryp);
+        *mbentryp = mboxlist_entry_create();
+        (*mbentryp)->name = xstrdup(notifyname);
+        (*mbentryp)->mbtype = MBTYPE_COLLECTION;
     }
 
   done:
@@ -365,15 +365,15 @@ int dav_lookup_notify_collection(const char *userid, mbentry_t **mbentry)
     return r;
 }
 
-static int _create_notify_collection(const char *userid, struct mailbox **mailbox)
+static int _create_notify_collection(const char *userid, mbentry_t **mbentryp) 
 {
     /* lock the namespace lock and try again */
     struct mboxlock *namespacelock = user_namespacelock(userid);
 
-    mbentry_t *mbentry = NULL;
-    int r = dav_lookup_notify_collection(userid, &mbentry);
+    int r = dav_lookup_notify_collection(userid, mbentryp);
 
     if (r == IMAP_MAILBOX_NONEXISTENT) {
+        mbentry_t *mbentry = *mbentryp;
         if (!mbentry) goto done;
         else if (mbentry->server) {
             proxy_findserver(mbentry->server, &http_protocol, httpd_userid,
@@ -383,44 +383,35 @@ static int _create_notify_collection(const char *userid, struct mailbox **mailbo
 
         r = mboxlist_createmailbox(mbentry, 0/*options*/, 0/*highestmodseq*/,
                                    1/*isadmin*/, userid, NULL/*authstate*/,
-                                   0/*flags*/, mailbox);
-        /* we lost the race, that's OK */
-        if (r == IMAP_MAILBOX_LOCKED) r = 0;
-        if (r) syslog(LOG_ERR, "IOERROR: failed to create %s (%s)",
+                                   0/*flags*/, NULL);
+
+        if (r) xsyslog(LOG_ERR, "IOERROR: failed to create notify collection",
+                      "mailbox=<%s> error=<%s>",
                       mbentry->name, error_message(r));
     }
-    else if (!r && mailbox) {
-        /* Open mailbox for writing */
-        r = mailbox_open_iwl(mbentry->name, mailbox);
-        if (r) {
-            syslog(LOG_ERR, "mailbox_open_iwl(%s) failed: %s",
-                   mbentry->name, error_message(r));
-        }
-    }
 
  done:
     mboxname_release(&namespacelock);
-    mboxlist_entry_free(&mbentry);
     return r;
 }
 
-static int create_notify_collection(const char *userid, struct mailbox **mailbox)
+static int create_notify_collection(const char *userid, struct mailbox **mailboxp)
 {
     /* notifications collection */
     mbentry_t *mbentry = NULL;
     int r = dav_lookup_notify_collection(userid, &mbentry);
-    if (r) {
+
+    if (r == IMAP_MAILBOX_NONEXISTENT) {
         mboxlist_entry_free(&mbentry);
-        return _create_notify_collection(userid, mailbox);
+        r = _create_notify_collection(userid, &mbentry);
     }
 
-    if (mailbox) {
+    if (!r && mailboxp) {
         /* Open mailbox for writing */
-        r = mailbox_open_iwl(mbentry->name, mailbox);
-        if (r) {
-            syslog(LOG_ERR, "mailbox_open_iwl(%s) failed: %s",
-                   mbentry->name, error_message(r));
-        }
+        r = mailbox_open_iwl(mbentry->name, mailboxp);
+        if (r) xsyslog(LOG_ERR, "IOERROR: failed to open notify collection",
+                       "mailbox=<%s> error=<%s>",
+                       mbentry->name, error_message(r));
     }
 
     mboxlist_entry_free(&mbentry);
@@ -507,7 +498,7 @@ static int notify_get(struct transaction_t *txn, struct mailbox *mailbox,
        from application/davnotification+xml to application/xml */
 
     /* Parse dlist representing notification type, and data */
-    dlist_parsemap(&dl, 1, 0, wdata->filename, strlen(wdata->filename));
+    dlist_parsemap(&dl, 1, wdata->filename, strlen(wdata->filename));
     dlist_getatom(dl, "T", &type_str);
     dlist_getlist(dl, "D", &al);
 
@@ -889,7 +880,7 @@ done:
     return r;
 }
 
-HIDDEN int dav_send_notification(xmlDocPtr doc, struct dlist *extradata,
+static int dav_send_notification(xmlDocPtr doc, struct dlist *extradata,
                                  const char *userid, const char *resource)
 {
     struct mailbox *mailbox = NULL;
@@ -961,6 +952,44 @@ HIDDEN int dav_send_notification(xmlDocPtr doc, struct dlist *extradata,
     return r;
 }
 
+struct scheduled_notification {
+    xmlDocPtr doc;
+    struct dlist *extradata;
+    char *userid;
+    char *resource;
+    struct scheduled_notification *next;
+};
+
+static struct scheduled_notification *scheduled_notifications = NULL;
+
+HIDDEN int dav_schedule_notification(xmlDocPtr doc, struct dlist *extradata,
+                                     const char *userid, const char *resource)
+{
+    struct scheduled_notification *new = xzmalloc(sizeof(struct scheduled_notification));
+    new->doc = xmlCopyDoc(doc, 1);
+    new->extradata = extradata;  // not a copy, eaten by dav_send_notification
+    new->userid = xstrdupnull(userid);
+    new->resource = xstrdupnull(resource);
+    new->next = scheduled_notifications;
+    scheduled_notifications = new;
+    return 0;
+}
+
+HIDDEN void dav_run_notifications()
+{
+    struct scheduled_notification *next = NULL;
+    struct scheduled_notification *item;
+    for (item = scheduled_notifications; item; item = next) {
+        next = item->next;
+        dav_send_notification(item->doc, item->extradata, item->userid, item->resource);
+	xmlFreeDoc(item->doc);
+	free(item->userid);
+	free(item->resource);
+	free(item);
+    }
+    scheduled_notifications = NULL;
+}
+
 
 /* Perform a POST request on a WebDAV notification resource */
 HIDDEN int notify_post(struct transaction_t *txn)
@@ -1079,7 +1108,7 @@ HIDDEN int notify_post(struct transaction_t *txn)
     }
 
     /* Parse dlist representing notification type, and data */
-    dlist_parsemap(&dl, 1, 0, wdata->filename, strlen(wdata->filename));
+    dlist_parsemap(&dl, 1, wdata->filename, strlen(wdata->filename));
     dlist_getatom(dl, "T", &type_str);
     if (strcmp(type_str, SHARE_INVITE_NOTIFICATION)) {
         ret = HTTP_NOT_ALLOWED;
@@ -1091,7 +1120,7 @@ HIDDEN int notify_post(struct transaction_t *txn)
 
     /* [Un]subscribe */
     r = mboxlist_changesub(mboxname, txn->req_tgt.userid,
-                           httpd_authstate, add, 0, 0);
+                           httpd_authstate, add, 0, 0, 0);
     if (r) {
         syslog(LOG_ERR, "mboxlist_changesub(%s, %s) failed: %s",
                mboxname, txn->req_tgt.userid, error_message(r));
@@ -1694,7 +1723,7 @@ static int propfind_notifytype(const xmlChar *name, xmlNsPtr ns,
                         name, ns, NULL, 0);
 
     /* Parse dlist representing notification type, namespace, and attributes */
-    dlist_parsemap(&dl, 1, 0, wdata->filename, strlen(wdata->filename));
+    dlist_parsemap(&dl, 1, wdata->filename, strlen(wdata->filename));
     dlist_getatom(dl, "T", &type);
     dlist_getatom(dl, "NS", &ns_href);
     dlist_getlist(dl, "A", &al);
@@ -2115,8 +2144,8 @@ HIDDEN int dav_post_share(struct transaction_t *txn, struct meth_params *pparams
                                strhash(txn->req_tgt.mbentry->name),
                                strhash(userid));
 
-                    r = dav_send_notification(notify->doc, extradata,
-                                              userid, buf_cstring(&resource));
+                    r = dav_schedule_notification(notify->doc, extradata,
+                                                  userid, buf_cstring(&resource));
                 }
 
                 free(userid);
@@ -2134,6 +2163,7 @@ HIDDEN int dav_post_share(struct transaction_t *txn, struct meth_params *pparams
     if (notify) xmlFreeDoc(notify->doc);
     buf_free(&resource);
     mboxname_release(&namespacelock);
+    dav_run_notifications();
 
     return ret;
 }
