@@ -185,6 +185,8 @@ static void jmap_init(struct buf *serverinfo)
     my_jmap_settings.server_capabilities = json_object();
 
     jmap_core_init(&my_jmap_settings);
+    jmap_blob_init(&my_jmap_settings);
+    jmap_quota_init(&my_jmap_settings);
     jmap_mail_init(&my_jmap_settings);
     jmap_mdn_init(&my_jmap_settings);
     jmap_contact_init(&my_jmap_settings);
@@ -196,10 +198,10 @@ static void jmap_init(struct buf *serverinfo)
 #endif
     jmap_admin_init(&my_jmap_settings);
 
-    if (ws_enabled) {
-        jmap_push_poll = config_getduration(IMAPOPT_JMAP_PUSHPOLL, 's');
-        if (jmap_push_poll < 0) jmap_push_poll = 0;
+    jmap_push_poll = config_getduration(IMAPOPT_JMAP_PUSHPOLL, 's');
+    if (jmap_push_poll < 0) jmap_push_poll = 0;
 
+    if (ws_enabled) {
         json_object_set_new(my_jmap_settings.server_capabilities,
                 JMAP_URN_WEBSOCKET,
                 json_pack("{s:s s:b}",
@@ -238,7 +240,8 @@ static void jmap_reset(void)
 
 static void jmap_shutdown(void)
 {
-    free_hash_table(&my_jmap_settings.methods, NULL);
+    free_hash_table(&my_jmap_settings.methods,
+                    (void (*)(void *)) &ptrarray_free);
     json_decref(my_jmap_settings.server_capabilities);
     ptrarray_fini(&my_jmap_settings.getblob_handlers);
     int i;
@@ -318,7 +321,8 @@ static int jmap_parse_path(const char *path, struct request_target_t *tgt,
             tgt->flags = JMAP_ENDPOINT_WS;
             tgt->allow |= ALLOW_CONNECT;
         }
-        else if (!strncmp(tgt->collection,
+        else if (jmap_push_poll &&
+                 !strncmp(tgt->collection,
                           JMAP_EVENTSOURCE_COL, strlen(JMAP_EVENTSOURCE_COL))) {
             tgt->flags = JMAP_ENDPOINT_EVENTSOURCE;
         }
@@ -513,6 +517,12 @@ static int meth_post(struct transaction_t *txn,
         txn->flags.conn = CONN_CLOSE;
     }
 
+    /* Check the size of the request */
+    else if (buf_len(&txn->req_body.payload) >
+        (size_t) my_jmap_settings.limits[MAX_SIZE_REQUEST]) {
+        ret = JMAP_LIMIT_SIZE;
+    }
+
     /* Parse the JSON request */
     else if (!(ret = parse_json_body(txn, &req))) {
         ret = jmap_api(txn, req, &res, &my_jmap_settings);
@@ -601,6 +611,9 @@ static int jmap_getblob_default_handler(jmap_req_t *req,
 
         // binary decode if needed
         int encoding = part->charset_enc & 0xff;
+        if (!encoding && part->encoding) {
+            encoding = encoding_lookupname(part->encoding);
+        }
         base = charset_decode_mimebody(base, len, encoding, &decbuf, &len);
 
         if (!base) {
@@ -630,7 +643,7 @@ static int jmap_getblob_default_handler(jmap_req_t *req,
     }
 
  done:
-    if (mbox) jmap_closembox(req, &mbox);
+    mailbox_close(&mbox);
     if (body) {
         message_free_body(body);
         free(body);
@@ -935,8 +948,7 @@ static int _create_upload_collection(const char *accountid,
                 httpd_userid, mailbox_acl(*mailboxp), newacl);
 
         /* ok, change the mailboxes database */
-        r = mboxlist_sync_setacls(mbentry->name, newacl,
-                                  mailbox_modseq_dirty(*mailboxp));
+        r = mboxlist_setacls(mbentry->name, newacl, mailbox_modseq_dirty(*mailboxp), /*silent*/0);
         if (r) {
             syslog(LOG_ERR, "mboxlist_sync_setacls(%s) failed: %s",
                    mbentry->name, error_message(r));
@@ -1022,6 +1034,7 @@ static int jmap_upload(struct transaction_t *txn)
     const char **hdr;
     time_t now = time(NULL);
     struct appendstate as;
+    char *accountid = NULL;
     char *normalisedtype = NULL;
     int rawmessage = 0;
     struct conversations_state *cstate = NULL;
@@ -1041,14 +1054,16 @@ static int jmap_upload(struct transaction_t *txn)
 
     if (datalen > (size_t) my_jmap_settings.limits[MAX_SIZE_UPLOAD]) {
         txn->error.desc = "JSON upload byte size exceeds maxSizeUpload";
-        return HTTP_CONTENT_TOO_LARGE;
+        ret = HTTP_CONTENT_TOO_LARGE;
+        goto done;
     }
 
     /* Resource must be {accountId}/ with no trailing path */
-    char *accountid = xstrdup(txn->req_tgt.resource);
+    accountid = xstrdup(txn->req_tgt.resource);
     char *slash = strchr(accountid, '/');
     if (!slash || *(slash + 1) != '\0') {
         ret = HTTP_NOT_FOUND;
+        txn->error.desc = "unknown uploadUrl";
         goto done;
     }
     *slash = '\0';
@@ -1058,6 +1073,7 @@ static int jmap_upload(struct transaction_t *txn)
         syslog(LOG_ERR, "jmap_upload: can't open conversations db for %s: %s",
                accountid, error_message(r));
         ret = HTTP_SERVER_ERROR;
+        txn->error.desc = "can't open upload conversations db";
         goto done;
     }
 
@@ -1066,6 +1082,7 @@ static int jmap_upload(struct transaction_t *txn)
         syslog(LOG_ERR, "jmap_upload: can't open upload collection for %s: %s",
                accountid, error_message(r));
         ret = HTTP_NOT_FOUND;
+        txn->error.desc = "can't open upload collection";
         goto done;
     }
 
@@ -1105,6 +1122,8 @@ static int jmap_upload(struct transaction_t *txn)
         // otherwise we gotta clean up and make it an attachment
         if (ftruncate(fileno(f), 0L) < 0 || fseek(f, 0L, SEEK_SET) < 0) {
             syslog(LOG_ERR, "IOERROR: failed to reset file in JMAP upload");
+            ret = HTTP_SERVER_ERROR;
+            goto done;
         }
     }
 
@@ -1152,7 +1171,7 @@ static int jmap_upload(struct transaction_t *txn)
         fprintf(f, "Message-ID: %s\r\n", hdr[0]);
     }
 
-    fprintf(f, "Content-Type: %s\r\n", normalisedtype);
+    fputs("Content-Type: application/octet-stream\r\n", f);
 
     int domain = data_domain(data, datalen);
     switch (domain) {
@@ -1197,8 +1216,15 @@ wrotebody:
         append_abort(&as);
         syslog(LOG_ERR, "append_fromstage(%s) failed: %s",
                mailbox_name(mailbox), error_message(r));
-        ret = HTTP_SERVER_ERROR;
-        txn->error.desc = "append_fromstage() failed";
+        if (r == IMAP_QUOTA_EXCEEDED || r == IMAP_NO_OVERQUOTA) {
+            /* XXX  Should never happen, but DTRT anyways */
+            ret = HTTP_NO_STORAGE;
+            txn->error.desc = "Mailbox is over quota";
+        }
+        else {
+            ret = HTTP_SERVER_ERROR;
+            txn->error.desc = "append_fromstage() failed";
+        }
         goto done;
     }
 
@@ -1224,6 +1250,8 @@ wrotebody:
     json_object_set_new(resp, "expires", json_string(datestr));
     json_object_set_new(resp, "type", json_string(normalisedtype));
 
+    ret = HTTP_CREATED;
+
 done:
     free(normalisedtype);
     free(accountid);
@@ -1250,11 +1278,29 @@ done:
     // checkpoint before replying
     sync_checkpoint(httpd_in);
 
-    /* Output the JSON object */
-    if (resp)
-        ret = json_response(HTTP_CREATED, txn, resp);
+    if (!resp) {
+        /* Create Problem Details Object */
+        const char *type, *limit;
 
-    return ret;
+        if (ret == HTTP_CONTENT_TOO_LARGE) {
+            type = "urn:ietf:params:jmap:error:limit";
+            limit = "maxSizeUpload";
+        }
+        else {
+            type = "about:blank";
+            limit = NULL;
+        }
+
+        resp = json_pack("{s:s s:s* s:s s:i s:s}",
+                         "type", type,
+                         "limit", limit,
+                         "title", error_message(ret) + 4,
+                         "status", atoi(error_message(ret)),
+                         "detail", txn->error.desc);
+    }
+
+    /* Output the JSON object */
+    return json_response(ret, txn, resp);
 }
 
 /* Handle a GET on the session endpoint */
@@ -1269,8 +1315,11 @@ static int jmap_get_session(struct transaction_t *txn)
             json_string(JMAP_BASE_URL JMAP_DOWNLOAD_COL JMAP_DOWNLOAD_TPL));
     json_object_set_new(jsession, "uploadUrl",
             json_string(JMAP_BASE_URL JMAP_UPLOAD_COL JMAP_UPLOAD_TPL));
-    json_object_set_new(jsession, "eventSourceUrl",
-            json_string(JMAP_BASE_URL JMAP_EVENTSOURCE_COL JMAP_EVENTSOURCE_TPL));
+
+    if (jmap_push_poll) {
+        json_object_set_new(jsession, "eventSourceUrl",
+                            json_string(JMAP_BASE_URL JMAP_EVENTSOURCE_COL JMAP_EVENTSOURCE_TPL));
+    }
 
     /* state */
     char *inboxname = mboxname_user_mbox(httpd_userid, NULL);
@@ -1411,8 +1460,16 @@ static int jmap_ws(struct transaction_t *txn, enum wslay_opcode opcode,
     /* Always start with fresh working buffer */
     buf_reset(&txn->buf);
 
-    /* Parse the JSON request */
-    ret = parse_json_body(txn, &req);
+    /* Check the size of the request */
+    if (buf_len(&txn->req_body.payload) >
+        (size_t) my_jmap_settings.limits[MAX_SIZE_REQUEST]) {
+        ret = JMAP_LIMIT_SIZE;
+    }
+    else {
+        /* Parse the JSON request */
+        ret = parse_json_body(txn, &req);
+    }
+
     if (ret) {
         ret = jmap_error_response(txn, ret, &res);
     }
@@ -1423,14 +1480,14 @@ static int jmap_ws(struct transaction_t *txn, enum wslay_opcode opcode,
             /* Process the API request */
             ret = jmap_api(txn, req, &res, &my_jmap_settings);
         }
-        else if (!strcmpsafe(type, "WebSocketPushEnable")) {
+        else if (jmap_push_poll && !strcmpsafe(type, "WebSocketPushEnable")) {
             /* Log request */
             spool_replace_header(xstrdup(":jmap"),
                                  xstrdup("WebSocketPushEnable"), txn->req_hdrs);
             ws_push_enable(txn, req);
             ret = HTTP_NO_CONTENT;
         }
-        else if (!strcmpsafe(type, "WebSocketPushDisable")) {
+        else if (jmap_push_poll && !strcmpsafe(type, "WebSocketPushDisable")) {
             /* Log request */
             spool_replace_header(xstrdup(":jmap"),
                                  xstrdup("WebSocketPushDisable"), txn->req_hdrs);
@@ -1473,7 +1530,7 @@ static int jmap_ws(struct transaction_t *txn, enum wslay_opcode opcode,
                 const char *val = json_string_value(jval);
 
                 if (val &&
-                    strarray_find_case(httpd_log_headers, hdrname, 0) >= 0) {
+                    strarray_contains_case(httpd_log_headers, hdrname)) {
                     buf_printf(logbuf, "; %s=\"%s\"", hdrname, val);
                 }
             }
