@@ -72,9 +72,7 @@
 #define TLS_VERSION      "TLS/1.2"
 
 /* Supported HTML DOCTYPE */
-#define HTML_DOCTYPE \
-    "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" " \
-    "\"http://www.w3.org/TR/html4/loose.dtd\">"
+#define HTML_DOCTYPE "<!DOCTYPE html>"
 
 #define XML_DECLARATION \
     "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
@@ -93,10 +91,8 @@
 
 /* SASL usage based on availability */
 #if defined(SASL_NEED_HTTP) && defined(SASL_HTTP_REQUEST)
-  #define HTTP_DIGEST_MECH "DIGEST-MD5"
   #define SASL_USAGE_FLAGS (SASL_NEED_HTTP | SASL_SUCCESS_DATA)
 #else
-  #define HTTP_DIGEST_MECH NULL  /* not supported by our SASL version */
   #define SASL_USAGE_FLAGS SASL_SUCCESS_DATA
 #endif /* SASL_NEED_HTTP */
 
@@ -185,12 +181,10 @@ struct auth_scheme_t {
 /* Auth scheme identifiers */
 enum {
     AUTH_BASIC        = (1<<0),
-    AUTH_DIGEST       = (1<<1),
-    AUTH_SPNEGO       = (1<<2),
-    AUTH_NTLM         = (1<<3),
-    AUTH_BEARER       = (1<<4),
-    AUTH_SCRAM_SHA1   = (1<<5),
-    AUTH_SCRAM_SHA256 = (1<<6)
+    AUTH_SPNEGO       = (1<<1),
+    AUTH_BEARER       = (1<<2),
+    AUTH_SCRAM_SHA1   = (1<<3),
+    AUTH_SCRAM_SHA256 = (1<<4)
 };
 
 /* Auth scheme flags */
@@ -424,9 +418,10 @@ typedef void (*txn_done_t)(struct transaction_t *txn);
 
 /* HTTP version flags */
 enum {
-    VER_1_0 =           0,
-    VER_1_1 =           1,
-    VER_2 =             2
+    VER_0_9 =           0,
+    VER_1_0 =           1,
+    VER_1_1 =           2,
+    VER_2 =             3,
 };
 
 /* Connection token flags */
@@ -561,7 +556,6 @@ extern struct namespace_t namespace_cgi;
 extern struct buf serverinfo;
 extern ptrarray_t backend_cached;
 extern struct protstream *httpd_in;
-extern struct protstream *httpd_out;
 extern int https;
 extern sasl_conn_t *httpd_saslconn;
 extern int httpd_timeout;
@@ -574,7 +568,7 @@ extern char *httpd_extradomain;
 extern struct auth_state *httpd_authstate;
 extern struct namespace httpd_namespace;
 extern const char *httpd_localip, *httpd_remoteip;
-extern unsigned long config_httpmodules;
+extern uint64_t config_httpmodules;
 extern strarray_t *httpd_log_headers;
 extern char *httpd_altsvc;
 
@@ -591,7 +585,7 @@ extern void free_accept(struct accept *a);
 extern void parse_query_params(struct transaction_t *txn, const char *query);
 extern time_t calc_compile_time(const char *time, const char *date);
 extern const char *http_statusline(unsigned ver, long code);
-extern char *httpdate_gen(char *buf, size_t len, time_t t);
+void httpdate_gen(char *buf, size_t len, time_t t) __attribute__((nonnull));
 extern void simple_hdr(struct transaction_t *txn,
                        const char *name, const char *value, ...)
                       __attribute__((format(printf, 3, 4)));
