@@ -46,6 +46,7 @@
 #include "annotate.h"
 #include "bufarray.h"
 #include "hash.h"
+#include "imparse.h"
 #include "mailbox.h"
 #include "message.h"
 #include "prot.h"
@@ -107,6 +108,8 @@ struct fetchargs {
     struct auth_state *authstate;
     hash_table *cidhash;          /* for XCONVFETCH */
     struct conversations_state *convstate; /* for FETCH_MAILBOXIDS */
+
+    range_t partial;              /* For PARTIAL */
 };
 
 /* Bitmasks for fetchitems */
@@ -204,6 +207,7 @@ enum {
     GETSEARCH_RETURN = 0x02,
     GETSEARCH_CHARSET_FIRST = 0x04,
     GETSEARCH_SOURCE = 0x08,
+    GETSEARCH_MODSEQ = 0x10,
 };
 
 /* Bitmasks for search source options (RFC 7377) */
@@ -224,7 +228,8 @@ enum {
     SEARCH_RETURN_ALL =         (1<<2),
     SEARCH_RETURN_COUNT =       (1<<3),
     SEARCH_RETURN_SAVE =        (1<<4),  /* RFC 5182 */
-    SEARCH_RETURN_RELEVANCY =   (1<<5)   /* RFC 6203 */
+    SEARCH_RETURN_RELEVANCY =   (1<<5),  /* RFC 6203 */
+    SEARCH_RETURN_PARTIAL =     (1<<6),  /* RFC 9394 */
 };
 
 /* Things that may be searched for */
@@ -235,6 +240,7 @@ struct searchargs {
     /* used only during parsing */
     int fuzzy_depth;
     uint64_t maxargssize_mark;
+    unsigned did_objectid : 1;
 
     /* For ESEARCH & XCONVMULTISORT */
     const char *tag;
@@ -245,13 +251,22 @@ struct searchargs {
     int isadmin;
 
     /* For MULTISEARCH */
-    unsigned filter;
-    strarray_t subtree;
-    strarray_t subtree_one;
-    strarray_t mailboxes;
+    struct {
+        unsigned filter;
+        strarray_t subtree;
+        strarray_t subtree_one;
+        strarray_t mailboxes;
+    } multi;
 
     /* For SEARCHRES */
     ptrarray_t result_vars;
+
+    /* For PARTIAL */
+    struct {
+        range_t range;
+        uint32_t start_msgno; /* based on last PARTIAL of same search */
+        uint32_t start_count; /* based on last PARTIAL of same search */
+    } partial;
 };
 
 /* Windowing arguments for the XCONVSORT command */
@@ -308,6 +323,8 @@ enum {
     STATUS_HIGHESTMODSEQ =      (1<<5),  /* RFC 7162 */
     STATUS_SIZE =               (1<<6),  /* RFC 8438 */
     STATUS_MAILBOXID =          (1<<7),  /* RFC 8474 */
+    STATUS_DELETED =            (1<<8),  /* RFC 9051 */
+    STATUS_DELETED_STORAGE =    (1<<9),  /* RFC 9208 */
 
     /* Non-standard */
     STATUS_XCONVEXISTS =        (1<<11),
@@ -320,7 +337,7 @@ enum {
 
 #define STATUS_CONVITEMS (STATUS_XCONVEXISTS|STATUS_XCONVUNSEEN|STATUS_XCONVMODSEQ)
 #define STATUS_MBENTRYITEMS (STATUS_MAILBOXID|STATUS_UIDVALIDITY)
-#define STATUS_INDEXITEMS (STATUS_MESSAGES|STATUS_UIDNEXT|STATUS_SIZE|STATUS_HIGHESTMODSEQ|STATUS_CREATEDMODSEQ|STATUS_MBOPTIONS)
+#define STATUS_INDEXITEMS (STATUS_MESSAGES|STATUS_UIDNEXT|STATUS_SIZE|STATUS_HIGHESTMODSEQ|STATUS_CREATEDMODSEQ|STATUS_MBOPTIONS|STATUS_DELETED|STATUS_DELETED_STORAGE)
 #define STATUS_SEENITEMS (STATUS_RECENT|STATUS_UNSEEN)
 
 struct getmetadata_options {
@@ -340,10 +357,12 @@ struct listargs {
     unsigned ret;               /* Return options */
     const char *ref;            /* Reference name */
     strarray_t pat;             /* Mailbox pattern(s) */
-    const char *scan;           /* SCAN content */
     unsigned statusitems;       /* for RETURN STATUS */
     struct getmetadata_options metaopts; /* for RETURN METADATA */
     strarray_t metaitems;       /* for RETURN METADATA */
+    const char *denormalized;   /* for IMAP4rev2 OLDNAME -
+                                   Denormalized UTF-8 mailbox name from
+                                   SELECT/EXAMINE/CREATE/DELETE/RENAME/APPEND */
 };
 
 /* Value for List command variant */
@@ -376,7 +395,7 @@ enum {
     LIST_RET_SPECIALUSE =       (1<<2),  /* RFC 6154 */
     LIST_RET_STATUS =           (1<<3),  /* RFC 5819 */
     LIST_RET_MYRIGHTS =         (1<<4),  /* RFC 8440 */
-    LIST_RET_METADATA =         (1<<5)   /* draft-murchison-imap-list-metadata */
+    LIST_RET_METADATA =         (1<<5)   /* RFC 9590 */
     /* New options MUST be handled in imapd.c:list_data_remote() */
 };
 
@@ -395,19 +414,28 @@ enum {
     MBOX_ATTRIBUTE_HASCHILDREN =        (1<<7),
     MBOX_ATTRIBUTE_HASNOCHILDREN =      (1<<8),
     MBOX_ATTRIBUTE_CHILDINFO_SUBSCRIBED=(1<<9),
+
+    /* from RFC 5465 */
+    MBOX_ATTRIBUTE_NOACCESS =           (1<<10),
 };
 
+#define MBOX_ATTRIBUTE_CHILDINFO_MASK   (MBOX_ATTRIBUTE_CHILDINFO_SUBSCRIBED)
+
 struct mbox_name_attribute {
     uint32_t flag;   /* MBOX_ATTRIBUTE_* */
     const char *id;  /* string value */
 };
 
 extern const struct mbox_name_attribute mbox_name_attributes[];
+extern const struct mbox_name_attribute mbox_name_childinfo[];
 
 /* Bitmask for client capabilities */
 enum {
     CAPA_CONDSTORE =    (1<<0),  /* RFC 7162 */
-    CAPA_QRESYNC =      (1<<1)   /* RFC 7162 */
+    CAPA_QRESYNC =      (1<<1),  /* RFC 7162 */
+    CAPA_IMAP4REV2 =    (1<<2),  /* RFC 9051 */
+    CAPA_UIDONLY =      (1<<3),  /* RFC 9586 */
+    CAPA_UTF8_ACCEPT =  (1<<4),  /* RFC 6855 */
 };
 
 /* Bitmask for urlfetch params (RFC 5524) */
@@ -419,4 +447,33 @@ enum {
 
 extern struct protstream *imapd_out, *imapd_in;
 
+struct client_behavior_registry {
+    uint32_t did_annotate     : 1;   /* used SETANNOTATION or
+                                        FETCH-ed ANNOTATION */
+    uint32_t did_binary       : 1;   /* fetched BINARY or APPEND literal8  */
+    uint32_t did_catenate     : 1;   /* used CATENATE on APPEND  */
+    uint32_t did_condstore    : 1;   /* gave CONDSTORE on SELECT */
+    uint32_t did_compress     : 1;   /* started COMPRESS */
+    uint32_t did_idle         : 1;   /* used IDLE */
+    uint32_t did_imap4rev2    : 1;   /* used ENABLE IMAP4rev2  */
+    uint32_t did_metadata     : 1;   /* called GETMETADATA or SETMETADATA */
+    uint32_t did_multisearch  : 1;   /* called ESEARCH */
+    uint32_t did_move         : 1;   /* used MOVE */
+    uint32_t did_notify       : 1;   /* used NOTIFY */
+    uint32_t did_objectid     : 1;   /* used STATUS MAILBOXID or
+                                        SEARCH/FETCH EMAILID/THREADID */
+    uint32_t did_partial      : 1;   /* used SEARCH/FETCH PARTIAL */
+    uint32_t did_preview      : 1;   /* fetched PREVIEW */
+    uint32_t did_qresync      : 1;   /* gave QRESYNC on SELECT */
+    uint32_t did_savedate     : 1;   /* fetched SAVEDATE */
+    uint32_t did_searchres    : 1;   /* used SAVE on SEARCH */
+    uint32_t did_replace      : 1;   /* used REPLACE */
+    uint32_t did_uidonly      : 1;   /* used ENABLE UIDONLY  */
+    uint32_t did_unselect     : 1;   /* used UNSELECT */
+    uint32_t did_utf8_accept  : 1;   /* used ENABLE UTF8=ACCEPT  */
+
+    /* non-standard - track for possible deprecation */
+    uint32_t did_xlist        : 1;   /* used XLIST  */
+};
+
 #endif /* INCLUDED_IMAPD_H */
