@@ -42,7 +42,10 @@
 #include <config.h>
 
 #include <ctype.h>
+#include <errno.h>
 #include <limits.h>
+#include <stdint.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sysexits.h>
 
@@ -92,9 +95,8 @@ EXPORTED int getxstring(struct protstream *pin, struct protstream *pout,
                         struct buf *buf, enum getxstring_flags flags)
 {
     int c;
-    int i;
     int isnowait;
-    int len;
+    uint32_t i, len;
     static int lminus = -1;
 
     if (lminus == -1) lminus = config_getswitch(IMAPOPT_LITERALMINUS);
@@ -151,7 +153,7 @@ EXPORTED int getxstring(struct protstream *pin, struct protstream *pout,
         /* Literal */
         isnowait = !pin->isclient;
         buf_reset(buf);
-        c = getint32(pin, &len);
+        c = getuint32(pin, &len);
 
         /* For IMAP, LITERAL+ is only valid from client->server.  For MUPDATE
          * it's valid in either direction.
@@ -162,14 +164,14 @@ EXPORTED int getxstring(struct protstream *pin, struct protstream *pout,
                 /* Fail per RFC 7888, Section 4, choice 2 */
                 fatal(error_message(IMAP_LITERAL_MINUS_TOO_LARGE), EX_PROTOCOL);
             }
-            if (config_maxliteral && len >= 0 && (unsigned) len > config_maxliteral) {
+            if (config_maxliteral && len > config_maxliteral) {
                 /* Fail per RFC 7888, Section 4, choice 2 */
                 fatal(error_message(IMAP_LITERAL_TOO_LARGE), EX_PROTOCOL);
             }
             isnowait++;
             c = prot_getc(pin);
         }
-        if (len == -1 || c != '}') {
+        if (c != '}') {
             buf_cstring(buf);
             if (c != EOF) prot_ungetc(c, pin);
             return EOF;
@@ -188,7 +190,7 @@ EXPORTED int getxstring(struct protstream *pin, struct protstream *pout,
         }
 
         if (!isnowait) {
-            if (config_maxliteral && len >= 0 && (unsigned) len > config_maxliteral) {
+            if (config_maxliteral && len > config_maxliteral) {
                 return IMAP_LITERAL_TOO_LARGE;
             }
 
@@ -278,183 +280,117 @@ fail:
     return EOF;
 }
 
-EXPORTED int getint32(struct protstream *pin, int32_t *num)
+/* can't flag with -1 if there is no number here, so return EOF */
+static int _getint(struct protstream *pin, uint64_t max, uint64_t *num)
 {
-    int32_t result = 0;
     int c = EOF;
-    int gotchar = 0;
+    static struct buf buf = BUF_INITIALIZER;
+
+    *num = 0;
+    buf_reset(&buf);
 
-    /* INT_MAX == 2147483647 */
     while ((c = prot_getc(pin)) != EOF && cyrus_isdigit(c)) {
-        if (result > 214748364 || (result == 214748364 && (c > '7')))
-            fatal("num too big", EX_PROTOCOL);
-        result = result * 10 + c - '0';
-        gotchar = 1;
+        buf_putc(&buf, c);
     }
 
-    if (!gotchar) {
+    if (!buf_len(&buf)) {
         if (c != EOF) prot_ungetc(c, pin);
         return EOF;
     }
 
-    *num = result;
+    errno = 0;
+    *num = strtoull(buf_cstring(&buf), NULL, 10);
+
+    if (errno || *num > max)
+        fatal("num too big", EX_PROTOCOL);
 
     return c;
 }
 
-/* Like getint32() but explicitly signed, i.e. negative numbers
- * are accepted */
-EXPORTED int getsint32(struct protstream *pin, int32_t *num)
+/* Like _getint() but explicitly signed, i.e. negative numbers are accepted */
+static int _getsint(struct protstream *pin, uint64_t max, int64_t *num)
 {
     int c;
     int sgn = 1;
+    uint64_t result;
 
     c = prot_getc(pin);
-    if (c == EOF)
+    if (c == EOF) {
+        *num = 0;
         return EOF;
+    }
 
-    if (c == '-')
+    if (c == '-') {
         sgn = -1;
-    else if (c == '+')
-        sgn = 1;
-    else
+        max++; // min = (-max - 1)
+    }
+    else if (c != '+') {
         prot_ungetc(c, pin);
+    }
+
+    c = _getint(pin, max, &result);
+
+    *num = sgn * (int64_t) result;
+
+    return c;
+}
+
+EXPORTED int getint32(struct protstream *pin, int32_t *num)
+{
+    uint64_t result;
+    int c = _getint(pin, INT32_MAX, &result);
+
+    *num = (int32_t) result;
+
+    return c;
+}
+
+EXPORTED int getsint32(struct protstream *pin, int32_t *num)
+{
+    int64_t result;
+    int c = _getsint(pin, INT32_MAX, &result);
+
+    *num = (int32_t) result;
 
-    c = getint32(pin, num);
-    if (c == EOF)
-        return EOF;
-    /* this is slightly buggy: the number INT_MIN = -2147483648
-     * is a valid signed 32b int but is not accepted */
-    if (sgn < 0)
-        *num = - (*num);
     return c;
 }
 
-/* can't flag with -1 if there is no number here, so return EOF */
 EXPORTED int getuint32(struct protstream *pin, uint32_t *num)
 {
-    uint32_t result = 0;
-    int c = EOF;
-    int gotchar = 0;
+    uint64_t result;
+    int c = _getint(pin, UINT32_MAX, &result);
 
-    /* UINT_MAX == 4294967295U */
-    while ((c = prot_getc(pin)) != EOF && cyrus_isdigit(c)) {
-        if (result > 429496729 || (result == 429496729 && (c > '5')))
-            fatal("num too big", EX_PROTOCOL);
-        result = result * 10 + c - '0';
-        gotchar = 1;
-    }
-
-    if (!gotchar) {
-        if (c != EOF) prot_ungetc(c, pin);
-        return EOF;
-    }
-
-    *num = result;
+    *num = (uint32_t) result;
 
     return c;
 }
 
 EXPORTED int getint64(struct protstream *pin, int64_t *num)
 {
-    int64_t result = 0;
-    int c = EOF;
-    int gotchar = 0;
+    uint64_t result;
+    int c = _getint(pin, INT64_MAX, &result);
 
-    /* LLONG_MAX == 9223372036854775807LL */
-    while ((c = prot_getc(pin)) != EOF && cyrus_isdigit(c)) {
-        if (result > 922337203685477580LL || (result == 922337203685477580LL && (c > '7')))
-            fatal("num too big", EX_PROTOCOL);
-        result = result * 10 + c - '0';
-        gotchar = 1;
-    }
-
-    if (!gotchar) {
-        if (c != EOF) prot_ungetc(c, pin);
-        return EOF;
-    }
-
-    *num = result;
+    *num = (int64_t) result;
 
     return c;
 }
 
-/* Like getint64() but explicitly signed, i.e. negative numbers
- * are accepted */
 EXPORTED int getsint64(struct protstream *pin, int64_t *num)
 {
-    int c;
-    int sgn = 1;
-
-    c = prot_getc(pin);
-    if (c == EOF)
-        return EOF;
-
-    if (c == '-')
-        sgn = -1;
-    else if (c == '+')
-        sgn = 1;
-    else
-        prot_ungetc(c, pin);
-
-    c = getint64(pin, num);
-    if (c == EOF)
-        return EOF;
-    /* this is slightly buggy: the number LLONG_MIN == -9223372036854775808LL
-     * is a valid signed 64b int but is not accepted */
-    if (sgn < 0)
-        *num = - (*num);
-    return c;
+    return _getsint(pin, INT64_MAX, num);
 }
 
-/* can't flag with -1 if there is no number here, so return EOF */
 EXPORTED int getuint64(struct protstream *pin, uint64_t *num)
 {
-    uint64_t result = 0;
-    int c = EOF;
-    int gotchar = 0;
-
-    /* ULLONG_MAX == 18446744073709551615ULL */
-    while ((c = prot_getc(pin)) != EOF && cyrus_isdigit(c)) {
-        if (result > 1844674407370955161ULL || (result == 1844674407370955161ULL && (c > '5')))
-            fatal("num too big", EX_PROTOCOL);
-        result = result * 10 + c - '0';
-        gotchar = 1;
-    }
-
-    if (!gotchar) {
-        if (c != EOF) prot_ungetc(c, pin);
-        return EOF;
-    }
-
-    *num = result;
-
-    return c;
+    return _getint(pin, UINT64_MAX, num);
 }
 
-/* This would call getuint64() if
- * all were right with the world */
 EXPORTED int getmodseq(struct protstream *pin, modseq_t *num)
 {
-    int c = EOF;
-    unsigned int i = 0;
-    char buf[32];
-    int gotchar = 0;
+    uint64_t result;
+    int c = _getint(pin, ULLONG_MAX, &result);
 
-    while (i < sizeof(buf) &&
-           (c = prot_getc(pin)) != EOF &&
-           cyrus_isdigit(c)) {
-        buf[i++] = c;
-        gotchar = 1;
-    }
-
-    if (!gotchar || i == sizeof(buf)) {
-        if (c != EOF) prot_ungetc(c, pin);
-        return EOF;
-    }
-
-    buf[i] = '\0';
-    *num = strtoull(buf, NULL, 10);
+    *num = (modseq_t) result;
 
     return c;
 }
@@ -652,12 +588,12 @@ EXPORTED int get_search_source_opts(struct protstream *pin,
     static struct buf opt;
 
     /* Client specified source opts, so clear default of "selected" */
-    searchargs->filter = 0;
+    searchargs->multi.filter = 0;
 
     c = prot_getc(pin);
     if (c != '(') {
         prot_printf(pout,
-                    "%s BAD Missing source options in Search\r\n",
+                    "%s BAD Missing source options in Esearch\r\n",
                     searchargs->tag);
         goto bad;
     }
@@ -668,35 +604,35 @@ EXPORTED int get_search_source_opts(struct protstream *pin,
 
         lcase(opt.s);
         if (!strcmp(opt.s, "selected")) {
-            searchargs->filter |= SEARCH_SOURCE_SELECTED;
+            searchargs->multi.filter |= SEARCH_SOURCE_SELECTED;
         }
         else if (!strcmp(opt.s, "inboxes")) {
-            searchargs->filter |= SEARCH_SOURCE_INBOXES;
+            searchargs->multi.filter |= SEARCH_SOURCE_INBOXES;
         }
         else if (!strcmp(opt.s, "personal")) {
-            searchargs->filter |= SEARCH_SOURCE_PERSONAL;
+            searchargs->multi.filter |= SEARCH_SOURCE_PERSONAL;
         }
         else if (!strcmp(opt.s, "subscribed")) {
-            searchargs->filter |= SEARCH_SOURCE_SUBSCRIBED;
+            searchargs->multi.filter |= SEARCH_SOURCE_SUBSCRIBED;
         }
         else if (!strcmp(opt.s, "subtree")) {
-            searchargs->filter |= SEARCH_SOURCE_SUBTREE;
-            c = get_search_source_mboxes(pin, pout,
-                                         searchargs, &searchargs->subtree);
+            searchargs->multi.filter |= SEARCH_SOURCE_SUBTREE;
+            c = get_search_source_mboxes(pin, pout, searchargs,
+                                         &searchargs->multi.subtree);
         }
         else if (!strcmp(opt.s, "subtree-one")) {
-            searchargs->filter |= SEARCH_SOURCE_SUBTREE_ONE;
-            c = get_search_source_mboxes(pin, pout,
-                                         searchargs, &searchargs->subtree_one);
+            searchargs->multi.filter |= SEARCH_SOURCE_SUBTREE_ONE;
+            c = get_search_source_mboxes(pin, pout, searchargs,
+                                         &searchargs->multi.subtree_one);
         }
         else if (!strcmp(opt.s, "mailboxes")) {
-            searchargs->filter |= SEARCH_SOURCE_MAILBOXES;
-            c = get_search_source_mboxes(pin, pout,
-                                         searchargs, &searchargs->mailboxes);
+            searchargs->multi.filter |= SEARCH_SOURCE_MAILBOXES;
+            c = get_search_source_mboxes(pin, pout, searchargs,
+                                         &searchargs->multi.mailboxes);
         }
         else {
             prot_printf(pout,
-                        "%s BAD Invalid Search source option %s\r\n",
+                        "%s BAD Invalid Esearch source option %s\r\n",
                         searchargs->tag, opt.s);
             goto bad;
         }
@@ -706,7 +642,13 @@ EXPORTED int get_search_source_opts(struct protstream *pin,
     if (c == IMAP_LITERAL_TOO_LARGE) return c;
     if (c != ')') {
         prot_printf(pout,
-                    "%s BAD Missing close parenthesis in Search\r\n",
+                    "%s BAD Missing close parenthesis in Esearch\r\n",
+                    searchargs->tag);
+        goto bad;
+    }
+    else if (!searchargs->multi.filter) {
+        prot_printf(pout,
+                    "%s BAD Missing source mailbox(es) in Esearch\r\n",
                     searchargs->tag);
         goto bad;
     }
@@ -728,7 +670,7 @@ EXPORTED int get_search_return_opts(struct protstream *pin,
                                     struct searchargs *searchargs)
 {
     int c;
-    static struct buf opt;
+    static struct buf opt, arg;
 
     c = prot_getc(pin);
     if (c != '(') {
@@ -760,6 +702,21 @@ EXPORTED int get_search_return_opts(struct protstream *pin,
         else if (!strcmp(opt.s, "relevancy")) { /* RFC 6203 */
             searchargs->returnopts |= SEARCH_RETURN_RELEVANCY;
         }
+        else if (!strcmp(opt.s, "partial")) {   /* RFC 9394 */
+            int r = -1;
+
+            if (c == ' ') {
+                c = getword(pin, &arg);
+                r = imparse_range(arg.s, &searchargs->partial.range);
+            }
+            if (r) {
+                prot_printf(pout, "%s BAD Invalid range in Search\r\n",
+                            searchargs->tag);
+                goto bad;
+            }
+
+            searchargs->returnopts |= SEARCH_RETURN_PARTIAL;
+        }
         else {
             prot_printf(pout,
                         "%s BAD Invalid Search return option %s\r\n",
@@ -774,7 +731,7 @@ EXPORTED int get_search_return_opts(struct protstream *pin,
 
     } while (c == ' ');
 
-    if (!(searchargs->returnopts & ~(SEARCH_RETURN_SAVE|SEARCH_RETURN_RELEVANCY))) {
+    if (!(searchargs->returnopts & ~(SEARCH_RETURN_SAVE | SEARCH_RETURN_RELEVANCY))) {
         /* RFC 4731:
          * If the list of result options is empty, that requests the server to
          * return an ESEARCH response instead of the SEARCH response.  This is
@@ -782,6 +739,18 @@ EXPORTED int get_search_return_opts(struct protstream *pin,
          */
         searchargs->returnopts |= SEARCH_RETURN_ALL;
     }
+    else if ((searchargs->returnopts & SEARCH_RETURN_ALL) &&
+             (searchargs->returnopts & SEARCH_RETURN_PARTIAL)) {
+        /* RFC 9394, Section 3.1:
+         * A single command MUST NOT contain more than one PARTIAL or ALL
+         * search return option; that is, either one PARTIAL, one ALL,
+         * or neither PARTIAL nor ALL is allowed.
+         */
+        prot_printf(pout,
+                    "%s BAD Invalid return options in Search\r\n",
+                    searchargs->tag);
+        goto bad;
+    }
 
     if (c != ')') {
         prot_printf(pout,
@@ -978,6 +947,10 @@ static int get_search_criterion(struct protstream *pin,
         charset_free(&base->charset);
         base->charset = charset_lookupname(arg.s);
         if (base->charset == CHARSET_UNKNOWN_CHARSET) goto badcharset;
+        if ((client_capa & CAPA_UTF8_ACCEPT) &&
+            strcmp("utf-8", charset_canon_name(base->charset))) {
+            goto utf8accept;
+        }
         base->state &= ~GETSEARCH_CHARSET_FIRST;
     }
 
@@ -1161,6 +1134,8 @@ static int get_search_criterion(struct protstream *pin,
             c = getastring(pin, pout, &arg);
             if (c <= EOF) goto missingarg;
             bytestring_match(parent, arg.s, criteria.s, base);
+
+            base->did_objectid = 1;
         }
         else goto badcri;
         break;
@@ -1237,7 +1212,7 @@ static int get_search_criterion(struct protstream *pin,
     case 'l':
         if (!strcmp(criteria.s, "larger")) {            /* RFC 3501 */
             if (c != ' ') goto missingarg;
-            c = getint32(pin, (int32_t *)&u);
+            c = getuint32(pin, &u);
             if (c == EOF) goto badnumber;
             e = search_expr_new(parent, SEOP_GT);
             e->attr = search_attr_find("size");
@@ -1297,23 +1272,19 @@ static int get_search_criterion(struct protstream *pin,
             indexflag_match(parent, MESSAGE_RECENT, /*not*/1);
         }
         else if (!strcmp(criteria.s, "older")) {    /* RFC 5032 */
-#if SIZEOF_TIME_T >= 8
-            int64_t uu;
-#endif
             if (c != ' ') goto missingarg;
 #if SIZEOF_TIME_T >= 8
-            c = getint64(pin, (int64_t *)&uu);
+            uint64_t uu;
+            c = getuint64(pin, &uu);
+            end = now - uu;
 #else
-            c = getint32(pin, (int32_t *)&u);
+            c = getuint32(pin, &u);
+            end = now - u;
 #endif
             if (c == EOF) goto badinterval;
             e = search_expr_new(parent, SEOP_LE);
             e->attr = search_attr_find("internaldate");
-#if SIZEOF_TIME_T >= 8
-            e->value.t = now - uu;
-#else
-            e->value.t = now - u;
-#endif
+            e->value.t = end;
         }
         else if (!strcmp(criteria.s, "on")) {   /* RFC 3501 */
             if (c != ' ') goto missingarg;
@@ -1401,7 +1372,7 @@ static int get_search_criterion(struct protstream *pin,
         }
         else if (!strcmp(criteria.s, "smaller")) {  /* RFC 3501 */
             if (c != ' ') goto missingarg;
-            c = getint32(pin, (int32_t *)&u);
+            c = getuint32(pin, &u);
             if (c == EOF) goto badnumber;
             e = search_expr_new(parent, SEOP_LT);
             e->attr = search_attr_find("size");
@@ -1452,6 +1423,8 @@ static int get_search_criterion(struct protstream *pin,
             c = getastring(pin, pout, &arg);
             if (c <= EOF) goto missingarg;
             bytestring_match(parent, arg.s, criteria.s, base);
+
+            base->did_objectid = 1;
         }
         else goto badcri;
         break;
@@ -1527,23 +1500,19 @@ static int get_search_criterion(struct protstream *pin,
 
     case 'y':
         if (!strcmp(criteria.s, "younger")) {           /* RFC 5032 */
-#if SIZEOF_TIME_T >= 8
-            int64_t uu;
-#endif
             if (c != ' ') goto missingarg;
 #if SIZEOF_TIME_T >= 8
-            c = getint64(pin, (int64_t *)&uu);
+            uint64_t uu;
+            c = getuint64(pin, &uu);
+            start = now - uu;
 #else
-            c = getint32(pin, (int32_t *)&u);
+            c = getuint32(pin, &u);
+            start = now - u;
 #endif
             if (c == EOF) goto badinterval;
             e = search_expr_new(parent, SEOP_GE);
             e->attr = search_attr_find("internaldate");
-#if SIZEOF_TIME_T >= 8
-            e->value.t = now - uu;
-#else
-            e->value.t = now - u;
-#endif
+            e->value.t = start;
         }
         else goto badcri;
         break;
@@ -1614,6 +1583,12 @@ static int get_search_criterion(struct protstream *pin,
                error_message(IMAP_UNRECOGNIZED_CHARSET));
     if (c != EOF) prot_ungetc(c, pin);
     return EOF;
+
+ utf8accept:
+    prot_printf(pout, "%s BAD Charset MUST be UTF-8 when using UTF8=ACCEPT\r\n",
+                base->tag);
+    if (c != EOF) prot_ungetc(c, pin);
+    return EOF;
 }
 
 /*
