@@ -57,10 +57,17 @@
 #include "ptrarray.h"
 #include "strarray.h"
 
+#define JMAP_INT_MAX    9007199254740991LL  /*  2^53-1 */
+#define JMAP_INT_MIN    (-JMAP_INT_MAX)     /* -2^53+1 */
+
 #define JMAP_URN_CORE       "urn:ietf:params:jmap:core"
+#define JMAP_URN_CORE_INFO  "urn:ietf:params:jmap:core:backendInfo"
+#define JMAP_URN_BLOB       "urn:ietf:params:jmap:blob"
+#define JMAP_URN_QUOTA      "urn:ietf:params:jmap:quota"
 #define JMAP_URN_MAIL       "urn:ietf:params:jmap:mail"
 #define JMAP_URN_SUBMISSION "urn:ietf:params:jmap:submission"
 #define JMAP_URN_VACATION   "urn:ietf:params:jmap:vacationresponse"
+#define JMAP_URN_SIEVE      "urn:ietf:params:jmap:sieve"
 #define JMAP_URN_WEBSOCKET  "urn:ietf:params:jmap:websocket"
 #define JMAP_URN_MDN        "urn:ietf:params:jmap:mdn"
 #define JMAP_URN_CONTACTS   "urn:ietf:params:jmap:contacts"
@@ -152,20 +159,7 @@ typedef struct jmap_req {
     double sys_start;
     json_t *perf_details;
 
-    /* The JMAP request keeps its own cache of opened mailboxes,
-     * which can be used by calling jmap_openmbox. If the
-     * force_openmboxrw is set, this causes all following
-     * mailboxes to be opened read-writeable, irrespective if
-     * the caller asked for a read-only lock. This allows to
-     * prevent lock promotion conflicts, in case a cached mailbox
-     * was opened read-only by a helper but it now asked to be
-     * locked exclusively. Since the mailbox lock does not
-     * support lock promition, this would currently abort with
-     * an error. */
-    int force_openmbox_rw;
-
     /* Internal state */
-    ptrarray_t *mboxes;
     hash_table *mbstates;
     hash_table *created_ids;
     hash_table *inmemory_blobs;
@@ -246,6 +240,8 @@ extern int jmap_is_using(jmap_req_t *req, const char *capa);
 
 /* Protocol implementations */
 extern void jmap_core_init(jmap_settings_t *settings);
+extern void jmap_blob_init(jmap_settings_t *settings);
+extern void jmap_quota_init(jmap_settings_t *settings);
 extern void jmap_mail_init(jmap_settings_t *settings);
 extern void jmap_mdn_init(jmap_settings_t *settings);
 extern void jmap_contact_init(jmap_settings_t *settings);
@@ -257,6 +253,8 @@ extern void jmap_sieve_init(jmap_settings_t *settings);
 extern void jmap_admin_init(jmap_settings_t *settings);
 
 extern void jmap_core_capabilities(json_t *account_capabilities);
+extern void jmap_blob_capabilities(json_t *account_capabilities);
+extern void jmap_quota_capabilities(json_t *account_capabilities);
 extern void jmap_mail_capabilities(json_t *account_capabilities, int mayCreateTopLevel);
 extern void jmap_emailsubmission_capabilities(json_t *account_capabilities);
 extern void jmap_mdn_capabilities(json_t *account_capabilities);
@@ -278,13 +276,8 @@ extern void jmap_admin_capabilities(json_t *account_capabilities);
 extern void jmap_accounts(json_t *accounts, json_t *primary_accounts);
 
 /* Request-scoped mailbox cache */
-extern int  jmap_openmbox(jmap_req_t *req, const char *name,
-                          struct mailbox **mboxp, int rw);
 extern int jmap_openmbox_by_uniqueid(jmap_req_t *req, const char *id,
                                      struct mailbox **mboxp, int rw);
-extern int  jmap_isopenmbox(jmap_req_t *req, const char *name);
-extern void jmap_closembox(jmap_req_t *req, struct mailbox **mboxp);
-
 extern int jmap_mboxlist_lookup(const char *name,
                                 mbentry_t **entryptr, struct txn **tid);
 
@@ -350,7 +343,9 @@ enum {
     JMAP_PROP_SERVER_SET = (1<<0),
     JMAP_PROP_IMMUTABLE  = (1<<1),
     JMAP_PROP_SKIP_GET   = (1<<2), // skip in Foo/get if not requested by client
-    JMAP_PROP_ALWAYS_GET = (1<<3)  // always include in Foo/get
+    JMAP_PROP_ALWAYS_GET = (1<<3), // always include in Foo/get
+    JMAP_PROP_REJECT_GET = (1<<4), // reject as unknown in Foo/get
+    JMAP_PROP_REJECT_SET = (1<<5)  // reject as unknown in Foo/set
 };
 
 extern const jmap_property_t *jmap_property_find(const char *name,
@@ -443,11 +438,16 @@ extern json_t *jmap_changes_reply(struct jmap_changes *changes);
 struct jmap_copy {
     /* Request arguments */
     const char *from_account_id;
+    const char *if_from_in_state;
+    const char *if_in_state;
     json_t *create;
     int blob_copy;
     int on_success_destroy_original;
+    const char *destroy_from_if_in_state;
 
     /* Response fields */
+    char *old_state;
+    char *new_state;
     json_t *created;
     json_t *not_created;
 };
@@ -599,10 +599,13 @@ extern void jmap_parse_sharewith_patch(json_t *arg, json_t **shareWith);
 
 extern void jmap_mbentry_cache_free(jmap_req_t *req);
 extern const mbentry_t *jmap_mbentry_by_uniqueid(jmap_req_t *req, const char *id);
+extern const mbentry_t *jmap_mbentry_by_uniqueid_all(jmap_req_t *req, const char *id);
 extern mbentry_t *jmap_mbentry_by_uniqueid_copy(jmap_req_t *req, const char *id);
 extern mbentry_t *jmap_mbentry_from_dav(jmap_req_t *req, struct dav_data *dav);
 
 extern int jmap_findmbox_role(jmap_req_t *req, const char *role,
                               const mbentry_t **mbentryptr);
 
+extern void jmap_add_methods(jmap_method_t methods[], jmap_settings_t *settings);
+
 #endif /* JMAP_API_H */
