@@ -115,18 +115,13 @@ static jmap_method_t jmap_backup_methods_nonstandard[] = {
 
 HIDDEN void jmap_backup_init(jmap_settings_t *settings)
 {
-    jmap_method_t *mp;
-    for (mp = jmap_backup_methods_standard; mp->name; mp++) {
-        hash_insert(mp->name, mp, &settings->methods);
-    }
+    jmap_add_methods(jmap_backup_methods_standard, settings);
 
     if (config_getswitch(IMAPOPT_JMAP_NONSTANDARD_EXTENSIONS)) {
         json_object_set_new(settings->server_capabilities,
                             JMAP_BACKUP_EXTENSION, json_object());
 
-        for (mp = jmap_backup_methods_nonstandard; mp->name; mp++) {
-            hash_insert(mp->name, mp, &settings->methods);
-        }
+        jmap_add_methods(jmap_backup_methods_nonstandard, settings);
     }
 
     /* Initialize PRODID value
@@ -375,7 +370,7 @@ static int restore_collection_cb(const mbentry_t *mbentry, void *rock)
         return 0;
     }
 
-    r = jmap_openmbox(rrock->req, mbentry->name, &rrock->mailbox, /*rw*/0);
+    r = mailbox_open_irl(mbentry->name, &rrock->mailbox);
     if (r) {
         syslog(LOG_ERR, "IOERROR: failed to open mailbox %s for reading",
                mbentry->name);
@@ -389,7 +384,7 @@ static int restore_collection_cb(const mbentry_t *mbentry, void *rock)
                mailbox_name(rrock->mailbox), rrock->jrestore->cutoff,
                rrock->mailbox->i.changes_epoch);
 
-        jmap_closembox(rrock->req, &rrock->mailbox);
+        mailbox_close(&rrock->mailbox);
         return HTTP_UNPROCESSABLE;
     }
 
@@ -498,8 +493,8 @@ static int restore_collection_cb(const mbentry_t *mbentry, void *rock)
             /* Close and re-open mailbox (to avoid deadlocks).
                We also do this on initial entry into the loop
                to switch from a read lock to a write lock. */
-            jmap_closembox(rrock->req, &rrock->mailbox);
-            r = jmap_openmbox(rrock->req, mbentry->name, &rrock->mailbox, /*rw*/1);
+            mailbox_close(&rrock->mailbox);
+            r = mailbox_open_iwl(mbentry->name, &rrock->mailbox);
             if (r) {
                 syslog(LOG_ERR, "IOERROR: failed to open mailbox %s for writing",
                        mbentry->name);
@@ -518,7 +513,7 @@ static int restore_collection_cb(const mbentry_t *mbentry, void *rock)
         rrock->deletedmodseq = rrock->mailbox->i.deletedmodseq;
 
     if (!rrock->keep_open)
-        jmap_closembox(rrock->req, &rrock->mailbox);
+        mailbox_close(&rrock->mailbox);
 
     return r;
 }
@@ -777,7 +772,7 @@ static int restore_contact(message_t *recreatemsg, message_t *destroymsg,
                                              vcardproperty_new_prodid(_prodid),
                                              vcardproperty_new_uid(makeuuid()),
                                              vcardproperty_new_fn(buf_cstring(&crock->buf)),
-                                             0);
+                                             NULL);
 
                     crock->group_vcard = gcard;
                 }
@@ -882,7 +877,7 @@ static int restore_addressbook_cb(const mbentry_t *mbentry, void *rock)
 #endif
     crock->group_vcard = NULL;
 
-    jmap_closembox(rrock->req, mailboxp);
+    mailbox_close(mailboxp);
 
     return r;
 }
@@ -992,7 +987,7 @@ static int do_scheduling(jmap_req_t *req,
     caldav_get_schedule_addresses(req->txn->req_hdrs, mboxname,
                                   req->userid, schedule_addresses);
 
-    if (strarray_find_case(schedule_addresses, organizer, 0) >= 0) {
+    if (strarray_contains_case(schedule_addresses, organizer)) {
         /* Organizer scheduling object resource */
         sched_request(req->userid, req->userid, schedule_addresses, organizer,
                       oldical, ical, SCHED_MECH_JMAP_RESTORE);
@@ -1008,7 +1003,7 @@ static int do_scheduling(jmap_req_t *req,
                  prop;
                  prop = icalcomponent_get_next_property(comp,
                                                         ICAL_ATTENDEE_PROPERTY)) {
-                const char *addr = icalproperty_get_attendee(prop);
+                const char *addr = icalproperty_get_decoded_calendaraddress(prop);
 
                 if (!addr || strncasecmp(addr, "mailto:", 7) ||
                     strcasecmp(strarray_nth(schedule_addresses, 0), addr+7)) {
@@ -1280,7 +1275,7 @@ static int recreate_ical_resources(const mbentry_t *mbentry,
     jmap_req_t *req = rrock->req;
     int r;
 
-    r = jmap_openmbox(req, mbentry->name, &mailbox, /*rw*/1);
+    r = mailbox_open_iwl(mbentry->name, &mailbox);
     if (r) {
         syslog(LOG_ERR, "IOERROR: failed to open mailbox %s", mbentry->name);
         return r;
@@ -1309,8 +1304,8 @@ static int recreate_ical_resources(const mbentry_t *mbentry,
             message_get_uid((message_t *) nextmsg, &nextuid);
 
             mailbox_iter_done(&iter);
-            jmap_closembox(req, &mailbox);
-            r = jmap_openmbox(req, mbentry->name, &mailbox, /*rw*/1);
+            mailbox_close(&mailbox);
+            r = mailbox_open_iwl(mbentry->name, &mailbox);
             if (r) {
                 syslog(LOG_ERR, "IOERROR: failed to open mailbox %s for writing",
                        mbentry->name);
@@ -1323,7 +1318,7 @@ static int recreate_ical_resources(const mbentry_t *mbentry,
     }
     mailbox_iter_done(&iter);
 
-    jmap_closembox(req, &mailbox);
+    mailbox_close(&mailbox);
 
     return 0;
 }
@@ -1624,7 +1619,7 @@ static int restore_message_list_cb(const mbentry_t *mbentry, void *rock)
         isdestroyed_mbox = 1;
     }
 
-    r = jmap_openmbox(rrock->req, mbentry->name, &mailbox, /*rw*/0);
+    r = mailbox_open_irl(mbentry->name, &mailbox);
     if (r) {
         syslog(LOG_ERR, "IOERROR: failed to open mailbox %s", mbentry->name);
         return r;
@@ -1783,7 +1778,7 @@ static int restore_message_list_cb(const mbentry_t *mbentry, void *rock)
     }
 
     mailbox_iter_done(&iter);
-    jmap_closembox(rrock->req, &mailbox);
+    mailbox_close(&mailbox);
 
     return 0;
 }
@@ -1803,7 +1798,7 @@ static void restore_mailbox_plan_cb(const char *guid __attribute__((unused)),
     struct message_t *message = (struct message_t *) data;
     ptrarray_t *deleted = &message->deleted;
     hash_table *mailboxes = (hash_table *) rock;
-    time_t last_removed;
+    time_t last_removed = 0;
     int i;
 
     if (!message->ignore) ptrarray_sort(deleted, &rmail_cmp);
@@ -2006,8 +2001,8 @@ static void restore_mailbox_cb(const char *mboxname, void *data, void *rock)
     }
 
     if (!r) {
-        r = jmap_openmbox(req, mboxname, &mailbox,
-                          /*rw*/newmailbox == NULL || num_unflag);
+        int rw = (newmailbox == NULL || num_unflag);
+        r = rw ? mailbox_open_iwl(mboxname, &mailbox) : mailbox_open_irl(mboxname, &mailbox);
         if (r) {
             syslog(LOG_ERR, "IOERROR: failed to open mailbox %s: %s",
                    mboxname, error_message(r));
@@ -2044,7 +2039,7 @@ static void restore_mailbox_cb(const char *mboxname, void *data, void *rock)
 
         if (++count % BATCH_SIZE == 0) {
             /* Close and re-open mailboxes (to avoid deadlocks) */
-            jmap_closembox(req, &mailbox);
+            mailbox_close(&mailbox);
 
             if (newmailbox) {
                 mailbox_close(&newmailbox);
@@ -2057,8 +2052,9 @@ static void restore_mailbox_cb(const char *mboxname, void *data, void *rock)
                 }
             }
 
-            r = jmap_openmbox(req, mboxname, &mailbox,
-                              /*rw*/newmailbox == NULL || num_unflag);
+            int rw = (newmailbox == NULL || num_unflag);
+            r = rw ? mailbox_open_iwl(mboxname, &mailbox)
+                   : mailbox_open_irl(mboxname, &mailbox);
             if (r) {
                 syslog(LOG_ERR, "IOERROR: failed to open mailbox %s: %s",
                        mboxname, error_message(r));
@@ -2115,9 +2111,8 @@ static void restore_mailbox_cb(const char *mboxname, void *data, void *rock)
         mailbox_commit(mailbox);
     }
 
-    jmap_closembox(req, &mailbox);
-
   done:
+    mailbox_close(&mailbox);
     mailbox_close(&newmailbox);
     mbname_free(&mbname);
     rrock->result = r;
