@@ -325,8 +325,8 @@ HIDDEN int jmap_email_hasattachment(const struct body *part,
                 !strcmp(part->subtype, "PDF"))) {
         return 1;
     }
-    else if ((!strcmp(part->type, "MESSAGE") &&
-                !strcmp(part->subtype, "RFC822"))) {
+    else if (!strcmp(part->type, "MESSAGE")) {
+        // any message/* is an attachment
         return 1;
     }
     else if ((!strcmp(part->type, "TEXT") &&
@@ -484,6 +484,7 @@ struct matchmime_receiver {
     struct search_text_receiver super;
     struct matchmime *matchmime;
     struct buf buf;
+    enum search_part part;
 };
 
 static int _matchmime_tr_begin_mailbox(search_text_receiver_t *rx __attribute__((unused)),
@@ -524,9 +525,10 @@ static int _matchmime_tr_begin_bodypart(search_text_receiver_t *rx __attribute__
     return 0;
 }
 
-static void _matchmime_tr_begin_part(search_text_receiver_t *rx __attribute__((unused)),
-                                     int part __attribute__((unused)))
+static void _matchmime_tr_begin_part(search_text_receiver_t *rx, enum search_part part)
 {
+    struct matchmime_receiver *tr = (struct matchmime_receiver *) rx;
+    tr->part = part;
 }
 
 static int _matchmime_tr_append_text(search_text_receiver_t *rx,
@@ -552,11 +554,12 @@ static int _matchmime_tr_append_text(search_text_receiver_t *rx,
     return 0;
 }
 
-static void _matchmime_tr_end_part(search_text_receiver_t *rx, int part)
+static void _matchmime_tr_end_part(search_text_receiver_t *rx)
 {
     struct matchmime_receiver *tr = (struct matchmime_receiver *) rx;
-    xapian_dbw_doc_part(tr->matchmime->dbw, &tr->buf, part);
+    xapian_dbw_doc_part(tr->matchmime->dbw, &tr->buf, tr->part);
     buf_reset(&tr->buf);
+    tr->part = SEARCH_PART_NONE;
 }
 
 static void _matchmime_tr_end_bodypart(search_text_receiver_t *rx __attribute__((unused)))
@@ -589,7 +592,7 @@ static int _matchmime_tr_audit_mailbox(search_text_receiver_t *rx __attribute__(
 
 static int _matchmime_tr_index_charset_flags(int base_flags)
 {
-    return base_flags | CHARSET_KEEPCASE;
+    return xapian_charset_flags(base_flags);
 }
 
 static int _matchmime_tr_index_message_format(int format __attribute__((unused)),
@@ -755,7 +758,7 @@ static int _email_matchmime_convkeyword(struct convmatch *convmatch,
     if (!strcasecmp(flag, "\\Seen")) {
         num = 0;
     }
-    else if (!convmatch->cstate->counted_flags) {
+    else if (!convmatch->cstate || !convmatch->cstate->counted_flags) {
         num = -1;
     }
     else {
@@ -916,6 +919,7 @@ static int _email_matchmime_evaluate(json_t *filter,
         int i;
         for (i = 0 ; i < SEARCH_NUM_PARTS ; i++) {
             switch (i) {
+                case SEARCH_PART_ANY:
                 case SEARCH_PART_LISTID:
                 case SEARCH_PART_TYPE:
                 case SEARCH_PART_LANGUAGE:
@@ -1003,10 +1007,22 @@ static int _email_matchmime_evaluate(json_t *filter,
         xapian_query_t *xq = xapian_query_new_match(db, SEARCH_PART_ATTACHMENTBODY, match);
         ptrarray_append(&xqs, MATCHMIME_XQ_OR_MATCHALL(xq));
     }
+    if ((match = json_string_value(json_object_get(filter, "inReplyTo")))) {
+        xapian_query_t *xq = xapian_query_new_match(db, SEARCH_PART_INREPLYTO, match);
+        if (xq) ptrarray_append(&xqs, xq);
+    }
     if ((match = json_string_value(json_object_get(filter, "listId")))) {
         xapian_query_t *xq = xapian_query_new_match(db, SEARCH_PART_LISTID, match);
         if (xq) ptrarray_append(&xqs, xq);
     }
+    if ((match = json_string_value(json_object_get(filter, "messageId")))) {
+        xapian_query_t *xq = xapian_query_new_match(db, SEARCH_PART_MESSAGEID, match);
+        if (xq) ptrarray_append(&xqs, xq);
+    }
+    if ((match = json_string_value(json_object_get(filter, "references")))) {
+        xapian_query_t *xq = xapian_query_new_match(db, SEARCH_PART_REFERENCES, match);
+        if (xq) ptrarray_append(&xqs, xq);
+    }
     if (JNOTNULL(jval = json_object_get(filter, "isHighPriority"))) {
         xapian_query_t *xq = xapian_query_new_match(db, SEARCH_PART_PRIORITY, "1");
         if (xq && !json_boolean_value(jval)) {
@@ -1245,7 +1261,8 @@ static int _matchmime_index_message(matchmime_t *matchmime, json_t *jfilter)
             _matchmime_tr_index_charset_flags,
             _matchmime_tr_index_message_format
         },
-        matchmime, BUF_INITIALIZER
+        matchmime, BUF_INITIALIZER,
+        SEARCH_PART_NONE
     };
 
     /* Determine if we need to extract text from attachments for this
@@ -1297,7 +1314,7 @@ HIDDEN matchmime_t *jmap_email_matchmime_new(const struct buf *mime, json_t **er
     const char *paths[2];
     paths[0] = matchmime->dbpath;
     paths[1] = NULL;
-    r = xapian_dbw_open(paths, &matchmime->dbw, /*mode*/0, /*nosync*/1);
+    r = xapian_dbw_open(paths, &matchmime->dbw, XAPIAN_DBW_NOSYNC);
     if (r) {
         syslog(LOG_ERR, "jmap_matchmime: can't open search backend: %s",
                 error_message(r));
