@@ -127,18 +127,13 @@ static jmap_method_t jmap_emailsubmission_methods_nonstandard[] = {
 
 HIDDEN void jmap_emailsubmission_init(jmap_settings_t *settings)
 {
-    jmap_method_t *mp;
-    for (mp = jmap_emailsubmission_methods_standard; mp->name; mp++) {
-        hash_insert(mp->name, mp, &settings->methods);
-    }
+    jmap_add_methods(jmap_emailsubmission_methods_standard, settings);
 
     json_object_set_new(settings->server_capabilities,
             JMAP_URN_SUBMISSION, json_object());
 
     if (config_getswitch(IMAPOPT_JMAP_NONSTANDARD_EXTENSIONS)) {
-        for (mp = jmap_emailsubmission_methods_nonstandard; mp->name; mp++) {
-            hash_insert(mp->name, mp, &settings->methods);
-        }
+        jmap_add_methods(jmap_emailsubmission_methods_nonstandard, settings);
     }
 }
 
@@ -667,7 +662,7 @@ static void _emailsubmission_create(jmap_req_t *req,
     }
 
     /* Open the mailboxes */
-    r = jmap_openmbox(req, mboxname, &mbox, 1);
+    r = mailbox_open_iwl(mboxname, &mbox);
     if (r) goto done;
 
     /* Load the message */
@@ -793,7 +788,7 @@ static void _emailsubmission_create(jmap_req_t *req,
      * message open. But we don't want to long-lock the
      * mailbox while sending the mail over to a SMTP host */
     msgrecord_unref(&mr);
-    jmap_closembox(req, &mbox);
+    mailbox_close(&mbox);
 
     if (!*sm) {
         /* Open the SMTP connection */
@@ -917,7 +912,7 @@ done:
     if (fd_msg != -1) close(fd_msg);
     if (msg) json_decref(msg);
     if (mr) msgrecord_unref(&mr);
-    if (mbox) jmap_closembox(req, &mbox);
+    mailbox_close(&mbox);
     if (myenvelope) json_decref(myenvelope);
     free(mboxname);
     buf_free(&buf);
@@ -1329,7 +1324,7 @@ static int jmap_emailsubmission_get(jmap_req_t *req)
         goto done;
     }
     else {
-        r = jmap_openmbox(req, mbentry->name, &mbox, 0);
+        r = mailbox_open_irl(mbentry->name, &mbox);
     }
     mboxlist_entry_free(&mbentry);
     if (r) goto done;
@@ -1370,7 +1365,7 @@ static int jmap_emailsubmission_get(jmap_req_t *req)
         mailbox_iter_done(&iter);
     }
 
-    if (mbox) jmap_closembox(req, &mbox);
+    mailbox_close(&mbox);
 
     /* Build response */
     get.state = modseqtoa(jmap_modseq(req, MBTYPE_JMAPSUBMIT,
@@ -1415,11 +1410,13 @@ static int _submission_setargs_parse(jmap_req_t *req,
         else if (JNOTNULL(arg)) r = 0;
     }
 
-    else if (!strcmp(key, "onSuccessDestroyEmail") && JNOTNULL(arg)) {
+    else if (!strcmp(key, "onSuccessDestroyEmail")) {
         // need urn:ietf:params:jmap:mail to destroy emails
         if (!jmap_is_using(req, JMAP_URN_MAIL)) return 0;
-        jmap_parse_strings(arg, parser, "onSuccessDestroyEmail");
-        set->onSuccessDestroy = arg;
+        if (JNOTNULL(arg)) {
+            if (jmap_parse_strings(arg, parser, "onSuccessDestroyEmail"))
+                set->onSuccessDestroy = arg;
+        }
     }
 
     else r = 0;
@@ -1507,7 +1504,7 @@ static int jmap_emailsubmission_set(jmap_req_t *req)
         goto done;
     }
 
-    r = jmap_openmbox(req, mbentry->name, &submbox, 1);
+    r = mailbox_open_iwl(mbentry->name, &submbox);
     assert(submbox);
     mboxlist_entry_free(&mbentry);
     if (r) goto done;
@@ -1643,7 +1640,7 @@ static int jmap_emailsubmission_set(jmap_req_t *req)
     }
 
 done:
-    jmap_closembox(req, &submbox);
+    mailbox_close(&submbox);
     jmap_parser_fini(&parser);
     jmap_set_fini(&set);
     json_decref(success_emailids);
@@ -1680,7 +1677,7 @@ static int jmap_emailsubmission_changes(jmap_req_t *req)
         goto done;
     }
 
-    r = jmap_openmbox(req, mbentry->name, &mbox, 0);
+    r = mailbox_open_irl(mbentry->name, &mbox);
     mboxlist_entry_free(&mbentry);
     if (r) goto done;
 
@@ -1721,7 +1718,7 @@ static int jmap_emailsubmission_changes(jmap_req_t *req)
     }
     mailbox_iter_done(&iter);
 
-    jmap_closembox(req, &mbox);
+    mailbox_close(&mbox);
 
     /* Set new state */
     // XXX - this is wrong!  If we want to do this, we need to sort all the changes by
@@ -1965,21 +1962,21 @@ static int submission_filter_match(void *vf, void *rock)
                 json_string_value(json_object_get(sfrock->submission,
                                                   "identityId"));
 
-            if (strarray_find(f->identityIds, identityId, 0) == -1) return 0;
+            if (!strarray_contains(f->identityIds, identityId)) return 0;
         }
         if (f->emailIds) {
             sfrock->emailId =
                 json_string_value(json_object_get(sfrock->submission,
                                                   "emailId"));
 
-            if (strarray_find(f->emailIds, sfrock->emailId, 0) == -1) return 0;
+            if (!strarray_contains(f->emailIds, sfrock->emailId)) return 0;
         }
         if (f->threadIds) {
             sfrock->threadId =
                 json_string_value(json_object_get(sfrock->submission,
                                                   "threadId"));
 
-            if (strarray_find(f->threadIds, sfrock->threadId, 0) == -1) return 0;
+            if (!strarray_contains(f->threadIds, sfrock->threadId)) return 0;
         }
     }
 
@@ -2057,7 +2054,7 @@ static int sub_sort_compare(const void **vp1, const void **vp2)
     struct sub_match *m1 = (struct sub_match *) *vp1;
     struct sub_match *m2 = (struct sub_match *) *vp2;
     const struct sortcrit *sortcrit = m1->sortcrit;
-    int reverse, ret = 0, i = 0;
+    int reverse = 0, ret = 0, i = 0;
 
     for (i = 0; !ret && sortcrit[i].key != SORT_UID; i++) {
         /* determine sort order from reverse flag bit */
@@ -2145,7 +2142,7 @@ static int jmap_emailsubmission_query(jmap_req_t *req)
         goto done;
     }
 
-    r = jmap_openmbox(req, mbentry->name, &mbox, 0);
+    r = mailbox_open_irl(mbentry->name, &mbox);
     mboxlist_entry_free(&mbentry);
     if (r) goto done;
 
@@ -2204,7 +2201,7 @@ static int jmap_emailsubmission_query(jmap_req_t *req)
     }
     mailbox_iter_done(&iter);
 
-    jmap_closembox(req, &mbox);
+    mailbox_close(&mbox);
 
     /* Sort results */
     if (sortcrit) {
