@@ -113,8 +113,8 @@ HIDDEN void jmap_vacation_init(jmap_settings_t *settings)
     }
 
 #ifdef USE_SIEVE
-    unsigned long config_ext = config_getbitfield(IMAPOPT_SIEVE_EXTENSIONS);
-    unsigned long required =
+    uint64_t config_ext = config_getbitfield(IMAPOPT_SIEVE_EXTENSIONS);
+    uint64_t required =
         IMAP_ENUM_SIEVE_EXTENSIONS_VACATION   |
         IMAP_ENUM_SIEVE_EXTENSIONS_RELATIONAL |
         IMAP_ENUM_SIEVE_EXTENSIONS_DATE;
@@ -124,18 +124,13 @@ HIDDEN void jmap_vacation_init(jmap_settings_t *settings)
 
     if (!sieve_vacation_enabled) return;
 
-    jmap_method_t *mp;
-    for (mp = jmap_vacation_methods_standard; mp->name; mp++) {
-        hash_insert(mp->name, mp, &settings->methods);
-    }
+    jmap_add_methods(jmap_vacation_methods_standard, settings);
 
     json_object_set_new(settings->server_capabilities,
             JMAP_URN_VACATION, json_object());
 
     if (config_getswitch(IMAPOPT_JMAP_NONSTANDARD_EXTENSIONS)) {
-        for (mp = jmap_vacation_methods_nonstandard; mp->name; mp++) {
-            hash_insert(mp->name, mp, &settings->methods);
-        }
+        jmap_add_methods(jmap_vacation_methods_nonstandard, settings);
     }
 }
 
@@ -322,7 +317,7 @@ static int jmap_vacation_get(jmap_req_t *req)
         goto done;
     }
 
-    r = sieve_ensure_folder(req->accountid, &mailbox);
+    r = sieve_ensure_folder(req->accountid, &mailbox, /*silent*/0);
     if (r) goto done;
 
     mailbox_unlock_index(mailbox, NULL);
@@ -561,7 +556,7 @@ static int jmap_vacation_set(struct jmap_req *req)
     jmap_set_parse(req, &parser, vacation_props, NULL, NULL, &set, &jerr);
     if (jerr) goto done;
 
-    r = sieve_ensure_folder(req->accountid, &mailbox);
+    r = sieve_ensure_folder(req->accountid, &mailbox, /*silent*/0);
     if (r) goto done;
 
     db = sievedb_open_userid(req->accountid);
