@@ -79,7 +79,7 @@
  * changes to backend_version() in backend.c.
  */
 #define MAILBOX_MINOR_VERSION       (19) /* read comment above! */
-#define MAILBOX_CACHE_MINOR_VERSION (11)
+#define MAILBOX_CACHE_MINOR_VERSION (13)
 
 #define FNAME_HEADER "/cyrus.header"
 #define FNAME_INDEX "/cyrus.index"
@@ -257,7 +257,7 @@ struct mailbox {
     size_t index_len;   /* mapped size */
 
     int index_locktype; /* 0 = none, 1 = shared, 2 = exclusive */
-    int is_readonly; /* true = open index and cache files readonly */
+    int is_readonly; /* tells us whether the index_fd is opened RW or RO */
 
     ino_t header_file_ino;
     bit32 header_file_crc;
@@ -310,6 +310,12 @@ struct mailbox {
     struct index_change *index_changes;
     uint32_t index_change_alloc;
     uint32_t index_change_count;
+
+    /* refcounting */
+    int refcount;
+    char *lockname;
+    struct mboxlock *namelock;
+    struct mailbox *next;
 };
 
 #define ITER_SKIP_UNLINKED (1<<0)
@@ -582,6 +588,7 @@ extern void mailbox_close(struct mailbox **mailboxptr);
 extern int mailbox_delete(struct mailbox **mailboxptr);
 
 /* reading details */
+extern const struct mboxlist_entry *mailbox_mbentry(const struct mailbox *mailbox);
 extern const char *mailbox_name(const struct mailbox *mailbox);
 extern const char *mailbox_uniqueid(const struct mailbox *mailbox);
 extern const char *mailbox_partition(const struct mailbox *mailbox);
@@ -591,6 +598,9 @@ extern uint32_t mailbox_mbtype(const struct mailbox *mailbox);
 extern modseq_t mailbox_foldermodseq(const struct mailbox *mailbox);
 extern uint32_t mailbox_uidvalidity(const struct mailbox *mailbox);
 
+/* caller must free - tab separated lists of userids */
+extern char *mailbox_visible_users(const struct mailbox *mailbox);
+
 struct caldav_db *mailbox_open_caldav(struct mailbox *mailbox);
 struct carddav_db *mailbox_open_carddav(struct mailbox *mailbox);
 struct webdav_db *mailbox_open_webdav(struct mailbox *mailbox);
@@ -642,10 +652,10 @@ extern int mailbox_lock_index(struct mailbox *mailbox, int locktype);
 extern int mailbox_index_islocked(struct mailbox *mailbox, int write);
 
 extern int mailbox_expunge_cleanup(struct mailbox *mailbox, struct mailbox_iter *iter,
-                                   time_t expunge_mark, unsigned *ndeleted);
+                                   time_t expunge_mark, unsigned *ndeleted, int limit);
 extern int mailbox_expunge(struct mailbox *mailbox, struct mailbox_iter *iter,
                            mailbox_decideproc_t *decideproc, void *deciderock,
-                           unsigned *nexpunged, int event_type);
+                           unsigned *nexpunged, int event_type, int limit);
 extern void mailbox_archive(struct mailbox *mailbox, struct mailbox_iter *iter,
                             mailbox_decideproc_t *decideproc, void *deciderock);
 extern void mailbox_remove_files_from_object_storage(struct mailbox *mailbox, unsigned flags);
@@ -743,11 +753,8 @@ extern int mailbox_delete_dav(struct mailbox *mailbox);
 extern int mailbox_add_sieve(struct mailbox *mailbox);
 extern int mailbox_add_email_alarms(struct mailbox *mailbox);
 
-/* Rename a CID.  Note - this is just one mailbox! */
-extern int mailbox_cid_rename(struct mailbox *mailbox,
-                              conversation_id_t from_cid,
-                              conversation_id_t to_cid);
 extern int mailbox_add_conversations(struct mailbox *mailbox, int silent);
+__attribute__((nonnull))
 extern int mailbox_get_xconvmodseq(struct mailbox *mailbox, modseq_t *);
 extern int mailbox_update_xconvmodseq(struct mailbox *mailbox, modseq_t, int force);
 #define mailbox_has_conversations(m) mailbox_has_conversations_full(m, 0)
