@@ -102,7 +102,7 @@ static strarray_t *excluded_specialuse;
 static int enable_subfolder = 1;
 
 static int enabled_events = 0;
-static unsigned long extra_params;
+static uint64_t extra_params;
 
 static struct mboxevent event_template =
 { 0,
@@ -134,6 +134,7 @@ static struct mboxevent event_template =
     /* 23 */ { EVENT_MBTYPE, "vnd.cmu.mbtype", EVENT_PARAM_STRING, { 0 }, 0 },
     { EVENT_SERVERFQDN, "serverFQDN", EVENT_PARAM_STRING, { 0 }, 0 },
     { EVENT_MAILBOX_ACL, "vnd.cmu.mailboxACL", EVENT_PARAM_STRING, { 0 }, 0 },
+    { EVENT_VISIBLE_USERS, "vnd.cmu.visibleUsers", EVENT_PARAM_STRING, { 0 }, 0 },
     /* 24 */ { EVENT_DAV_FILENAME, "vnd.cmu.davFilename", EVENT_PARAM_STRING, { 0 }, 0 },
     /* 25 */ { EVENT_DAV_UID, "vnd.cmu.davUid", EVENT_PARAM_STRING, { 0 }, 0 },
     /* 26 */ { EVENT_ENVELOPE, "vnd.cmu.envelope", EVENT_PARAM_STRING, { 0 }, 0 },
@@ -230,7 +231,7 @@ static void init_internal() {
 EXPORTED int mboxevent_init(void)
 {
     const char *options;
-    int groups;
+    uint64_t groups;
 
     if (!(notifier = config_getstring(IMAPOPT_EVENT_NOTIFIER)) && !idle_notifier)
         return 0;
@@ -245,7 +246,7 @@ EXPORTED int mboxevent_init(void)
     excluded_specialuse = strarray_split(options, NULL, 0);
 
     /* special meaning to disable event notification on all sub folders */
-    if (strarray_find_case(excluded_specialuse, "ALL", 0) >= 0)
+    if (strarray_contains_case(excluded_specialuse, "ALL"))
         enable_subfolder = 0;
 
     /* get event types's extra parameters */
@@ -321,7 +322,7 @@ static int mboxevent_enabled_for_mailbox(struct mailbox *mailbox)
 
         for (i = 0; i < strarray_size(specialuse) ; i++) {
             const char *attribute = strarray_nth(specialuse, i);
-            if (strarray_find(excluded_specialuse, attribute, 0) >= 0) {
+            if (strarray_contains(excluded_specialuse, attribute)) {
                 enabled = 0;
                 goto done;
             }
@@ -577,6 +578,8 @@ static int mboxevent_expected_param(enum event_type type, enum event_param param
         return (type & MAILBOX_EVENTS);
     case EVENT_MAILBOX_ACL:
         return (type & MAILBOX_EVENTS);
+    case EVENT_VISIBLE_USERS:
+        return (type & MAILBOX_EVENTS);
     case EVENT_QUOTA_MESSAGES:
         return type & QUOTA_EVENTS;
     case EVENT_MESSAGE_CONTENT:
@@ -701,7 +704,7 @@ EXPORTED void mboxevent_notify(struct mboxevent **mboxevents)
         if (event->type == EVENT_FLAGS_SET &&
             event->next &&
             event->next->type == EVENT_FLAGS_CLEAR &&
-            strarray_find_case(&event->next->flagnames, "\\Seen", 0) >= 0) {
+            strarray_contains_case(&event->next->flagnames, "\\Seen")) {
 
             struct mboxevent *other = event->next;
             // swap the outsides first
@@ -827,23 +830,23 @@ EXPORTED void mboxevent_add_flags(struct mboxevent *event, char *flagnames[MAX_U
 
     /* add system flags */
     if (system_flags & FLAG_DELETED) {
-        if (strarray_find_case(excluded_flags, "\\Deleted", 0) < 0)
+        if (!strarray_contains_case(excluded_flags, "\\Deleted"))
             strarray_add_case(&event->flagnames, "\\Deleted");
     }
     if (system_flags & FLAG_ANSWERED) {
-        if (strarray_find_case(excluded_flags, "\\Answered", 0) < 0)
+        if (!strarray_contains_case(excluded_flags, "\\Answered"))
             strarray_add_case(&event->flagnames, "\\Answered");
     }
     if (system_flags & FLAG_FLAGGED) {
-        if (strarray_find_case(excluded_flags, "\\Flagged", 0) < 0)
+        if (!strarray_contains_case(excluded_flags, "\\Flagged"))
             strarray_add_case(&event->flagnames, "\\Flagged");
     }
     if (system_flags & FLAG_DRAFT) {
-        if (strarray_find_case(excluded_flags, "\\Draft", 0) < 0)
+        if (!strarray_contains_case(excluded_flags, "\\Draft"))
             strarray_add_case(&event->flagnames, "\\Draft");
     }
     if (system_flags & FLAG_SEEN) {
-        if (strarray_find_case(excluded_flags, "\\Seen", 0) < 0)
+        if (!strarray_contains_case(excluded_flags, "\\Seen"))
             strarray_add_case(&event->flagnames, "\\Seen");
     }
 
@@ -855,7 +858,7 @@ EXPORTED void mboxevent_add_flags(struct mboxevent *event, char *flagnames[MAX_U
         if (!(flagnames[flag] && (flagmask & (1<<(flag & 31)))))
             continue;
 
-        if (strarray_find_case(excluded_flags, flagnames[flag], 0) < 0)
+        if (!strarray_contains_case(excluded_flags, flagnames[flag]))
             strarray_add_case(&event->flagnames, flagnames[flag]);
     }
 }
@@ -888,7 +891,7 @@ EXPORTED void mboxevent_set_access(struct mboxevent *event,
         return;
     }
 
-    /* all events needs uri parameter */
+    /* all events need uri parameter */
     memset(&imapurl, 0, sizeof(struct imapurl));
     imapurl.server = config_servername;
 
@@ -1590,12 +1593,9 @@ void mboxevent_extract_quota(struct mboxevent *event, const struct quota *quota,
         char *extname = mboxname_to_external(quota->root, &namespace, NULL);
         imapurl.mailbox = extname;
 
-        if (!event->params[EVENT_URI].filled) {
-            struct buf url = BUF_INITIALIZER;
-            imapurl_toURL(&url, &imapurl);
-            FILL_STRING_PARAM(event, EVENT_URI, buf_release(&url));
-        }
-
+        struct buf url = BUF_INITIALIZER;
+        imapurl_toURL(&url, &imapurl);
+        FILL_STRING_PARAM(event, EVENT_URI, buf_release(&url));
         xzfree(extname);
 
         /* Note that userbuf for shared folders is NULL, and xstrdup
@@ -1696,18 +1696,18 @@ EXPORTED void mboxevent_extract_mailbox(struct mboxevent *event,
         event->uidset = NULL;
     }
 
-    /* all events needs uri parameter */
+    /* all events need uri parameter */
     struct buf url = BUF_INITIALIZER;
     imapurl_toURL(&url, &imapurl);
     FILL_STRING_PARAM(event, EVENT_URI, buf_release(&url));
 
-    buf_free(&url);
     free(extname);
 
     FILL_STRING_PARAM(event, EVENT_MBTYPE,
         xstrdup(mboxlist_mbtype_to_string(mailbox_mbtype(mailbox))));
 
     FILL_STRING_PARAM(event, EVENT_MAILBOX_ACL, xstrdup(mailbox_acl(mailbox)));
+    FILL_STRING_PARAM(event, EVENT_VISIBLE_USERS, mailbox_visible_users(mailbox));
 
     /* mailbox related events also require mailboxID */
     if (event->type & MAILBOX_EVENTS) {
@@ -1811,7 +1811,6 @@ void mboxevent_extract_old_mailbox(struct mboxevent *event,
     imapurl_toURL(&url, &imapurl);
     FILL_STRING_PARAM(event, EVENT_OLD_MAILBOX_ID, buf_release(&url));
 
-    buf_free(&url);
     free(extname);
 }
 
