@@ -71,6 +71,7 @@
 #include "util.h"
 #include "auth.h"
 #include "global.h"
+#include "slowio.h"
 #include "tls.h"
 
 #include "imapd.h"
@@ -93,6 +94,8 @@
 /* generated headers are not necessarily in current directory */
 #include "imap/imap_err.h"
 
+#include "master/service.h"
+
 #include "statuscache.h"
 
 #include "iostat.h"
@@ -101,17 +104,6 @@
 #define MAXHOSTNAMELEN 256
 #endif
 
-#ifdef HAVE_KRB
-/* kerberos des is purported to conflict with OpenSSL DES */
-#define DES_DEFS
-#include <krb.h>
-
-/* MIT's kpop authentication kludge */
-char klrealm[REALM_SZ];
-AUTH_DAT kdata;
-#endif /* HAVE_KRB */
-static int kflag = 0;
-
 extern int optind;
 extern char *optarg;
 extern int opterr;
@@ -171,10 +163,15 @@ const int config_need_data = CONFIG_NEED_PARTITION_DATA;
 static struct namespace popd_namespace;
 
 /* PROXY stuff */
-struct backend *backend = NULL;
+static struct backend *backend = NULL;
+
+static const struct tls_alpn_t pop3_alpn_map[] = {
+    { "pop3", NULL, NULL },
+    { "",     NULL, NULL }
+};
 
 static struct protocol_t pop3_protocol =
-{ "pop3", "pop", TYPE_STD,
+{ "pop3", "pop", pop3_alpn_map, TYPE_STD,
   { { { 0, "+OK " },
       { "CAPA", NULL, ".", NULL,
         CAPAF_ONE_PER_LINE,
@@ -198,11 +195,10 @@ static void cmd_auth(char *arg);
 static void cmd_capa(void);
 static void cmd_pass(char *pass);
 static void cmd_user(char *user);
-static void cmd_starttls(int pop3s);
+static void cmd_stls(int pop3s);
 static int blat(int msg, int lines);
 static int openinbox(void);
 static void cmdloop(void);
-static void kpop(void);
 static unsigned parse_msgno(char **ptr);
 static void uidl_msg(uint32_t msgno);
 static int msg_exists_or_err(uint32_t msgno);
@@ -405,14 +401,15 @@ static void popd_reset(void)
     saslprops_reset(&saslprops);
 
     popd_exists = 0;
+
+    slowio_reset();
 }
 
 /*
  * run once when process is forked;
  * MUST NOT exit directly; must return with non-zero error code
  */
-int service_init(int argc __attribute__((unused)),
-                 char **argv __attribute__((unused)),
+int service_init(int argc, char **argv,
                  char **envp __attribute__((unused)))
 {
     int r;
@@ -439,8 +436,12 @@ int service_init(int argc __attribute__((unused)),
 
     mboxevent_setnamespace(&popd_namespace);
 
-    while ((opt = getopt(argc, argv, "skp:")) != EOF) {
+    while ((opt = getopt(argc, argv, "Hsp:")) != EOF) {
         switch(opt) {
+        case 'H': /* expect HAProxy protocol header */
+            haproxy_protocol = 1;
+            break;
+
         case 's': /* pop3s (do starttls right away) */
             pop3s = 1;
             if (!tls_enabled()) {
@@ -450,10 +451,6 @@ int service_init(int argc __attribute__((unused)),
             }
             break;
 
-        case 'k':
-            kflag++;
-            break;
-
         case 'p': /* external protection */
             extprops_ssf = atoi(optarg);
             break;
@@ -537,11 +534,9 @@ int service_main(int argc __attribute__((unused)),
     prot_settimeout(popd_in, popd_timeout);
     prot_setflushonread(popd_in, popd_out);
 
-    if (kflag) kpop();
-
     /* we were connected on pop3s port so we should do
        TLS negotiation immediatly */
-    if (pop3s == 1) cmd_starttls(1);
+    if (pop3s == 1) cmd_stls(1);
 
     /* Create APOP challenge for banner */
     *popd_apop_chal = 0;
@@ -576,9 +571,6 @@ int service_main(int argc __attribute__((unused)),
         mboxevent_free(&mboxevent);
     }
 
-    /* don't bother reusing KPOP connections */
-    if (kflag) shut_down(0);
-
     /* cleanup */
     popd_reset();
 
@@ -605,7 +597,6 @@ void service_abort(int error)
 
 static void usage(void)
 {
-    if (kflag) prometheus_decrement(CYRUS_POP3_ACTIVE_CONNECTIONS);
     prometheus_increment(CYRUS_POP3_SHUTDOWN_TOTAL_STATUS_ERROR);
 
     prot_printf(popd_out, "-ERR usage: pop3d [-C <alt_config>] [-k] [-s]\r\n");
@@ -725,109 +716,12 @@ EXPORTED void fatal(const char* s, int code)
         prot_flush(popd_out);
     }
     syslog(LOG_ERR, "Fatal error: %s", s);
+
+    if (code != EX_PROTOCOL && config_fatals_abort) abort();
+
     shut_down(code);
 }
 
-#ifdef HAVE_KRB
-/* translate IPv4 mapped IPv6 address to IPv4 address */
-#ifdef IN6_IS_ADDR_V4MAPPED
-static void sockaddr_unmapped(struct sockaddr *sa, socklen_t *len)
-{
-    struct sockaddr_in6 *sin6;
-    struct sockaddr_in *sin4;
-    uint32_t addr;
-    int port;
-
-    if (sa->sa_family != AF_INET6)
-        return;
-    sin6 = (struct sockaddr_in6 *)sa;
-    if (!IN6_IS_ADDR_V4MAPPED((&sin6->sin6_addr)))
-        return;
-    sin4 = (struct sockaddr_in *)sa;
-    addr = *(uint32_t *)&sin6->sin6_addr.s6_addr[12];
-    port = sin6->sin6_port;
-    memset(sin4, 0, sizeof(struct sockaddr_in));
-    sin4->sin_addr.s_addr = addr;
-    sin4->sin_port = port;
-    sin4->sin_family = AF_INET;
-#ifdef HAVE_SOCKADDR_SA_LEN
-    sin4->sin_len = sizeof(struct sockaddr_in);
-#endif
-    *len = sizeof(struct sockaddr_in);
-}
-#else
-static void sockaddr_unmapped(struct sockaddr *sa __attribute__((unused)),
-                              socklen_t *len __attribute__((unused)))
-{
-    return;
-}
-#endif
-
-
-/*
- * MIT's kludge of a kpop protocol
- * Client does a krb_sendauth() first thing
- */
-void kpop(void)
-{
-    Key_schedule schedule;
-    KTEXT_ST ticket;
-    char instance[INST_SZ];
-    char version[9];
-    const char *srvtab;
-    int r;
-    socklen_t len;
-
-    if (!popd_haveaddr) {
-        fatal("Cannot get client's IP address", EX_OSERR);
-    }
-
-    srvtab = config_getstring(IMAPOPT_SRVTAB);
-
-    sockaddr_unmapped((struct sockaddr *)&popd_remoteaddr, &len);
-    if (popd_remoteaddr.ss_family != AF_INET) {
-        prot_printf(popd_out,
-                    "-ERR [AUTH] Kerberos authentication failure: %s\r\n",
-                    "not an IPv4 connection");
-        shut_down(0);
-    }
-
-    strcpy(instance, "*");
-    r = krb_recvauth(0L, 0, &ticket, "pop", instance,
-                     (struct sockaddr_in *) &popd_remoteaddr,
-                     (struct sockaddr_in *) NULL,
-                     &kdata, (char*) srvtab, schedule, version);
-
-    if (r) {
-        prot_printf(popd_out, "-ERR [AUTH] Kerberos authentication failure: %s\r\n",
-                    krb_err_txt[r]);
-        syslog(LOG_NOTICE,
-               "badlogin: %s kpop ? %s%s%s@%s %s",
-               popd_clienthost, kdata.pname,
-               kdata.pinst[0] ? "." : "", kdata.pinst,
-               kdata.prealm, krb_err_txt[r]);
-        shut_down(0);
-    }
-
-    r = krb_get_lrealm(klrealm,1);
-    if (r) {
-        prot_printf(popd_out, "-ERR [AUTH] Kerberos failure: %s\r\n",
-                    krb_err_txt[r]);
-        syslog(LOG_NOTICE,
-               "badlogin: %s kpop ? %s%s%s@%s krb_get_lrealm: %s",
-               popd_clienthost, kdata.pname,
-               kdata.pinst[0] ? "." : "", kdata.pinst,
-               kdata.prealm, krb_err_txt[r]);
-        shut_down(0);
-    }
-}
-#else
-void kpop(void)
-{
-    usage();
-}
-#endif
-
 static int expunge_deleted(void)
 {
     struct index_record record;
@@ -1075,7 +969,7 @@ done:
                     /* XXX  discard any input pipelined after STLS */
                     prot_flush(popd_in);
 
-                    cmd_starttls(0);
+                    cmd_stls(0);
                 }
             }
             else {
@@ -1296,7 +1190,7 @@ void uidl_msg(uint32_t msgno)
 }
 
 #ifdef HAVE_SSL
-static void cmd_starttls(int pop3s)
+static void cmd_stls(int pop3s)
 {
     int result;
     char *localip, *remoteip;
@@ -1308,10 +1202,11 @@ static void cmd_starttls(int pop3s)
         return;
     }
 
+    SSL_CTX *ctx = NULL;
     result=tls_init_serverengine("pop3",
                                  5,        /* depth to verify */
                                  !pop3s,   /* can client auth? */
-                                 NULL);
+                                 &ctx);
 
     if (result == -1) {
 
@@ -1343,6 +1238,7 @@ static void cmd_starttls(int pop3s)
                                1, /* write */
                                pop3s ? 180 : popd_timeout,
                                &saslprops,
+                               pop3_alpn_map,
                                &tls_conn);
 
     /* put the iplocalport and ipremoteport back */
@@ -1351,25 +1247,15 @@ static void cmd_starttls(int pop3s)
 
     /* if error */
     if (result==-1) {
-        if (pop3s == 0) {
-            prot_printf(popd_out, "-ERR [SYS/PERM] Starttls failed\r\n");
-            syslog(LOG_NOTICE, "[pop3d] STARTTLS failed: %s", popd_clienthost);
-        } else {
-            syslog(LOG_NOTICE, "pop3s failed: %s", popd_clienthost);
-            shut_down(0);
-        }
-        return;
+        syslog(LOG_NOTICE, "TLS negiation failed: %s", popd_clienthost);
+        shut_down(EX_PROTOCOL);
     }
 
     /* tell SASL about the negotiated layer */
     result = saslprops_set_tls(&saslprops, popd_saslconn);
     if (result != SASL_OK) {
-        syslog(LOG_NOTICE, "saslprops_set_tls() failed: cmd_starttls()");
-        if (pop3s == 0) {
-            fatal("saslprops_set_tls() failed: cmd_starttls()", EX_TEMPFAIL);
-        } else {
-            shut_down(0);
-        }
+        syslog(LOG_NOTICE, "saslprops_set_tls() failed: cmd_stls()");
+        shut_down(EX_TEMPFAIL);
     }
 
     /* tell the prot layer about our new layers */
@@ -1380,9 +1266,9 @@ static void cmd_starttls(int pop3s)
     popd_tls_required = 0;
 }
 #else
-static void cmd_starttls(int pop3s __attribute__((unused)))
+static void cmd_stls(int pop3s __attribute__((unused)))
 {
-    fatal("cmd_starttls() called, but no OpenSSL", EX_SOFTWARE);
+    fatal("cmd_stls() called, but no OpenSSL", EX_SOFTWARE);
 }
 #endif /* HAVE_SSL */
 
@@ -1468,7 +1354,7 @@ static void cmd_user(char *user)
 
     /* possibly disallow USER */
     if (popd_tls_required ||
-        !(kflag || popd_starttls_done || (extprops_ssf > 1) ||
+        !(popd_starttls_done || (extprops_ssf > 1) ||
           config_getswitch(IMAPOPT_ALLOWPLAINTEXT))) {
         prot_printf(popd_out,
                     "-ERR [AUTH] USER command only available under a layer\r\n");
@@ -1509,29 +1395,6 @@ static void cmd_pass(char *pass)
         return;
     }
 
-#ifdef HAVE_KRB
-    if (kflag) {
-        if (strcmp(popd_userid, kdata.pname) != 0 ||
-            kdata.pinst[0] ||
-            strcmp(klrealm, kdata.prealm) != 0) {
-            prot_printf(popd_out, "-ERR [AUTH] Invalid login\r\n");
-            syslog(LOG_NOTICE,
-                   "badlogin: %s kpop %s %s%s%s@%s access denied",
-                   popd_clienthost, popd_userid,
-                   kdata.pname, kdata.pinst[0] ? "." : "",
-                   kdata.pinst, kdata.prealm);
-            return;
-        }
-
-        syslog(LOG_NOTICE, "login: %s %s%s KPOP%s %s SESSIONID=<%s>", popd_clienthost,
-               popd_userid, popd_subfolder ? popd_subfolder : "",
-               popd_starttls_done ? "+TLS" : "", "User logged in", session_id());
-
-        openinbox();
-        return;
-    }
-#endif
-
     if (!strcmp(popd_userid, "anonymous")) {
         if (config_getswitch(IMAPOPT_ALLOWANONYMOUSLOGIN)) {
             pass = beautify_string(pass);
@@ -1648,7 +1511,7 @@ static void cmd_capa(void)
     prot_printf(popd_out, "AUTH-RESP-CODE\r\n");
 
     if (!popd_tls_required && !popd_authstate &&
-        (kflag || popd_starttls_done || (extprops_ssf > 1)
+        (popd_starttls_done || (extprops_ssf > 1)
          || config_getswitch(IMAPOPT_ALLOWPLAINTEXT))) {
         prot_printf(popd_out, "USER\r\n");
     }
@@ -2010,7 +1873,7 @@ int openinbox(void)
         mailbox_unlock_index(popd_mailbox, NULL);
     }
 
-    limits.procname = "pop3d";
+    limits.servicename = config_ident;
     limits.clienthost = popd_clienthost;
     limits.userid = popd_userid;
     if (proc_checklimits(&limits)) {
