@@ -173,14 +173,14 @@ static int sync_log_enabled(const char *channel)
         return 0;       /* entire mechanism is disabled */
     if (!sync_log_suppressed)
         return 1;       /* _suppress() wasn't called */
-    if (unsuppressable && strarray_find(unsuppressable, channel, 0) >= 0)
+    if (unsuppressable && strarray_contains(unsuppressable, channel))
         return 1;       /* channel is unsuppressable */
     return 0;           /* suppressed */
 }
 
 static void sync_log_base(const char *channel, const char *string)
 {
-    int fd;
+    int fd = -1;
     struct stat sbuffile, sbuffd;
     int retries = 0;
     const char *fname;
