@@ -91,6 +91,7 @@
 #include "prot.h"
 #include "quota.h"
 #include "seen.h"
+#include "slowio.h"
 #include "sync_log.h"
 #include "telemetry.h"
 #include "tls.h"
@@ -104,6 +105,8 @@
 /* generated headers are not necessarily in current directory */
 #include "imap/imap_err.h"
 
+#include "master/service.h"
+
 #include "message_guid.h"
 #include "sync_support.h"
 /*#include "cdb.h"*/
@@ -138,8 +141,6 @@ static int sync_sieve_mailbox_enabled = 0;
 static int sync_archive_enabled = 0;
 static struct proc_handle *proc_handle = NULL;
 
-static int opt_force = 0;
-
 /* commands that have specific names */
 static void cmdloop(void);
 static void cmd_authenticate(char *mech, char *resp);
@@ -230,6 +231,8 @@ static void sync_reset(void)
     sync_archive_enabled = 0;
 
     saslprops_reset(&saslprops);
+
+    slowio_reset();
 }
 
 /*
@@ -257,9 +260,6 @@ int service_init(int argc __attribute__((unused)),
         case 'p': /* external protection */
             extprops_ssf = atoi(optarg);
             break;
-        case 'f':
-            opt_force = 1;
-            break;
         default:
             usage();
         }
@@ -467,6 +467,7 @@ EXPORTED void fatal(const char* s, int code)
         prot_flush(sync_out);
     }
     syslog(LOG_ERR, "Fatal error: %s", s);
+    if (code != EX_PROTOCOL && config_fatals_abort) abort();
     shut_down(code);
 }
 
@@ -852,19 +853,20 @@ static void cmd_starttls(void)
                                1, /* write */
                                180, /* 3 minutes */
                                &saslprops,
+                               NULL, /* no ALPN id for csync */
                                &tls_conn);
 
     /* if error */
     if (result==-1) {
-        prot_printf(sync_out, "NO Starttls failed\r\n");
-        syslog(LOG_NOTICE, "STARTTLS failed: %s", sync_clienthost);
-        return;
+        syslog(LOG_NOTICE, "TLS negotiation failed: %s", sync_clienthost);
+        shut_down(EX_PROTOCOL);
     }
 
     /* tell SASL about the negotiated layer */
     result = saslprops_set_tls(&saslprops, sync_saslconn);
     if (result != SASL_OK) {
-        fatal("saslprops_set_tls() failed: cmd_starttls()", EX_TEMPFAIL);
+        syslog(LOG_NOTICE, "saslprops_set_tls() failed: cmd_starttls()");
+        shut_down(EX_TEMPFAIL);
     }
 
     /* tell the prot layer about our new layers */
