@@ -149,6 +149,7 @@ struct sync_folder {
     modseq_t xconvmodseq;
     modseq_t raclmodseq;
     modseq_t foldermodseq;
+    char *groups;
     int ispartial;
     struct quota quota;
     int   mark;
@@ -179,6 +180,7 @@ struct sync_folder *sync_folder_list_add(struct sync_folder_list *l,
                                          modseq_t xconvmodseq,
                                          modseq_t raclmodseq,
                                          modseq_t foldermodseq,
+                                         const char *groups,
                                          int ispartial);
 
 struct sync_folder *sync_folder_lookup(struct sync_folder_list *l,
@@ -476,7 +478,7 @@ const char *sync_restore(struct dlist *kin,
 #define SYNC_FLAG_VERBOSE   (1<<0)
 #define SYNC_FLAG_LOGGING   (1<<1)
 #define SYNC_FLAG_LOCALONLY (1<<2)
-#define SYNC_FLAG_DELETE_REMOTE (1<<3)
+// 3 was DELETE_REMOTE; now we use tombstones
 #define SYNC_FLAG_NO_COPYBACK (1<<4)
 #define SYNC_FLAG_BATCH (1<<5)
 #define SYNC_FLAG_SIEVE_MAILBOX (1<<6)
