@@ -176,7 +176,7 @@ static const char *user_sieve_path_byid(const char *mboxid)
 
 EXPORTED const char *user_sieve_path(const char *inuser)
 {
-    const char *sieve_path;
+    const char *sieve_path = NULL;
     char *user = xstrdupnull(inuser);
     char *p;
 
@@ -198,10 +198,10 @@ EXPORTED const char *user_sieve_path(const char *inuser)
         char *inboxname = mboxname_user_mbox(user, NULL);
         mbentry_t *mbentry = NULL;
 
-        int r = mboxlist_lookup(inboxname, &mbentry, NULL);
+        int r = mboxlist_lookup_allow_all(inboxname, &mbentry, NULL);
         free(inboxname);
 
-        if (r || (mbentry->mbtype & MBTYPE_LEGACY_DIRS)) {
+        if (r || (mbentry->mbtype & MBTYPE_LEGACY_DIRS) || !mbentry->uniqueid) {
             legacy = 1;
         }
         else {
@@ -274,7 +274,7 @@ EXPORTED int user_deletedata(const mbentry_t *mbentry, int wipe_user)
 
     assert(user_isnamespacelocked(userid));
 
-    if (!(mbentry->mbtype & MBTYPE_LEGACY_DIRS)) {
+    if (!(mbentry->mbtype & MBTYPE_LEGACY_DIRS) && mbentry->uniqueid) {
         for (suffixes = user_file_suffixes; *suffixes; suffixes++) {
             strarray_appendm(&paths,
                              mboxid_conf_getpath(mbentry->uniqueid, *suffixes));
@@ -379,7 +379,7 @@ static int user_renamesub(const char *name, void* rock)
         name = newname;
     }
 
-    return mboxlist_changesub(name, rrock->newuser, NULL, 1, 1, 1);
+    return mboxlist_changesub(name, rrock->newuser, NULL, 1, 1, 1, /*silent*/1);
 }
 
 static int user_renamesieve(const char *olduser, const char *newuser)
@@ -633,13 +633,13 @@ EXPORTED char *user_hash_xapian(const char *userid, const char *root)
     char *basedir = NULL;
     int r;
 
-    r = mboxlist_lookup(inboxname, &mbentry, NULL);
+    r = mboxlist_lookup_allow_all(inboxname, &mbentry, NULL);
     if (r) goto out;
 
     mbname = mbname_from_intname(mbentry->name);
     if (!mbname_userid(mbname)) goto out;
 
-    if (mbentry->mbtype & MBTYPE_LEGACY_DIRS) {
+    if (mbentry->mbtype & MBTYPE_LEGACY_DIRS || !mbentry->uniqueid) {
         basedir = user_hash_xapian_byname(mbname, root);
     }
     else {
@@ -693,23 +693,21 @@ EXPORTED char *user_hash_xapian_byid(const char *mboxid, const char *root)
 
 static const char *_namelock_name_from_userid(const char *userid)
 {
-    const char *p;
     static struct buf buf = BUF_INITIALIZER;
-    if (!userid) userid = ""; // no userid == global lock
 
     buf_setcstr(&buf, "*U*");
-
-    for (p = userid; *p; p++) {
-        switch(*p) {
-            case '.':
-                buf_putc(&buf, '^');
-                break;
-            default:
-                buf_putc(&buf, *p);
-                break;
-        }
+    if (userid) {
+	char *inbox = mboxname_user_mbox(userid, NULL);
+	buf_appendcstr(&buf, inbox);
+	free(inbox);
     }
 
+    if (config_skip_userlock && !strcmp(config_skip_userlock, userid)) {
+        // add a trailing character to avoid clashing with wrapping lock
+        buf_putc(&buf, '~');
+    }
+
+
     return buf_cstring(&buf);
 }
 
@@ -722,8 +720,26 @@ EXPORTED struct mboxlock *user_namespacelock_full(const char *userid, int lockty
     return namelock;
 }
 
+EXPORTED int user_run_with_lock(const char *userid, int (*cb)(void *), void *rock)
+{
+    struct mboxlock *userlock = user_namespacelock(userid);
+    int r = cb(rock);
+    mboxname_release(&userlock);
+    return r;
+}
+
 EXPORTED int user_isnamespacelocked(const char *userid)
 {
     const char *name = _namelock_name_from_userid(userid);
     return mboxname_islocked(name);
 }
+
+EXPORTED int user_isreplicaonly(const char *userid)
+{
+    int file_exists = 0;
+    char *path = strconcat(config_dir, "/replicaonly/", userid, (char *)NULL);
+    struct stat sbuf;
+    file_exists = !stat(path, &sbuf);
+    free(path);
+    return file_exists;
+}
