@@ -89,6 +89,9 @@ EXPORTED unsigned config_maxquoted;
 EXPORTED unsigned config_maxliteral;
 EXPORTED int config_qosmarking;
 EXPORTED int config_debug;
+EXPORTED toggle_debug_cb config_toggle_debug_cb = NULL;
+EXPORTED int config_debug_slowio = 0;
+EXPORTED int config_fatals_abort = 0;
 
 static int config_loaded;
 
@@ -171,7 +174,7 @@ EXPORTED enum enum_value config_getenum(enum imapopt opt)
     return imapopts[opt].val.e;
 }
 
-EXPORTED unsigned long config_getbitfield(enum imapopt opt)
+EXPORTED uint64_t config_getbitfield(enum imapopt opt)
 {
     assert(config_loaded);
     assert(opt > IMAPOPT_ZERO && opt < IMAPOPT_LAST);
@@ -554,21 +557,6 @@ EXPORTED const char *config_archivepartitiondir(const char *partition)
     return dir;
 }
 
-EXPORTED const char *config_backupstagingpath(void)
-{
-    static const char *staging_path = NULL;
-
-    if (staging_path) return staging_path;
-
-    staging_path = config_getstring(IMAPOPT_BACKUP_STAGING_PATH);
-
-    if (!staging_path)
-        staging_path = strconcat(config_getstring(IMAPOPT_TEMP_PATH),
-                                 "/backup", NULL);
-
-    return staging_path;
-}
-
 static void config_ispartition(const char *key,
                                const char *val __attribute__((unused)),
                                void *rock)
@@ -659,6 +647,9 @@ EXPORTED void config_reset(void)
     config_maxword = 0;
     config_qosmarking = 0;
     config_debug = 0;
+    config_toggle_debug_cb = NULL;
+    config_debug_slowio = 0;
+    config_fatals_abort = 0;
 
     /* reset all the options */
     for (opt = IMAPOPT_ZERO; opt < IMAPOPT_LAST; opt++) {
@@ -856,7 +847,6 @@ EXPORTED void config_read(const char *alt_config, const int config_need_data)
     /* create an array of calendar-user-address-set domains */
     cua_domains = config_getstring(IMAPOPT_CALENDAR_USER_ADDRESS_SET);
     if (!cua_domains) cua_domains = config_defdomain;
-    if (!cua_domains) cua_domains = config_servername;
 
     tok_init(&tok, cua_domains, " \t", TOK_TRIMLEFT|TOK_TRIMRIGHT);
     while ((domain = tok_next(&tok)))
@@ -885,6 +875,13 @@ EXPORTED void config_read(const char *alt_config, const int config_need_data)
 
     /* allow debug logging */
     config_debug = config_getswitch(IMAPOPT_DEBUG);
+    if (config_toggle_debug_cb) config_toggle_debug_cb();
+
+    /* do we want artificially-slow I/O ops */
+    config_debug_slowio = config_getswitch(IMAPOPT_DEBUG_SLOWIO);
+
+    /* do we want to abort() on fatal errors */
+    config_fatals_abort = config_getswitch(IMAPOPT_FATALS_ABORT);
 }
 
 #define GROWSIZE 4096
@@ -943,16 +940,20 @@ static void config_read_file(const char *filename)
         infile = fopen(filename, "r");
 
     if (!infile) {
-        snprintf(buf, bufsize, "can't open configuration file %s: %s",
+        snprintf(errbuf, sizeof(errbuf),
+                 "can't open configuration file %s: %s",
                  filename, strerror(errno));
-        fatal(buf, EX_CONFIG);
+        free(buf);
+        fatal(errbuf, EX_CONFIG);
     }
 
     /* check to see if we've already read this file */
     if (hash_lookup(filename, &includehash)) {
-        snprintf(buf, bufsize, "configuration file %s included twice",
+        snprintf(errbuf, sizeof(errbuf),
+                 "configuration file %s included twice",
                  filename);
-        fatal(buf, EX_CONFIG);
+        free(buf);
+        fatal(errbuf, EX_CONFIG);
     }
     else {
         hash_insert(filename, (void*) 0xDEADBEEF, &includehash);
@@ -999,8 +1000,9 @@ static void config_read_file(const char *filename)
         }
         if (*p != ':') {
             snprintf(errbuf, sizeof(errbuf),
-                    "invalid option name on line %d of configuration file %s",
-                    lineno, filename);
+                     "invalid option name on line %d of configuration file %s",
+                     lineno, filename);
+            free(buf);
             fatal(errbuf, EX_CONFIG);
         }
         *p++ = '\0';
@@ -1015,8 +1017,9 @@ static void config_read_file(const char *filename)
 
         if (!*p) {
             snprintf(errbuf, sizeof(errbuf),
-                    "empty option value on line %d of configuration file",
-                    lineno);
+                     "empty option value on line %d of configuration file",
+                     lineno);
+            free(buf);
             fatal(errbuf, EX_CONFIG);
         }
 
@@ -1032,6 +1035,7 @@ static void config_read_file(const char *filename)
                 snprintf(errbuf, sizeof(errbuf),
                          "invalid directive on line %d of configuration file %s",
                          lineno, filename);
+                free(buf);
                 fatal(errbuf, EX_CONFIG);
             }
         }
@@ -1081,9 +1085,11 @@ static void config_read_file(const char *filename)
                     (imapopts[opt].seen == 2 && service_specific)
                 ) {
 
-                sprintf(errbuf,
-                        "option '%s' was specified twice in config file (second occurrence on line %d)",
-                        fullkey, lineno);
+                snprintf(errbuf, sizeof(errbuf),
+                         "option '%s' was specified twice in config file"
+                         " (second occurrence on line %d)",
+                         fullkey, lineno);
+                free(buf);
                 fatal(errbuf, EX_CONFIG);
 
             } else if (imapopts[opt].seen == 2 && !service_specific) {
@@ -1130,8 +1136,10 @@ static void config_read_file(const char *filename)
                 val = strtol(p, &ptr, 0);
                 if (!ptr || *ptr != '\0') {
                     /* error during conversion */
-                    sprintf(errbuf, "non-integer value for %s in line %d",
-                            imapopts[opt].optname, lineno);
+                    snprintf(errbuf, sizeof(errbuf),
+                             "non-integer value for %s in line %d",
+                             imapopts[opt].optname, lineno);
+                    free(buf);
                     fatal(errbuf, EX_CONFIG);
                 }
 
@@ -1143,8 +1151,10 @@ static void config_read_file(const char *filename)
                 int b = config_parse_switch(p);
                 if (b < 0) {
                     /* error during conversion */
-                    sprintf(errbuf, "non-switch value for %s in line %d",
-                            imapopts[opt].optname, lineno);
+                    snprintf(errbuf, sizeof(errbuf),
+                             "non-switch value for %s in line %d",
+                             imapopts[opt].optname, lineno);
+                    free(buf);
                     fatal(errbuf, EX_CONFIG);
                 }
                 imapopts[opt].val.b = b;
@@ -1163,13 +1173,19 @@ static void config_read_file(const char *filename)
                    as one value unless told otherwise */
 
                 if (imapopts[opt].t == OPT_ENUM) {
-                    /* normalize on/off values */
+                    /* normalize on/off values
+                     * we don't write to p in this section of the parser, so
+                     * this is safe, but if that ever changes it'll crash!
+                     */
                     if (!strcmp(p, "1") || !strcmp(p, "yes") ||
-                        !strcmp(p, "t") || !strcmp(p, "true")) {
-                        p = "on";
-                    } else if (!strcmp(p, "0") || !strcmp(p, "no") ||
-                               !strcmp(p, "f") || !strcmp(p, "false")) {
-                        p = "off";
+                        !strcmp(p, "t") || !strcmp(p, "true"))
+                    {
+                        p = (char *) "on";
+                    }
+                    else if (!strcmp(p, "0") || !strcmp(p, "no") ||
+                             !strcmp(p, "f") || !strcmp(p, "false"))
+                    {
+                        p = (char *) "off";
                     }
                 } else if (imapopts[opt].t == OPT_BITFIELD) {
                     /* split the string into separate values */
@@ -1187,8 +1203,10 @@ static void config_read_file(const char *filename)
 
                     if (!e->name) {
                         /* error during conversion */
-                        sprintf(errbuf, "invalid value '%s' for %s in line %d",
-                                p, imapopts[opt].optname, lineno);
+                        snprintf(errbuf, sizeof(errbuf),
+                                 "invalid value '%s' for %s in line %d",
+                                 p, imapopts[opt].optname, lineno);
+                        free(buf);
                         fatal(errbuf, EX_CONFIG);
                     }
                     else if (imapopts[opt].t == OPT_STRINGLIST)
@@ -1224,6 +1242,7 @@ static void config_read_file(const char *filename)
                     snprintf(errbuf, sizeof(errbuf),
                              "unparsable duration '%s' for %s in line %d",
                              p, imapopts[opt].optname, lineno);
+                    free(buf);
                     fatal(errbuf, EX_CONFIG);
                 }
 
@@ -1241,6 +1260,7 @@ static void config_read_file(const char *filename)
                     snprintf(errbuf, sizeof(errbuf),
                              "unparsable byte size '%s' for %s in line %d",
                              p, imapopts[opt].optname, lineno);
+                    free(buf);
                     fatal(errbuf, EX_CONFIG);
                 }
 
@@ -1263,10 +1283,12 @@ static void config_read_file(const char *filename)
       sharing this config file and whose names we cannot predict
 
             if (strncasecmp(key,"sasl_",5)
-            && strncasecmp(key,"partition-",10)) {
-                sprintf(errbuf,
-                        "option '%s' is unknown on line %d of config file",
-                        fullkey, lineno);
+                && strncasecmp(key,"partition-",10))
+            {
+                snprintf(errbuf, sizeof(errbuf),
+                         "option '%s' is unknown on line %d of config file",
+                         fullkey, lineno);
+                free(buf);
                 fatal(errbuf, EX_CONFIG);
             }
 */
@@ -1279,3 +1301,9 @@ static void config_read_file(const char *filename)
     fclose(infile);
     free(buf);
 }
+
+EXPORTED void config_toggle_debug(void)
+{
+    config_debug = !config_debug;
+    if (config_toggle_debug_cb) config_toggle_debug_cb();
+}
