@@ -188,7 +188,7 @@ repeat:
     r = _parse_param_key(state, &haseq);
     if (r) return r;
 
-    if (state->multiparam && strarray_find_case(state->multiparam, state->param->name, 0) >= 0)
+    if (state->multiparam && strarray_contains_case(state->multiparam, state->param->name))
         multiparam = 1;
 
     /* now get the value */
@@ -423,9 +423,9 @@ out:
 
 static int _parse_entry_value(struct vparse_state *state)
 {
-    if (state->multivalsemi && strarray_find_case(state->multivalsemi, state->entry->name, 0) >= 0)
+    if (state->multivalsemi && strarray_contains_case(state->multivalsemi, state->entry->name))
         return _parse_entry_multivalue(state, ';');
-    if (state->multivalcomma && strarray_find_case(state->multivalcomma, state->entry->name, 0) >= 0)
+    if (state->multivalcomma && strarray_contains_case(state->multivalcomma, state->entry->name))
         return _parse_entry_multivalue(state, ',');
 
     NOTESTART();
@@ -684,6 +684,11 @@ EXPORTED void vparse_free_entry(struct vparse_entry *entry)
     _free_entry(entry);
 }
 
+EXPORTED void vparse_free_param(struct vparse_param *param)
+{
+    _free_param(param);
+}
+
 EXPORTED void vparse_fillpos(struct vparse_state *state, struct vparse_errorpos *pos)
 {
     int l = 1;
@@ -1263,7 +1268,7 @@ EXPORTED int vparse_restriction_check(struct vparse_card *card)
                 }
 
                 /* Like-properties having the same ALTID only get counted once */
-                if (strarray_find(&altids[i], altid, 0) != -1) continue;
+                if (strarray_contains(&altids[i], altid)) continue;
 
                 strarray_append(&altids[i], altid);
                 counts[i]++;
