@@ -81,7 +81,7 @@ EXPORTED sieve_interp_t *sieve_interp_alloc(void *interp_context)
 EXPORTED const strarray_t *sieve_listextensions(sieve_interp_t *i)
 {
     if (i->extensions == NULL) {
-        unsigned long config_sieve_extensions =
+        uint64_t config_sieve_extensions =
             config_getbitfield(IMAPOPT_SIEVE_EXTENSIONS);
         struct buf buf = BUF_INITIALIZER;
         int ext_pos;
@@ -100,6 +100,7 @@ EXPORTED const strarray_t *sieve_listextensions(sieve_interp_t *i)
 
         /* add comparators */
         buf_appendcstr(&buf, " comparator-i;ascii-numeric");
+        buf_appendcstr(&buf, " comparator-i;unicode-casemap");
 
         /* add actions */
         if (i->fileinto &&
@@ -136,9 +137,9 @@ EXPORTED const strarray_t *sieve_listextensions(sieve_interp_t *i)
         if (i->snooze &&
             (config_sieve_extensions & IMAP_ENUM_SIEVE_EXTENSIONS_SNOOZE))
             buf_appendcstr(&buf, " vnd.cyrus.snooze");
-        if (i->imip &&
-            (config_sieve_extensions & IMAP_ENUM_SIEVE_EXTENSIONS_VND_CYRUS_IMIP))
-            buf_appendcstr(&buf, " vnd.cyrus.imip");
+        if (i->processcal &&
+            (config_sieve_extensions & IMAP_ENUM_SIEVE_EXTENSIONS_PROCESSCALENDAR))
+            buf_appendcstr(&buf, " processcalendar");
         if (config_sieve_extensions & IMAP_ENUM_SIEVE_EXTENSIONS_VND_CYRUS_IMPLICIT_KEEP_TARGET)
             buf_appendcstr(&buf, " vnd.cyrus.implicit_keep_target");
 
@@ -256,9 +257,9 @@ EXPORTED void sieve_register_keep(sieve_interp_t *interp, sieve_callback *f)
     interp->keep = f;
 }
 
-EXPORTED void sieve_register_imip(sieve_interp_t *interp, sieve_callback *f)
+EXPORTED void sieve_register_processcal(sieve_interp_t *interp, sieve_callback *f)
 {
-    interp->imip = f;
+    interp->processcal = f;
 }
 
 EXPORTED void sieve_register_notify(sieve_interp_t *interp,
@@ -504,7 +505,7 @@ static const struct sieve_capa_t {
 
     /* MIME Part Handling - RFC 5703 */
     { "enclose",      SIEVE_CAPA_ENCLOSE },
-    { "extracttest",  SIEVE_CAPA_EXTRACT },
+    { "extracttext",  SIEVE_CAPA_EXTRACT },
     { "foreverypart", SIEVE_CAPA_FOREVERYPART },
     { "mime",         SIEVE_CAPA_MIME },
     { "replace",      SIEVE_CAPA_REPLACE },
@@ -539,7 +540,7 @@ static const struct sieve_capa_t {
     /* Fcc - RFC 8580 */
     { "fcc", SIEVE_CAPA_FCC },
 
-    /* Mailboxid - draft-ietf-extra-sieve-mailboxid */
+    /* Mailboxid - RFC 9042 */
     { "mailboxid", SIEVE_CAPA_MAILBOXID },
 
     /* Log - vnd.cyrus.log */
@@ -555,12 +556,16 @@ static const struct sieve_capa_t {
     { "vnd.cyrus.snooze", SIEVE_CAPA_SNOOZE },        // legacy capability
     { "x-cyrus-snooze",   SIEVE_CAPA_SNOOZE },        // legacy capability
 
-    /* iMIP - vnd.cyrus.imip */
-    { "vnd.cyrus.imip", SIEVE_CAPA_IMIP },
+    /* ProcessCalendar - RFC 9671 */
+    { "processcalendar", SIEVE_CAPA_PROCESSCAL },
+    { "vnd.cyrus.imip",  SIEVE_CAPA_PROCESSCAL },     // legacy capability
 
     /* vnd.cyrus.implicit_keep_target */
     { "vnd.cyrus.implicit_keep_target", SIEVE_CAPA_IKEEP_TARGET },
 
+    /* i;unicode-casemap - RFC 5051 */
+    { "comparator-i;unicode-casemap", SIEVE_CAPA_COMP_UCASEMAP },
+
     { NULL, 0 }
 };
     
@@ -589,12 +594,13 @@ unsigned long long lookup_capability(const char *str)
 
 unsigned long long extension_isactive(sieve_interp_t *interp, const char *str)
 {
-    unsigned long config_ext = config_getbitfield(IMAPOPT_SIEVE_EXTENSIONS);
+    uint64_t config_ext = config_getbitfield(IMAPOPT_SIEVE_EXTENSIONS);
     unsigned long long capa = lookup_capability(str);
 
     switch (capa) {
     case SIEVE_CAPA_BASE:
     case SIEVE_CAPA_COMP_NUMERIC:
+    case SIEVE_CAPA_COMP_UCASEMAP:
     case SIEVE_CAPA_ENCODED_CHAR:
         /* always enabled */
         break;
@@ -763,9 +769,9 @@ unsigned long long extension_isactive(sieve_interp_t *interp, const char *str)
               (config_ext & IMAP_ENUM_SIEVE_EXTENSIONS_SNOOZE))) capa = 0;
         break;
 
-    case SIEVE_CAPA_IMIP:
-        if (!(interp->imip &&
-              (config_ext & IMAP_ENUM_SIEVE_EXTENSIONS_VND_CYRUS_IMIP))) capa = 0;
+    case SIEVE_CAPA_PROCESSCAL:
+        if (!(interp->processcal &&
+              (config_ext & IMAP_ENUM_SIEVE_EXTENSIONS_PROCESSCALENDAR))) capa = 0;
         break;
 
     case SIEVE_CAPA_IKEEP_TARGET:
