@@ -60,9 +60,11 @@
 #include <unistd.h>
 #include <netinet/in.h>
 #include <inttypes.h>
+#include <sysexits.h>
 
 #include <string.h>
 
+#include "libconfig.h"
 #include "map.h"
 #include "times.h"
 
@@ -74,7 +76,9 @@ EXPORTED void fatal(const char *s, int code)
 {
     fprintf(stderr, "Fatal error: %s (%d)\r\n", s, code);
 
-    exit(1);
+    if (code != EX_PROTOCOL && config_fatals_abort) abort();
+
+    exit(code);
 }
 
 static int load(int fd, bytecode_input_t ** d)
@@ -104,13 +108,15 @@ int main(int argc, char * argv[])
     bytecode_input_t *bc = NULL;
     int script_fd;
     int opt, usage_error = 0, gen_script = 0;
+    char *alt_config = NULL;
 
     unsigned long len;
 
     /* keep this in alphabetical order */
-    static const char short_options[] = "s";
+    static const char short_options[] = "C:s";
 
     static const struct option long_options[] = {
+        /* n.b. no long option for -C */
         { "as-sieve", no_argument, NULL, 's' },
 
         { 0, 0, 0, 0 },
@@ -120,6 +126,9 @@ int main(int argc, char * argv[])
                                     short_options, long_options, NULL)))
     {
         switch (opt) {
+        case 'C': /* alt config file */
+            alt_config = optarg;
+            break;
         case 's':
             gen_script = 1;
             break;
@@ -143,6 +152,9 @@ int main(int argc, char * argv[])
         exit(1);
     }
 
+    /* Load configuration file. */
+    config_read(alt_config, 0);
+
     len=load(script_fd,&bc);
     close(script_fd);
 
@@ -237,9 +249,10 @@ static void print_comparator(comp_t *comp)
     }
 
     switch (comp->collation) {
-    case B_OCTET:        printf(" (octet)");         break;
-    case B_ASCIICASEMAP: printf(" (ascii-casemap)"); break;
-    case B_ASCIINUMERIC: printf(" (ascii-numeric)"); break;
+    case B_OCTET:          printf(" (octet)");         break;
+    case B_ASCIICASEMAP:   printf(" (ascii-casemap)"); break;
+    case B_ASCIINUMERIC:   printf(" (ascii-numeric)"); break;
+    case B_UNICODECASEMAP: printf(" (unicode-casemap)"); break;
     }
 
     printf(" ]");
@@ -853,11 +866,11 @@ static void dump2(bytecode_input_t *d, int bc_len)
         case B_PROCESSIMIP:
             printf("PROCESSIMIP INVITESONLY(%d)"
                    " UPDATESONLY(%d) DELETECANCELED(%d)",
-                   !!cmd.u.imip.invites_only,
-                   !!cmd.u.imip.updates_only, !!cmd.u.imip.delete_canceled);
-            print_string(" CALENDARID", cmd.u.imip.calendarid);
-            print_string(" OUTCOME", cmd.u.imip.outcome_var);
-            print_string(" ERRSTR", cmd.u.imip.errstr_var);
+                   !!cmd.u.cal.invites_only,
+                   !!cmd.u.cal.updates_only, !!cmd.u.cal.delete_cancelled);
+            print_string(" CALENDARID", cmd.u.cal.calendarid);
+            print_string(" OUTCOME", cmd.u.cal.outcome_var);
+            print_string(" ERRSTR", cmd.u.cal.reason_var);
             break;
 
 
@@ -869,6 +882,19 @@ static void dump2(bytecode_input_t *d, int bc_len)
             break;
 
 
+        case B_PROCESSCAL:
+            printf("PROCESSCALENDAR ALLOWPUBLIC(%d) INVITESONLY(%d)"
+                   " UPDATESONLY(%d) DELETECANCELLED(%d)",
+                   !!cmd.u.cal.allow_public, !!cmd.u.cal.invites_only,
+                   !!cmd.u.cal.updates_only, !!cmd.u.cal.delete_cancelled);
+            print_stringlist(" ADDRESSES", cmd.u.cal.addresses);
+            print_string(" ORGANIZERS", cmd.u.cal.organizers);
+            print_string(" CALENDARID", cmd.u.cal.calendarid);
+            print_string(" OUTCOME", cmd.u.cal.outcome_var);
+            print_string(" REASON", cmd.u.cal.reason_var);
+            break;
+
+
         default:
             printf("%d (NOT AN OP)\n", cmd.type);
             exit(1);
@@ -1054,6 +1080,10 @@ static void generate_comparator(const comp_t *c,
         buf_printf(buf, " :comparator \"i;ascii-numeric\"");
         *requires |= SIEVE_CAPA_COMP_NUMERIC;
         break;
+    case B_UNICODECASEMAP:
+        buf_printf(buf, " :comparator \"i;unicode-casemap\"");
+        *requires |= SIEVE_CAPA_COMP_UCASEMAP;
+        break;
     }
 }
 
@@ -1675,14 +1705,14 @@ static int generate_block(bytecode_input_t *bc, int pos, int end,
             break;
 
         case B_PROCESSIMIP:
-            *requires |= SIEVE_CAPA_IMIP;
+            *requires |= SIEVE_CAPA_PROCESSCAL;
             generate_token("processimip", 0, buf);
-            generate_switch(":invitesonly", cmd.u.imip.invites_only, buf);
-            generate_switch(":updatesonly", cmd.u.imip.updates_only, buf);
-            generate_switch(":deletecanceled", cmd.u.imip.delete_canceled, buf);
-            generate_string(":calendarid", cmd.u.imip.calendarid, buf);
-            generate_string(":outcome", cmd.u.imip.outcome_var, buf);
-            generate_string(":errstr", cmd.u.imip.errstr_var, buf);
+            generate_switch(":invitesonly", cmd.u.cal.invites_only, buf);
+            generate_switch(":updatesonly", cmd.u.cal.updates_only, buf);
+            generate_switch(":deletecanceled", cmd.u.cal.delete_cancelled, buf);
+            generate_string(":calendarid", cmd.u.cal.calendarid, buf);
+            generate_string(":outcome", cmd.u.cal.outcome_var, buf);
+            generate_string(":errstr", cmd.u.cal.reason_var, buf);
             break;
 
         case B_IKEEP_TARGET:
@@ -1695,6 +1725,20 @@ static int generate_block(bytecode_input_t *bc, int pos, int end,
             generate_string(NULL, cmd.u.ikt.folder, buf);
             break;
 
+        case B_PROCESSCAL:
+            *requires |= SIEVE_CAPA_PROCESSCAL;
+            generate_token("processcalendar", 0, buf);
+            generate_switch(":allowpublic", cmd.u.cal.allow_public, buf);
+            generate_switch(":invitesonly", cmd.u.cal.invites_only, buf);
+            generate_switch(":updatesonly", cmd.u.cal.updates_only, buf);
+            generate_switch(":deletecancelled", cmd.u.cal.delete_cancelled, buf);
+            generate_stringlist(":addresses", cmd.u.cal.addresses, buf);
+            generate_string(":organizers", cmd.u.cal.organizers, buf);
+            generate_string(":calendarid", cmd.u.cal.calendarid, buf);
+            generate_string(":outcome", cmd.u.cal.outcome_var, buf);
+            generate_string(":reason", cmd.u.cal.reason_var, buf);
+            break;
+
         default:
             fprintf(stderr, "%d (NOT AN OP)\n", cmd.type);
             exit(1);
@@ -1726,7 +1770,7 @@ static void generate_script(bytecode_input_t *d, int bc_len)
 
     if (requires) {
         unsigned long long capa;
-        char *sep = "";
+        const char *sep = "";
         int n;
 
         printf("require [");
